@@ -1,13 +1,13 @@
 /*
- * This file Copyright (C) 2007-2010 Mnemosyne LLC
+ * This file Copyright (C) Mnemosyne LLC
  *
- * This file is licensed by the GPL version 2.  Works owned by the
+ * This file is licensed by the GPL version 2. Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
  * so that the bulk of its code can remain under the MIT license.
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: actions.c 11263 2010-09-24 15:22:53Z charles $
+ * $Id: actions.c 12639 2011-08-07 18:41:13Z jordan $
  */
 
 #include <string.h>
@@ -31,35 +31,26 @@
 
 #define UNUSED G_GNUC_UNUSED
 
-static TrCore *         myCore = NULL;
-
+static TrCore * myCore = NULL;
 static GtkActionGroup * myGroup = NULL;
 
 static void
-action_cb( GtkAction * a,
-           gpointer    user_data )
+action_cb( GtkAction * a, gpointer user_data )
 {
-    doAction ( gtk_action_get_name( a ), user_data );
+    gtr_actions_handler( gtk_action_get_name( a ), user_data );
 }
 
-#if !GTK_CHECK_VERSION( 2, 8, 0 )
- #define GTK_STOCK_INFO GTK_STOCK_PROPERTIES
-#endif
-
-#if !GTK_CHECK_VERSION( 2, 10, 0 )
- #define GTK_STOCK_SELECT_ALL NULL
-#endif
-
 static GtkRadioActionEntry sort_radio_entries[] =
 {
     { "sort-by-activity",  NULL, N_( "Sort by _Activity" ),  NULL, NULL, 0 },
     { "sort-by-name",      NULL, N_( "Sort by _Name" ),      NULL, NULL, 1 },
     { "sort-by-progress",  NULL, N_( "Sort by _Progress" ),  NULL, NULL, 2 },
-    { "sort-by-ratio",     NULL, N_( "Sort by Rati_o" ),     NULL, NULL, 3 },
-    { "sort-by-state",     NULL, N_( "Sort by Stat_e" ),     NULL, NULL, 4 },
-    { "sort-by-age",       NULL, N_( "Sort by A_ge" ),       NULL, NULL, 5 },
-    { "sort-by-time-left", NULL, N_( "Sort by Time _Left" ), NULL, NULL, 6 },
-    { "sort-by-size",      NULL, N_( "Sort by Si_ze" ),      NULL, NULL, 7 }
+    { "sort-by-queue",     NULL, N_( "Sort by _Queue" ),     NULL, NULL, 3 },
+    { "sort-by-ratio",     NULL, N_( "Sort by Rati_o" ),     NULL, NULL, 4 },
+    { "sort-by-state",     NULL, N_( "Sort by Stat_e" ),     NULL, NULL, 5 },
+    { "sort-by-age",       NULL, N_( "Sort by A_ge" ),       NULL, NULL, 6 },
+    { "sort-by-time-left", NULL, N_( "Sort by Time _Left" ), NULL, NULL, 7 },
+    { "sort-by-size",      NULL, N_( "Sort by Si_ze" ),      NULL, NULL, 8 }
 };
 
 static void
@@ -71,7 +62,7 @@ sort_changed_cb( GtkAction            * action UNUSED,
     const int    i = gtk_radio_action_get_current_value( current );
     const char * val = sort_radio_entries[i].name;
 
-    tr_core_set_pref( myCore, key, val );
+    gtr_core_set_pref( myCore, key, val );
 }
 
 static GtkToggleActionEntry show_toggle_entries[] =
@@ -87,12 +78,12 @@ toggle_pref_cb( GtkToggleAction *  action,
     const char *   key = gtk_action_get_name( GTK_ACTION( action ) );
     const gboolean val = gtk_toggle_action_get_active( action );
 
-    tr_core_set_pref_bool( myCore, key, val );
+    gtr_core_set_pref_bool( myCore, key, val );
 }
 
 static GtkToggleActionEntry  pref_toggle_entries[] =
 {
-    { "alt-speed-enabled", NULL, N_( "Enable Temporary Speed _Limits" ), NULL, NULL, G_CALLBACK( toggle_pref_cb ), FALSE },
+    { "alt-speed-enabled", NULL, N_( "Enable Alternative Speed _Limits" ), NULL, NULL, G_CALLBACK( toggle_pref_cb ), FALSE },
     { "compact-view",      NULL, N_( "_Compact View" ), "<alt>C", NULL, G_CALLBACK( toggle_pref_cb ), FALSE },
     { "sort-reversed",     NULL, N_( "Re_verse Sort Order" ), NULL, NULL, G_CALLBACK( toggle_pref_cb ), FALSE },
     { "show-filterbar",    NULL, N_( "_Filterbar" ), NULL, NULL, G_CALLBACK( toggle_pref_cb ), FALSE },
@@ -106,17 +97,19 @@ static GtkActionEntry entries[] =
     { "torrent-menu", NULL, N_( "_Torrent" ), NULL, NULL, NULL  },
     { "view-menu", NULL, N_( "_View" ), NULL, NULL, NULL  },
     { "sort-menu", NULL, N_( "_Sort Torrents By" ), NULL, NULL, NULL },
+    { "queue-menu", NULL, N_( "_Queue" ), NULL, NULL, NULL },
     { "edit-menu", NULL, N_( "_Edit" ), NULL, NULL, NULL },
     { "help-menu", NULL, N_( "_Help" ), NULL, NULL, NULL },
-    { "copy-magnet-link-to-clipboard",  GTK_STOCK_COPY, N_("Copy _Magnet Link to Clipboard" ), "<control>M", NULL,  G_CALLBACK( action_cb ) },
-    { "add-torrent-from-url",  GTK_STOCK_ADD, N_("Add _URL..." ), NULL, N_( "Add URL..." ),  G_CALLBACK( action_cb ) },
-    { "add-torrent-toolbar",  GTK_STOCK_ADD, NULL, NULL, N_( "Add a torrent" ),  G_CALLBACK( action_cb ) },
-    { "add-torrent-menu", GTK_STOCK_ADD, N_( "_Add File..." ), "<control>D", N_( "Add a torrent" ), G_CALLBACK( action_cb ) },
-    { "start-torrent", GTK_STOCK_MEDIA_PLAY, N_( "_Start" ), "<control>S", N_( "Start torrent" ), G_CALLBACK( action_cb ) },
+    { "copy-magnet-link-to-clipboard",  GTK_STOCK_COPY, N_("Copy _Magnet Link to Clipboard" ), "", NULL,  G_CALLBACK( action_cb ) },
+    { "open-torrent-from-url",  GTK_STOCK_OPEN, N_("Open _URL..." ), "<control>U", N_( "Open URL..." ),  G_CALLBACK( action_cb ) },
+    { "open-torrent-toolbar",  GTK_STOCK_OPEN, NULL, NULL, N_( "Open a torrent" ),  G_CALLBACK( action_cb ) },
+    { "open-torrent-menu", GTK_STOCK_OPEN, NULL, NULL, N_( "Open a torrent" ), G_CALLBACK( action_cb ) },
+    { "torrent-start", GTK_STOCK_MEDIA_PLAY, N_( "_Start" ), "<control>S", N_( "Start torrent" ), G_CALLBACK( action_cb ) },
+    { "torrent-start-now", GTK_STOCK_MEDIA_PLAY, N_( "Start _Now" ), "<shift><control>S", N_( "Start torrent now" ), G_CALLBACK( action_cb ) },
     { "show-stats", NULL, N_( "_Statistics" ), NULL, NULL, G_CALLBACK( action_cb ) },
     { "donate", NULL, N_( "_Donate" ), NULL, NULL, G_CALLBACK( action_cb ) },
-    { "verify-torrent", NULL, N_( "_Verify Local Data" ), "<control>V", NULL, G_CALLBACK( action_cb ) },
-    { "pause-torrent", GTK_STOCK_MEDIA_PAUSE, N_( "_Pause" ), "<control>P", N_( "Pause torrent" ), G_CALLBACK( action_cb ) },
+    { "torrent-verify", NULL, N_( "_Verify Local Data" ), "<control>V", NULL, G_CALLBACK( action_cb ) },
+    { "torrent-stop", GTK_STOCK_MEDIA_PAUSE, N_( "_Pause" ), "<control>P", N_( "Pause torrent" ), G_CALLBACK( action_cb ) },
     { "pause-all-torrents", GTK_STOCK_MEDIA_PAUSE, N_( "_Pause All" ), NULL, N_( "Pause all torrents" ), G_CALLBACK( action_cb ) },
     { "start-all-torrents", GTK_STOCK_MEDIA_PLAY, N_( "_Start All" ), NULL, N_( "Start all torrents" ), G_CALLBACK( action_cb ) },
     { "relocate-torrent", NULL, N_("Set _Location..." ), NULL, NULL, G_CALLBACK( action_cb ) },
@@ -128,10 +121,15 @@ static GtkActionEntry entries[] =
     { "deselect-all", NULL, N_( "Dese_lect All" ), "<shift><control>A", NULL, G_CALLBACK( action_cb ) },
     { "edit-preferences", GTK_STOCK_PREFERENCES, NULL, NULL, NULL, G_CALLBACK( action_cb ) },
     { "show-torrent-properties", GTK_STOCK_PROPERTIES, NULL, "<alt>Return", N_( "Torrent properties" ), G_CALLBACK( action_cb ) },
-    { "open-torrent-folder",  GTK_STOCK_OPEN, N_( "_Open Folder" ), NULL, NULL, G_CALLBACK( action_cb ) },
+    { "open-torrent-folder",  GTK_STOCK_OPEN, N_( "Open Fold_er" ), "<control>E", NULL, G_CALLBACK( action_cb ) },
     { "show-about-dialog", GTK_STOCK_ABOUT, NULL, NULL, NULL, G_CALLBACK( action_cb ) },
     { "help", GTK_STOCK_HELP, N_( "_Contents" ), "F1", NULL, G_CALLBACK( action_cb ) },
-    { "update-tracker", GTK_STOCK_NETWORK, N_( "Ask Tracker for _More Peers" ), NULL, NULL, G_CALLBACK( action_cb ) },
+    { "torrent-reannounce", GTK_STOCK_NETWORK, N_( "Ask Tracker for _More Peers" ), NULL, NULL, G_CALLBACK( action_cb ) },
+    { "queue-move-top", GTK_STOCK_GOTO_TOP, N_( "Move to _Top" ), NULL, NULL, G_CALLBACK( action_cb ) },
+    { "queue-move-up", GTK_STOCK_GO_UP, N_( "Move _Up" ), NULL, NULL, G_CALLBACK( action_cb ) },
+    { "queue-move-down", GTK_STOCK_GO_DOWN, N_( "Move _Down" ), NULL, NULL, G_CALLBACK( action_cb ) },
+    { "queue-move-bottom", GTK_STOCK_GOTO_BOTTOM, N_( "Move to _Bottom" ), NULL, NULL, G_CALLBACK( action_cb ) },
+    { "present-main-window", NULL, N_( "Present Main Window" ), NULL, NULL, G_CALLBACK( action_cb ) }
 };
 
 typedef struct
@@ -193,14 +191,13 @@ register_my_icons( void )
 static GtkUIManager * myUIManager = NULL;
 
 void
-actions_set_core( TrCore * core )
+gtr_actions_set_core( TrCore * core )
 {
     myCore = core;
 }
 
 void
-actions_init( GtkUIManager * ui_manager,
-              gpointer       callback_user_data )
+gtr_actions_init( GtkUIManager * ui_manager, gpointer callback_user_data )
 {
     int              i, n;
     int              active;
@@ -216,7 +213,7 @@ actions_init( GtkUIManager * ui_manager,
     gtk_action_group_set_translation_domain( action_group, NULL );
 
 
-    match = pref_string_get( PREF_KEY_SORT_MODE );
+    match = gtr_pref_string_get( PREF_KEY_SORT_MODE );
     for( i = 0, n = G_N_ELEMENTS( sort_radio_entries ), active = -1;
          active == -1 && i < n; ++i )
         if( !strcmp( sort_radio_entries[i].name, match ) )
@@ -236,7 +233,7 @@ actions_init( GtkUIManager * ui_manager,
 
     for( i = 0, n = G_N_ELEMENTS( pref_toggle_entries ); i < n; ++i )
         pref_toggle_entries[i].is_active =
-            pref_flag_get( pref_toggle_entries[i].name );
+            gtr_pref_flag_get( pref_toggle_entries[i].name );
 
     gtk_action_group_add_toggle_actions( action_group,
                                          pref_toggle_entries,
@@ -292,7 +289,7 @@ get_action( const char* name )
 }
 
 void
-action_activate( const char * name )
+gtr_action_activate( const char * name )
 {
     GtkAction * action = get_action( name );
 
@@ -301,8 +298,7 @@ action_activate( const char * name )
 }
 
 void
-action_sensitize( const char * name,
-                  gboolean     b )
+gtr_action_set_sensitive( const char * name, gboolean b )
 {
     GtkAction * action = get_action( name );
 
@@ -311,7 +307,7 @@ action_sensitize( const char * name,
 }
 
 void
-action_set_important( const char * name, gboolean b )
+gtr_action_set_important( const char * name, gboolean b )
 {
     GtkAction * action = get_action( name );
 
@@ -320,8 +316,7 @@ action_set_important( const char * name, gboolean b )
 }
 
 void
-action_toggle( const char * name,
-               gboolean     b )
+gtr_action_set_toggled( const char * name, gboolean b )
 {
     GtkAction * action = get_action( name );
 
@@ -329,7 +324,7 @@ action_toggle( const char * name,
 }
 
 GtkWidget*
-action_get_widget( const char * path )
+gtr_action_get_widget( const char * path )
 {
     return gtk_ui_manager_get_widget( myUIManager, path );
 }
