@@ -1,17 +1,18 @@
 /*
- * This file Copyright (C) 2007-2010 Mnemosyne LLC
+ * This file Copyright (C) Mnemosyne LLC
  *
- * This file is licensed by the GPL version 2.  Works owned by the
+ * This file is licensed by the GPL version 2. Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
  * so that the bulk of its code can remain under the MIT license.
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: hig.c 9868 2010-01-04 21:00:47Z charles $
+ * $Id: hig.c 12997 2011-10-20 00:37:39Z jordan $
  */
 
 #include <gtk/gtk.h>
 #include "hig.h"
+#include "util.h" /* gtr_hbox_new */
 
 GtkWidget*
 hig_workarea_create( void )
@@ -25,8 +26,7 @@ hig_workarea_create( void )
 }
 
 void
-hig_workarea_add_section_divider( GtkWidget * t,
-                                  int *       row )
+hig_workarea_add_section_divider( GtkWidget * t, guint * row )
 {
     GtkWidget * w = gtk_alignment_new( 0.0f, 0.0f, 0.0f, 0.0f );
 
@@ -36,20 +36,16 @@ hig_workarea_add_section_divider( GtkWidget * t,
 }
 
 void
-hig_workarea_add_section_title_widget( GtkWidget * t,
-                                       int *       row,
-                                       GtkWidget * w )
+hig_workarea_add_section_title_widget( GtkWidget * t, guint * row, GtkWidget * w )
 {
     gtk_table_attach( GTK_TABLE( t ), w, 0, 2, *row, *row + 1, ~0, 0, 0, 0 );
     ++ * row;
 }
 
 void
-hig_workarea_add_section_title( GtkWidget *  t,
-                                int *        row,
-                                const char * section_title )
+hig_workarea_add_section_title( GtkWidget *  t, guint * row, const char * section_title )
 {
-    char        buf[512];
+    char buf[512];
     GtkWidget * l;
 
     g_snprintf( buf, sizeof( buf ), "<b>%s</b>", section_title );
@@ -63,7 +59,7 @@ static GtkWidget*
 rowNew( GtkWidget * w )
 {
     GtkWidget * a;
-    GtkWidget * h = gtk_hbox_new( FALSE, 0 );
+    GtkWidget * h = gtr_hbox_new( FALSE, 0 );
 
     /* spacer */
     a = gtk_alignment_new( 0.0f, 0.0f, 0.0f, 0.0f );
@@ -81,9 +77,7 @@ rowNew( GtkWidget * w )
 }
 
 void
-hig_workarea_add_wide_control( GtkWidget * t,
-                               int *       row,
-                               GtkWidget * w )
+hig_workarea_add_wide_control( GtkWidget * t, guint * row, GtkWidget * w )
 {
     GtkWidget * r = rowNew( w );
 
@@ -92,9 +86,7 @@ hig_workarea_add_wide_control( GtkWidget * t,
 }
 
 void
-hig_workarea_add_wide_tall_control( GtkWidget * t,
-                                    int *       row,
-                                    GtkWidget * w )
+hig_workarea_add_wide_tall_control( GtkWidget * t, guint * row, GtkWidget * w )
 {
     GtkWidget * r = rowNew( w );
 
@@ -107,8 +99,8 @@ hig_workarea_add_wide_tall_control( GtkWidget * t,
 }
 
 GtkWidget *
-hig_workarea_add_wide_checkbutton( GtkWidget *  t,
-                                   int *        row,
+hig_workarea_add_wide_checkbutton( GtkWidget  * t,
+                                   guint      * row,
                                    const char * mnemonic_string,
                                    gboolean     is_active )
 {
@@ -120,9 +112,7 @@ hig_workarea_add_wide_checkbutton( GtkWidget *  t,
 }
 
 void
-hig_workarea_add_label_w( GtkWidget * t,
-                          int         row,
-                          GtkWidget * l )
+hig_workarea_add_label_w( GtkWidget * t, guint row, GtkWidget * l )
 {
     GtkWidget * w = rowNew( l );
 
@@ -130,9 +120,7 @@ hig_workarea_add_label_w( GtkWidget * t,
 }
 
 GtkWidget*
-hig_workarea_add_label( GtkWidget *  t,
-                        int          row,
-                        const char * mnemonic_string )
+hig_workarea_add_label( GtkWidget * t, guint row, const char * mnemonic_string )
 {
     GtkWidget * l = gtk_label_new_with_mnemonic( mnemonic_string );
 
@@ -141,9 +129,7 @@ hig_workarea_add_label( GtkWidget *  t,
 }
 
 static void
-hig_workarea_add_tall_control( GtkWidget * t,
-                               int         row,
-                               GtkWidget * control )
+hig_workarea_add_tall_control( GtkWidget * t, guint row, GtkWidget * control )
 {
     if( GTK_IS_MISC( control ) )
         gtk_misc_set_alignment( GTK_MISC( control ), 0.0f, 0.5f );
@@ -155,10 +141,8 @@ hig_workarea_add_tall_control( GtkWidget * t,
                       0, 0 );
 }
 
-void
-hig_workarea_add_control( GtkWidget * t,
-                          int         row,
-                          GtkWidget * control )
+static void
+hig_workarea_add_control( GtkWidget * t, guint row, GtkWidget * control )
 {
     if( GTK_IS_MISC( control ) )
         gtk_misc_set_alignment( GTK_MISC( control ), 0.0f, 0.5f );
@@ -170,7 +154,7 @@ hig_workarea_add_control( GtkWidget * t,
 
 void
 hig_workarea_add_row_w( GtkWidget * t,
-                        int *       row,
+                        guint     * row,
                         GtkWidget * label,
                         GtkWidget * control,
                         GtkWidget * mnemonic )
@@ -184,11 +168,11 @@ hig_workarea_add_row_w( GtkWidget * t,
 }
 
 GtkWidget*
-hig_workarea_add_row( GtkWidget *  t,
-                      int *        row,
+hig_workarea_add_row( GtkWidget  * t,
+                      guint      * row,
                       const char * mnemonic_string,
-                      GtkWidget *  control,
-                      GtkWidget *  mnemonic )
+                      GtkWidget  * control,
+                      GtkWidget  * mnemonic )
 {
     GtkWidget * l = gtk_label_new_with_mnemonic( mnemonic_string );
 
@@ -197,15 +181,15 @@ hig_workarea_add_row( GtkWidget *  t,
 }
 
 GtkWidget*
-hig_workarea_add_tall_row( GtkWidget *  table,
-                           int *        row,
+hig_workarea_add_tall_row( GtkWidget  * table,
+                           guint      * row,
                            const char * mnemonic_string,
-                           GtkWidget *  control,
-                           GtkWidget *  mnemonic )
+                           GtkWidget  * control,
+                           GtkWidget  * mnemonic )
 {
     GtkWidget * l = gtk_label_new_with_mnemonic( mnemonic_string );
-    GtkWidget * h = gtk_hbox_new( FALSE, 0 );
-    GtkWidget * v = gtk_vbox_new( FALSE, 0 );
+    GtkWidget * h = gtr_hbox_new( FALSE, 0 );
+    GtkWidget * v = gtr_vbox_new( FALSE, 0 );
     gtk_box_pack_start( GTK_BOX( h ), l, FALSE, FALSE, 0 );
     gtk_box_pack_start( GTK_BOX( v ), h, FALSE, FALSE, GUI_PAD_SMALL );
 
@@ -221,18 +205,7 @@ hig_workarea_add_tall_row( GtkWidget *  table,
 }
 
 void
-hig_workarea_finish( GtkWidget * t,
-                     int *       row )
+hig_workarea_finish( GtkWidget * t, guint * row )
 {
     gtk_table_resize( GTK_TABLE( t ), *row, 2 );
 }
-
-void
-hig_message_dialog_set_text( GtkMessageDialog * dialog,
-                             const char *       primary,
-                             const char *       secondary )
-{
-    gtk_message_dialog_set_markup ( dialog, primary );
-    gtk_message_dialog_format_secondary_text ( dialog, "%s", secondary );
-}
-
