@@ -7,11 +7,9 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: makemeta-ui.c 11738 2011-01-21 16:32:27Z jordan $
+ * $Id: makemeta-ui.c 12997 2011-10-20 00:37:39Z jordan $
  */
 
-#include <string.h>
-
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 
@@ -19,7 +17,6 @@
 #include <libtransmission/makemeta.h>
 #include <libtransmission/utils.h> /* tr_formatter_mem_B() */
 
-#include "conf.h"
 #include "hig.h"
 #include "makemeta-ui.h"
 #include "tr-core.h"
@@ -129,7 +126,7 @@ addTorrent( MakeMetaUI * ui )
 {
     char * path;
     const tr_metainfo_builder * b = ui->builder;
-    tr_ctor * ctor = tr_ctorNew( tr_core_session( ui->core ) );
+    tr_ctor * ctor = tr_ctorNew( gtr_core_session( ui->core ) );
 
     tr_ctorSetMetainfoFromFile( ctor, ui->target );
 
@@ -137,7 +134,7 @@ addTorrent( MakeMetaUI * ui )
     tr_ctorSetDownloadDir( ctor, TR_FORCE, path );
     g_free( path );
 
-    tr_core_add_ctor( ui->core, ctor );
+    gtr_core_add_ctor( ui->core, ctor );
 }
 
 static void
@@ -180,7 +177,7 @@ makeProgressDialog( GtkWidget * parent, MakeMetaUI * ui )
     fr = gtk_frame_new( NULL );
     gtk_container_set_border_width( GTK_CONTAINER( fr ), GUI_PAD_BIG );
     gtk_frame_set_shadow_type( GTK_FRAME( fr ), GTK_SHADOW_NONE );
-    v = gtk_vbox_new( TRUE, GUI_PAD );
+    v = gtr_vbox_new( TRUE, GUI_PAD );
     gtk_container_add( GTK_CONTAINER( fr ), v );
 
     l = gtk_label_new( _( "Creating torrent..." ) );
@@ -193,7 +190,7 @@ makeProgressDialog( GtkWidget * parent, MakeMetaUI * ui )
     ui->progress_bar = w;
     gtk_box_pack_start( GTK_BOX( v ), w, FALSE, FALSE, 0 );
 
-    ui->progress_tag = gtr_timeout_add_seconds( SECONDARY_WINDOW_REFRESH_INTERVAL_SECONDS, onProgressDialogRefresh, ui );
+    ui->progress_tag = gdk_threads_add_timeout_seconds( SECONDARY_WINDOW_REFRESH_INTERVAL_SECONDS, onProgressDialogRefresh, ui );
     g_object_weak_ref( G_OBJECT( d ), onProgressDialogDestroyed, ui );
     onProgressDialogRefresh( ui );
 
@@ -298,16 +295,16 @@ updatePiecesLabel( MakeMetaUI * ui )
     {
         char buf[128];
         tr_strlsize( buf, builder->totalSize, sizeof( buf ) );
-        g_string_append_printf( gstr, gtr_ngettext( "%1$s; %2$'d File",
-                                                    "%1$s; %2$'d Files",
-                                                    builder->fileCount ),
+        g_string_append_printf( gstr, ngettext( "%1$s; %2$'d File",
+                                                "%1$s; %2$'d Files",
+                                                builder->fileCount ),
                                 buf, builder->fileCount );
         g_string_append( gstr, "; " );
 
         tr_formatter_mem_B( buf, builder->pieceSize, sizeof( buf ) );
-        g_string_append_printf( gstr, gtr_ngettext( "%1$'d Piece @ %2$s",
-                                                    "%1$'d Pieces @ %2$s",
-                                                    builder->pieceCount ),
+        g_string_append_printf( gstr, ngettext( "%1$'d Piece @ %2$s",
+                                                "%1$'d Pieces @ %2$s",
+                                                builder->pieceCount ),
                                       builder->pieceCount, buf );
     }
     g_string_append( gstr, "</i>" );
@@ -370,13 +367,7 @@ onFileToggled( GtkToggleButton * tb, gpointer data )
 static const char *
 getDefaultSavePath( void )
 {
-    const char * path;
-#if GLIB_CHECK_VERSION( 2,14,0 )
-    path = g_get_user_special_dir( G_USER_DIRECTORY_DESKTOP );
-#else
-    path = g_get_home_dir( );
-#endif
-    return path;
+    return g_get_user_special_dir( G_USER_DIRECTORY_DESKTOP );
 }
 
 static void
@@ -423,10 +414,10 @@ on_drag_data_received( GtkWidget         * widget           UNUSED,
 GtkWidget*
 gtr_torrent_creation_dialog_new( GtkWindow  * parent, TrCore * core )
 {
-    int row = 0;
     const char * str;
     GtkWidget * d, *t, *w, *l, *fr, *sw, *v;
     GSList * slist;
+    guint row = 0;
     MakeMetaUI * ui = g_new0 ( MakeMetaUI, 1 );
 
     ui->core = core;
@@ -482,7 +473,7 @@ gtr_torrent_creation_dialog_new( GtkWindow  * parent, TrCore * core )
     hig_workarea_add_section_title ( t, &row, _( "Properties" ) );
 
         str = _( "_Trackers:" );
-        v = gtk_vbox_new( FALSE, GUI_PAD_SMALL );
+        v = gtr_vbox_new( FALSE, GUI_PAD_SMALL );
         ui->announce_text_buffer = gtk_text_buffer_new( NULL );
         w = gtk_text_view_new_with_buffer( ui->announce_text_buffer );
         gtk_widget_set_size_request( w, -1, 80 );
