@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: msgwin.c 12234 2011-03-25 17:42:47Z jordan $
+ * $Id: msgwin.c 12997 2011-10-20 00:37:39Z jordan $
  */
 
 #include <errno.h>
@@ -445,7 +445,7 @@ gtr_message_log_window_new( GtkWindow * parent, TrCore * core )
     gtk_window_set_title( GTK_WINDOW( win ), _( "Message Log" ) );
     gtk_window_set_default_size( GTK_WINDOW( win ), 560, 350 );
     gtk_window_set_role( GTK_WINDOW( win ), "message-log" );
-    vbox = gtk_vbox_new( FALSE, 0 );
+    vbox = gtr_vbox_new( FALSE, 0 );
 
     /**
     ***  toolbar
@@ -453,6 +453,10 @@ gtr_message_log_window_new( GtkWindow * parent, TrCore * core )
 
     toolbar = gtk_toolbar_new( );
     gtk_toolbar_set_style( GTK_TOOLBAR( toolbar ), GTK_TOOLBAR_BOTH_HORIZ );
+#if GTK_CHECK_VERSION( 3,0,0 )
+    gtk_style_context_add_class( gtk_widget_get_style_context( toolbar ),
+                                 GTK_STYLE_CLASS_PRIMARY_TOOLBAR );
+#endif
 
     item = gtk_tool_button_new_from_stock( GTK_STOCK_SAVE_AS );
     g_object_set( G_OBJECT( item ), "is-important", TRUE, NULL );
@@ -535,7 +539,7 @@ gtr_message_log_window_new( GtkWindow * parent, TrCore * core )
     gtk_box_pack_start( GTK_BOX( vbox ), w, TRUE, TRUE, 0 );
     gtk_container_add( GTK_CONTAINER( win ), vbox );
 
-    data->refresh_tag = gtr_timeout_add_seconds( SECONDARY_WINDOW_REFRESH_INTERVAL_SECONDS, onRefresh, data );
+    data->refresh_tag = gdk_threads_add_timeout_seconds( SECONDARY_WINDOW_REFRESH_INTERVAL_SECONDS, onRefresh, data );
     g_object_weak_ref( G_OBJECT( win ), onWindowDestroyed, data );
 
     scroll_to_bottom( data );
