@@ -1,13 +1,13 @@
 /*
- * This file Copyright (C) 2007-2010 Mnemosyne LLC
+ * This file Copyright (C) Mnemosyne LLC
  *
- * This file is licensed by the GPL version 2.  Works owned by the
+ * This file is licensed by the GPL version 2. Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
  * so that the bulk of its code can remain under the MIT license.
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: tr-icon.c 11124 2010-08-05 13:24:46Z charles $
+ * $Id: tr-icon.c 12648 2011-08-08 16:02:37Z jordan $
  */
 
 #include <glib/gi18n.h>
@@ -21,34 +21,26 @@
 #include "tr-icon.h"
 #include "util.h"
 
+static GQuark
+get_core_quark( void )
+{
+    static GQuark quark = 0;
+    if( !quark ) quark = g_quark_from_static_string( "tr-core" );
+    return quark;
+}
+
 #define ICON_NAME "transmission"
 
-#ifndef STATUS_ICON_SUPPORTED
-
-gpointer
-tr_icon_new( TrCore * core )
-{
-    return NULL;
-}
-
-void
-tr_icon_refresh( gpointer vicon UNUSED )
-{
-}
-
-#else
-
 #ifdef HAVE_LIBAPPINDICATOR
 void
-tr_icon_refresh( gpointer vindicator UNUSED )
+gtr_icon_refresh( gpointer vindicator UNUSED )
 {
 }
 #else
 static void
-activated( GtkStatusIcon   * self      UNUSED,
-           gpointer          user_data UNUSED )
+activated( GtkStatusIcon * self UNUSED, gpointer user_data UNUSED )
 {
-    action_activate ( "toggle-main-window" );
+    gtr_action_activate( "toggle-main-window" );
 }
 
 static void
@@ -57,7 +49,7 @@ popup( GtkStatusIcon *       self,
        guint                 when,
        gpointer         data UNUSED )
 {
-    GtkWidget * w = action_get_widget( "/icon-popup" );
+    GtkWidget * w = gtr_action_get_widget( "/icon-popup" );
 
     gtk_menu_popup ( GTK_MENU( w ), NULL, NULL,
                      gtk_status_icon_position_menu,
@@ -65,7 +57,7 @@ popup( GtkStatusIcon *       self,
 }
 
 void
-tr_icon_refresh( gpointer vicon )
+gtr_icon_refresh( gpointer vicon )
 {
     double KBps;
     double limit;
@@ -76,7 +68,7 @@ tr_icon_refresh( gpointer vicon )
     char tip[1024];
     const char * idle = _( "Idle" );
     GtkStatusIcon * icon = GTK_STATUS_ICON( vicon );
-    tr_session * session = tr_core_session( g_object_get_data( G_OBJECT( icon ), "tr-core" ) );
+    tr_session * session = gtr_core_session( g_object_get_qdata( G_OBJECT( icon ), get_core_quark( ) ) );
 
     /* up */
     KBps = tr_sessionGetRawSpeed_KBps( session, TR_UP );
@@ -116,11 +108,7 @@ tr_icon_refresh( gpointer vicon )
      * %4$s: current download limit, if any */
     g_snprintf( tip, sizeof( tip ), _( "Transmission\nUp: %1$s %2$s\nDown: %3$s %4$s" ), up, upLimit, down, downLimit );
 
-#if GTK_CHECK_VERSION( 2,16,0 )
     gtk_status_icon_set_tooltip_text( GTK_STATUS_ICON( icon ), tip );
-#else
-    gtk_status_icon_set_tooltip( GTK_STATUS_ICON( icon ), tip );
-#endif
 }
 #endif
 
@@ -145,31 +133,24 @@ getIconName( void )
     return icon_name;
 }
 
-#ifdef HAVE_LIBAPPINDICATOR
 gpointer
-tr_icon_new( TrCore * core)
+gtr_icon_new( TrCore * core)
 {
+#ifdef HAVE_LIBAPPINDICATOR
     GtkWidget * w;
     const char * icon_name = getIconName( );
     AppIndicator * indicator = app_indicator_new( ICON_NAME, icon_name, APP_INDICATOR_CATEGORY_SYSTEM_SERVICES );
     app_indicator_set_status( indicator, APP_INDICATOR_STATUS_ACTIVE );
-    w = action_get_widget( "/icon-popup" );
+    w = gtr_action_get_widget( "/icon-popup" );
     app_indicator_set_menu( indicator, GTK_MENU ( w ) );
-    g_object_set_data( G_OBJECT( indicator ), "tr-core", core );
+    g_object_set_qdata( G_OBJECT( indicator ), get_core_quark( ), core );
     return indicator;
-}
 #else
-gpointer
-tr_icon_new( TrCore * core )
-{
     const char * icon_name = getIconName( );
     GtkStatusIcon * icon = gtk_status_icon_new_from_icon_name( icon_name );
     g_signal_connect( icon, "activate", G_CALLBACK( activated ), NULL );
     g_signal_connect( icon, "popup-menu", G_CALLBACK( popup ), NULL );
-    g_object_set_data( G_OBJECT( icon ), "tr-core", core );
+    g_object_set_qdata( G_OBJECT( icon ), get_core_quark( ), core );
     return icon;
+#endif
 }
-
-#endif
-
-#endif
