@@ -7,19 +7,18 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: tr-prefs.c 12027 2011-02-24 15:10:18Z jordan $
+ * $Id: tr-prefs.c 12997 2011-10-20 00:37:39Z jordan $
  */
 
 #include <ctype.h> /* isspace */
 #include <limits.h> /* USHRT_MAX */
-#include <stdlib.h> /* free() */
+#include <string.h> /* strcmp() */
 #include <unistd.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <libtransmission/transmission.h>
 #include <libtransmission/utils.h>
 #include <libtransmission/version.h>
-#include <libtransmission/web.h>
 #include "conf.h"
 #include "hig.h"
 #include "tr-core.h"
@@ -55,7 +54,7 @@ toggled_cb( GtkToggleButton * w,
     const char *   key = g_object_get_data( G_OBJECT( w ), PREF_KEY );
     const gboolean flag = gtk_toggle_button_get_active( w );
 
-    tr_core_set_pref_bool( TR_CORE( core ), key, flag );
+    gtr_core_set_pref_bool( TR_CORE( core ), key, flag );
 }
 
 static GtkWidget*
@@ -107,12 +106,12 @@ spun_cb_idle( gpointer spin )
         if (data->isDouble)
         {
             const double value = gtk_spin_button_get_value( GTK_SPIN_BUTTON( spin ) );
-            tr_core_set_pref_double( TR_CORE( data->core ), key, value );
+            gtr_core_set_pref_double( TR_CORE( data->core ), key, value );
         }
         else
         {
             const int value = gtk_spin_button_get_value_as_int( GTK_SPIN_BUTTON( spin ) );
-            tr_core_set_pref_int( TR_CORE( data->core ), key, value );
+            gtr_core_set_pref_int( TR_CORE( data->core ), key, value );
         }
 
         /* cleanup */
@@ -140,7 +139,7 @@ spun_cb( GtkSpinButton * w, gpointer core, gboolean isDouble )
         data->isDouble = isDouble;
         g_object_set_data_full( o, IDLE_DATA, data, spin_idle_data_free );
         g_object_ref( G_OBJECT( o ) );
-        gtr_timeout_add_seconds( 1, spun_cb_idle, w );
+        gdk_threads_add_timeout_seconds( 1, spun_cb_idle, w );
     }
     g_timer_start( data->last_change );
 }
@@ -193,7 +192,7 @@ entry_changed_cb( GtkEntry * w, gpointer   core )
     const char * key = g_object_get_data( G_OBJECT( w ), PREF_KEY );
     const char * value = gtk_entry_get_text( w );
 
-    tr_core_set_pref( TR_CORE( core ), key, value );
+    gtr_core_set_pref( TR_CORE( core ), key, value );
 }
 
 static GtkWidget*
@@ -216,7 +215,7 @@ chosen_cb( GtkFileChooser * w, gpointer core )
 {
     const char * key = g_object_get_data( G_OBJECT( w ), PREF_KEY );
     char * value = gtk_file_chooser_get_filename( GTK_FILE_CHOOSER( w ) );
-    tr_core_set_pref( TR_CORE( core ), key, value );
+    gtr_core_set_pref( TR_CORE( core ), key, value );
     g_free( value );
 }
 
@@ -252,6 +251,62 @@ target_cb( GtkWidget * tb, gpointer target )
     gtk_widget_set_sensitive( GTK_WIDGET( target ), b );
 }
 
+/****
+*****  Download Tab
+****/
+
+static GtkWidget*
+downloadPage( GObject * core )
+{
+    GtkWidget * t;
+    GtkWidget * w;
+    GtkWidget * l;
+    const char * s;
+    guint row = 0;
+
+    t = hig_workarea_create( );
+
+    hig_workarea_add_section_title( t, &row, _( "Location" ) );
+
+    w = new_path_chooser_button( TR_PREFS_KEY_DOWNLOAD_DIR, core );
+    hig_workarea_add_row( t, &row, _( "Save to _Location:" ), w, NULL );
+
+    hig_workarea_add_section_divider( t, &row );
+    hig_workarea_add_section_title( t, &row, _( "Queue" ) );
+
+    s = _( "Maximum active _downloads:" );
+    w = new_spin_button( TR_PREFS_KEY_DOWNLOAD_QUEUE_SIZE, core, 0, INT_MAX, 1 );
+    hig_workarea_add_row( t, &row, s, w, NULL );
+
+    s = _( "Downloads sharing data in the last N minutes are _active:" );
+    w = new_spin_button( TR_PREFS_KEY_QUEUE_STALLED_MINUTES, core, 1, INT_MAX, 15 );
+    hig_workarea_add_row( t, &row, s, w, NULL );
+
+    hig_workarea_add_section_divider( t, &row );
+    hig_workarea_add_section_title( t, &row, _( "Incomplete" ) );
+
+    s = _( "Append \"._part\" to incomplete files' names" );
+    w = new_check_button( s, TR_PREFS_KEY_RENAME_PARTIAL_FILES, core );
+    hig_workarea_add_wide_control( t, &row, w );
+
+    s = _( "Keep _incomplete torrents in:" );
+    l = new_check_button( s, TR_PREFS_KEY_INCOMPLETE_DIR_ENABLED, core );
+    w = new_path_chooser_button( TR_PREFS_KEY_INCOMPLETE_DIR, core );
+    gtk_widget_set_sensitive( GTK_WIDGET( w ), gtr_pref_flag_get( TR_PREFS_KEY_INCOMPLETE_DIR_ENABLED ) );
+    g_signal_connect( l, "toggled", G_CALLBACK( target_cb ), w );
+    hig_workarea_add_row_w( t, &row, l, w, NULL );
+
+    s = _( "Call _script when torrent is completed:" );
+    l = new_check_button( s, TR_PREFS_KEY_SCRIPT_TORRENT_DONE_ENABLED, core );
+    w = new_file_chooser_button( TR_PREFS_KEY_SCRIPT_TORRENT_DONE_FILENAME, core );
+    gtk_widget_set_sensitive( GTK_WIDGET( w ), gtr_pref_flag_get( TR_PREFS_KEY_SCRIPT_TORRENT_DONE_ENABLED ) );
+    g_signal_connect( l, "toggled", G_CALLBACK( target_cb ), w );
+    hig_workarea_add_row_w( t, &row, l, w, NULL );
+
+    hig_workarea_finish( t, &row );
+    return t;
+}
+
 /****
 *****  Torrent Tab
 ****/
@@ -259,12 +314,12 @@ target_cb( GtkWidget * tb, gpointer target )
 static GtkWidget*
 torrentPage( GObject * core )
 {
-    int          row = 0;
     const char * s;
     GtkWidget *  t;
     GtkWidget *  w;
     GtkWidget *  w2;
     GtkWidget *  l;
+    guint row = 0;
 
     t = hig_workarea_create( );
     hig_workarea_add_section_title( t, &row, _( "Adding" ) );
@@ -281,37 +336,10 @@ torrentPage( GObject * core )
     w = new_check_button( s, TR_PREFS_KEY_TRASH_ORIGINAL, core );
     hig_workarea_add_wide_control( t, &row, w );
 
-#ifdef HAVE_GIO
     s = _( "Automatically _add torrents from:" );
     l = new_check_button( s, PREF_KEY_DIR_WATCH_ENABLED, core );
     w = new_path_chooser_button( PREF_KEY_DIR_WATCH, core );
-    gtk_widget_set_sensitive( GTK_WIDGET( w ),
-                             gtr_pref_flag_get( PREF_KEY_DIR_WATCH_ENABLED ) );
-    g_signal_connect( l, "toggled", G_CALLBACK( target_cb ), w );
-    hig_workarea_add_row_w( t, &row, l, w, NULL );
-#endif
-
-    hig_workarea_add_section_divider( t, &row );
-    hig_workarea_add_section_title( t, &row, _( "Downloading" ) );
-
-    s = _( "Append \"._part\" to incomplete files' names" );
-    w = new_check_button( s, TR_PREFS_KEY_RENAME_PARTIAL_FILES, core );
-    hig_workarea_add_wide_control( t, &row, w );
-
-    w = new_path_chooser_button( TR_PREFS_KEY_DOWNLOAD_DIR, core );
-    hig_workarea_add_row( t, &row, _( "Save to _Location:" ), w, NULL );
-
-    s = _( "Keep _incomplete torrents in:" );
-    l = new_check_button( s, TR_PREFS_KEY_INCOMPLETE_DIR_ENABLED, core );
-    w = new_path_chooser_button( TR_PREFS_KEY_INCOMPLETE_DIR, core );
-    gtk_widget_set_sensitive( GTK_WIDGET( w ), gtr_pref_flag_get( TR_PREFS_KEY_INCOMPLETE_DIR_ENABLED ) );
-    g_signal_connect( l, "toggled", G_CALLBACK( target_cb ), w );
-    hig_workarea_add_row_w( t, &row, l, w, NULL );
-
-    s = _( "Call scrip_t when torrent is completed:" );
-    l = new_check_button( s, TR_PREFS_KEY_SCRIPT_TORRENT_DONE_ENABLED, core );
-    w = new_file_chooser_button( TR_PREFS_KEY_SCRIPT_TORRENT_DONE_FILENAME, core );
-    gtk_widget_set_sensitive( GTK_WIDGET( w ), gtr_pref_flag_get( TR_PREFS_KEY_SCRIPT_TORRENT_DONE_ENABLED ) );
+    gtk_widget_set_sensitive( GTK_WIDGET( w ), gtr_pref_flag_get( PREF_KEY_DIR_WATCH_ENABLED ) );
     g_signal_connect( l, "toggled", G_CALLBACK( target_cb ), w );
     hig_workarea_add_row_w( t, &row, l, w, NULL );
 
@@ -343,15 +371,15 @@ torrentPage( GObject * core )
 static GtkWidget*
 desktopPage( GObject * core )
 {
-    int          row = 0;
+    GtkWidget * t;
+    GtkWidget * w;
     const char * s;
-    GtkWidget *  t;
-    GtkWidget *  w;
+    guint row = 0;
 
     t = hig_workarea_create( );
     hig_workarea_add_section_title( t, &row, _( "Desktop" ) );
 
-    s = _( "Inhibit _hibernation when torrents are active" );
+    s = _( "_Inhibit hibernation when torrents are active" );
     w = new_check_button( s, PREF_KEY_INHIBIT_HIBERNATION, core );
     hig_workarea_add_wide_control( t, &row, w );
 
@@ -359,15 +387,20 @@ desktopPage( GObject * core )
     w = new_check_button( s, PREF_KEY_SHOW_TRAY_ICON, core );
     hig_workarea_add_wide_control( t, &row, w );
 
-    s = _( "Show _popup notifications" );
-    w = new_check_button( s, PREF_KEY_SHOW_DESKTOP_NOTIFICATION, core );
+    hig_workarea_add_section_divider( t, &row );
+    hig_workarea_add_section_title ( t, &row, _( "Notification" ) );
+
+    s = _( "Show a notification when torrents are a_dded" );
+    w = new_check_button( s, PREF_KEY_TORRENT_ADDED_NOTIFICATION_ENABLED, core );
     hig_workarea_add_wide_control( t, &row, w );
 
-#ifdef HAVE_LIBCANBERRA
-    s = _( "Play _sound when downloads are complete" );
-    w = new_check_button( s, PREF_KEY_PLAY_DOWNLOAD_COMPLETE_SOUND, core );
+    s = _( "Show a notification when torrents _finish" );
+    w = new_check_button( s, PREF_KEY_TORRENT_COMPLETE_NOTIFICATION_ENABLED, core );
+    hig_workarea_add_wide_control( t, &row, w );
+
+    s = _( "Play a _sound when torrents finish" );
+    w = new_check_button( s, PREF_KEY_TORRENT_COMPLETE_SOUND_ENABLED, core );
     hig_workarea_add_wide_control( t, &row, w );
-#endif
 
     hig_workarea_finish( t, &row );
     return t;
@@ -392,10 +425,10 @@ updateBlocklistText( GtkWidget * w, TrCore * core )
 {
     char buf1[512];
     char buf2[512];
-    const int n = tr_blocklistGetRuleCount( tr_core_session( core ) );
+    const int n = tr_blocklistGetRuleCount( gtr_core_session( core ) );
     g_snprintf( buf1, sizeof( buf1 ),
-                gtr_ngettext( "Blocklist contains %'d rule",
-                              "Blocklist contains %'d rules", n ), n );
+                ngettext( "Blocklist contains %'d rule",
+                          "Blocklist contains %'d rules", n ), n );
     g_snprintf( buf2, sizeof( buf2 ), "<i>%s</i>", buf1 );
     gtk_label_set_markup( GTK_LABEL( w ), buf2 );
 }
@@ -425,9 +458,9 @@ onBlocklistUpdateResponse( GtkDialog * dialog, gint response UNUSED, gpointer gd
 static void
 onBlocklistUpdated( TrCore * core, int n, gpointer gdata )
 {
-    const tr_bool success = n >= 0;
-    const int count = n >=0 ? n : tr_blocklistGetRuleCount( tr_core_session( core ) );
-    const char * s = gtr_ngettext( "Blocklist has %'d rule.", "Blocklist has %'d rules.", count );
+    const bool success = n >= 0;
+    const int count = n >=0 ? n : tr_blocklistGetRuleCount( gtr_core_session( core ) );
+    const char * s = ngettext( "Blocklist has %'d rule.", "Blocklist has %'d rules.", count );
     struct blocklist_data * data = gdata;
     GtkMessageDialog * d = GTK_MESSAGE_DIALOG( data->updateBlocklistDialog );
     gtk_widget_set_sensitive( data->updateBlocklistButton, TRUE );
@@ -452,7 +485,7 @@ onBlocklistUpdate( GtkButton * w, gpointer gdata )
     data->updateBlocklistDialog = d;
     g_signal_connect( d, "response", G_CALLBACK(onBlocklistUpdateResponse), data );
     gtk_widget_show( d );
-    tr_core_blocklist_update( data->core );
+    gtr_core_blocklist_update( data->core );
     data->updateBlocklistTag = g_signal_connect( data->core, "blocklist-updated", G_CALLBACK( onBlocklistUpdated ), data );
 }
 
@@ -470,7 +503,7 @@ onIntComboChanged( GtkComboBox * combo_box, gpointer core )
 {
     const int val = gtr_combo_box_get_active_enum( combo_box );
     const char * key = g_object_get_data( G_OBJECT( combo_box ), PREF_KEY );
-    tr_core_set_pref_int( TR_CORE( core ), key, val );
+    gtr_core_set_pref_int( TR_CORE( core ), key, val );
 }
 
 static GtkWidget*
@@ -489,7 +522,6 @@ new_encryption_combo( GObject * core, const char * key )
 static GtkWidget*
 privacyPage( GObject * core )
 {
-    int row = 0;
     const char * s;
     GtkWidget * t;
     GtkWidget * w;
@@ -497,6 +529,7 @@ privacyPage( GObject * core )
     GtkWidget * h;
     GtkWidget * e;
     struct blocklist_data * data;
+    guint row = 0;
 
     data = g_new0( struct blocklist_data, 1 );
     data->core = TR_CORE( core );
@@ -516,10 +549,10 @@ privacyPage( GObject * core )
     gtk_misc_set_alignment( GTK_MISC( w ), 0.0f, 0.5f );
     updateBlocklistText( w, TR_CORE( core ) );
     data->label = w;
-    h = gtk_hbox_new( FALSE, GUI_PAD_BIG );
+    h = gtr_hbox_new( FALSE, GUI_PAD_BIG );
     gtk_box_pack_start( GTK_BOX( h ), w, TRUE, TRUE, 0 );
     b = data->updateBlocklistButton = gtk_button_new_with_mnemonic( _( "_Update" ) );
-    g_object_set_data( G_OBJECT( b ), "session", tr_core_session( TR_CORE( core ) ) );
+    g_object_set_data( G_OBJECT( b ), "session", gtr_core_session( TR_CORE( core ) ) );
     g_signal_connect( b, "clicked", G_CALLBACK( onBlocklistUpdate ), data );
     g_signal_connect( data->check, "toggled", G_CALLBACK( target_cb ), b ); target_cb( data->check, b );
     gtk_box_pack_start( GTK_BOX( h ), b, FALSE, FALSE, 0 );
@@ -543,19 +576,19 @@ privacyPage( GObject * core )
     s = _( "Use PE_X to find more peers" );
     w = new_check_button( s, TR_PREFS_KEY_PEX_ENABLED, core );
     s = _( "PEX is a tool for exchanging peer lists with the peers you're connected to." );
-    gtr_widget_set_tooltip_text( w, s );
+    gtk_widget_set_tooltip_text( w, s );
     hig_workarea_add_wide_control( t, &row, w );
 
     s = _( "Use _DHT to find more peers" );
     w = new_check_button( s, TR_PREFS_KEY_DHT_ENABLED, core );
     s = _( "DHT is a tool for finding peers without a tracker." );
-    gtr_widget_set_tooltip_text( w, s );
+    gtk_widget_set_tooltip_text( w, s );
     hig_workarea_add_wide_control( t, &row, w );
 
     s = _( "Use _Local Peer Discovery to find more peers" );
     w = new_check_button( s, TR_PREFS_KEY_LPD_ENABLED, core );
     s = _( "LPD is a tool for finding peers on your local network." );
-    gtr_widget_set_tooltip_text( w, s );
+    gtk_widget_set_tooltip_text( w, s );
     hig_workarea_add_wide_control( t, &row, w );
 
     hig_workarea_finish( t, &row );
@@ -614,25 +647,23 @@ struct remote_page
 static void
 refreshWhitelist( struct remote_page * page )
 {
-    GtkTreeIter    iter;
+    GtkTreeIter iter;
+    GString * gstr = g_string_new( NULL );
     GtkTreeModel * model = GTK_TREE_MODEL( page->store );
-    GString *      gstr = g_string_new( NULL );
 
-    if( gtk_tree_model_get_iter_first( model, &iter ) ) do
-        {
-            char * address;
-            gtk_tree_model_get( model, &iter,
-                                COL_ADDRESS, &address,
-                                -1 );
-            g_string_append( gstr, address );
-            g_string_append( gstr, "," );
-            g_free( address );
-        }
-        while( gtk_tree_model_iter_next( model, &iter ) );
+    if( gtk_tree_model_iter_nth_child( model, &iter, NULL, 0 ) ) do
+    {
+        char * address;
+        gtk_tree_model_get( model, &iter, COL_ADDRESS, &address, -1 );
+        g_string_append( gstr, address );
+        g_string_append( gstr, "," );
+        g_free( address );
+    }
+    while( gtk_tree_model_iter_next( model, &iter ) );
 
     g_string_truncate( gstr, gstr->len - 1 ); /* remove the trailing comma */
 
-    tr_core_set_pref( page->core, TR_PREFS_KEY_RPC_WHITELIST, gstr->str );
+    gtr_core_set_pref( page->core, TR_PREFS_KEY_RPC_WHITELIST, gstr->str );
 
     g_string_free( gstr, TRUE );
 }
@@ -741,7 +772,7 @@ static void
 onLaunchClutchCB( GtkButton * w UNUSED, gpointer data UNUSED )
 {
     const int port = gtr_pref_int_get( TR_PREFS_KEY_RPC_PORT );
-    char * uri = g_strdup_printf( "http://localhost:%d/transmission/web", port );
+    char * uri = g_strdup_printf( "http://localhost:%d/", port );
 
     gtr_open_uri( uri );
     g_free( uri );
@@ -761,11 +792,11 @@ remotePageFree( gpointer gpage )
 static GtkWidget*
 webPage( GObject * core )
 {
-    const char *         s;
-    int                  row = 0;
-    GtkWidget *          t;
-    GtkWidget *          w;
-    GtkWidget *          h;
+    GtkWidget * t;
+    GtkWidget * w;
+    GtkWidget * h;
+    const char * s;
+    guint row = 0;
     struct remote_page * page = g_new0( struct remote_page, 1 );
 
     page->core = TR_CORE( core );
@@ -780,49 +811,49 @@ webPage( GObject * core )
     w = new_check_button( s, TR_PREFS_KEY_RPC_ENABLED, core );
     page->rpc_tb = GTK_TOGGLE_BUTTON( w );
     g_signal_connect( w, "clicked", G_CALLBACK( onRPCToggled ), page );
-    h = gtk_hbox_new( FALSE, GUI_PAD_BIG );
+    h = gtr_hbox_new( FALSE, GUI_PAD_BIG );
     gtk_box_pack_start( GTK_BOX( h ), w, TRUE, TRUE, 0 );
     w = gtk_button_new_with_mnemonic( _( "_Open web client" ) );
-    page->widgets = g_slist_append( page->widgets, w );
+    page->widgets = g_slist_prepend( page->widgets, w );
     g_signal_connect( w, "clicked", G_CALLBACK( onLaunchClutchCB ), NULL );
     gtk_box_pack_start( GTK_BOX( h ), w, FALSE, FALSE, 0 );
     hig_workarea_add_wide_control( t, &row, h );
 
     /* port */
     w = new_spin_button( TR_PREFS_KEY_RPC_PORT, core, 0, USHRT_MAX, 1 );
-    page->widgets = g_slist_append( page->widgets, w );
+    page->widgets = g_slist_prepend( page->widgets, w );
     w = hig_workarea_add_row( t, &row, _( "HTTP _port:" ), w, NULL );
-    page->widgets = g_slist_append( page->widgets, w );
+    page->widgets = g_slist_prepend( page->widgets, w );
 
     /* require authentication */
     s = _( "Use _authentication" );
     w = new_check_button( s, TR_PREFS_KEY_RPC_AUTH_REQUIRED, core );
     hig_workarea_add_wide_control( t, &row, w );
     page->auth_tb = GTK_TOGGLE_BUTTON( w );
-    page->widgets = g_slist_append( page->widgets, w );
+    page->widgets = g_slist_prepend( page->widgets, w );
     g_signal_connect( w, "clicked", G_CALLBACK( onRPCToggled ), page );
 
     /* username */
     s = _( "_Username:" );
     w = new_entry( TR_PREFS_KEY_RPC_USERNAME, core );
-    page->auth_widgets = g_slist_append( page->auth_widgets, w );
+    page->auth_widgets = g_slist_prepend( page->auth_widgets, w );
     w = hig_workarea_add_row( t, &row, s, w, NULL );
-    page->auth_widgets = g_slist_append( page->auth_widgets, w );
+    page->auth_widgets = g_slist_prepend( page->auth_widgets, w );
 
     /* password */
     s = _( "Pass_word:" );
     w = new_entry( TR_PREFS_KEY_RPC_PASSWORD, core );
     gtk_entry_set_visibility( GTK_ENTRY( w ), FALSE );
-    page->auth_widgets = g_slist_append( page->auth_widgets, w );
+    page->auth_widgets = g_slist_prepend( page->auth_widgets, w );
     w = hig_workarea_add_row( t, &row, s, w, NULL );
-    page->auth_widgets = g_slist_append( page->auth_widgets, w );
+    page->auth_widgets = g_slist_prepend( page->auth_widgets, w );
 
     /* require authentication */
     s = _( "Only allow these IP a_ddresses to connect:" );
     w = new_check_button( s, TR_PREFS_KEY_RPC_WHITELIST_ENABLED, core );
     hig_workarea_add_wide_control( t, &row, w );
     page->whitelist_tb = GTK_TOGGLE_BUTTON( w );
-    page->widgets = g_slist_append( page->widgets, w );
+    page->widgets = g_slist_prepend( page->widgets, w );
     g_signal_connect( w, "clicked", G_CALLBACK( onRPCToggled ), page );
 
     /* access control list */
@@ -841,9 +872,9 @@ webPage( GObject * core )
         g_signal_connect( w, "button-release-event",
                           G_CALLBACK( on_tree_view_button_released ), NULL );
 
-        page->whitelist_widgets = g_slist_append( page->whitelist_widgets, w );
+        page->whitelist_widgets = g_slist_prepend( page->whitelist_widgets, w );
         v = page->view = GTK_TREE_VIEW( w );
-        gtr_widget_set_tooltip_text( w, _( "IP addresses may use wildcards, such as 192.168.*.*" ) );
+        gtk_widget_set_tooltip_text( w, _( "IP addresses may use wildcards, such as 192.168.*.*" ) );
         sel = gtk_tree_view_get_selection( v );
         g_signal_connect( sel, "changed",
                           G_CALLBACK( onWhitelistSelectionChanged ), page );
@@ -869,9 +900,9 @@ webPage( GObject * core )
         w = hig_workarea_add_row( t, &row, s, w, NULL );
         gtk_misc_set_alignment( GTK_MISC( w ), 0.0f, 0.0f );
         gtk_misc_set_padding( GTK_MISC( w ), 0, GUI_PAD );
-        page->whitelist_widgets = g_slist_append( page->whitelist_widgets, w );
+        page->whitelist_widgets = g_slist_prepend( page->whitelist_widgets, w );
 
-        h = gtk_hbox_new( TRUE, GUI_PAD );
+        h = gtr_hbox_new( TRUE, GUI_PAD );
         w = gtk_button_new_from_stock( GTK_STOCK_REMOVE );
         g_signal_connect( w, "clicked", G_CALLBACK(
                               onRemoveWhitelistClicked ), page );
@@ -879,10 +910,10 @@ webPage( GObject * core )
         onWhitelistSelectionChanged( sel, page );
         gtk_box_pack_start( GTK_BOX( h ), w, TRUE, TRUE, 0 );
         w = gtk_button_new_from_stock( GTK_STOCK_ADD );
-        page->whitelist_widgets = g_slist_append( page->whitelist_widgets, w );
+        page->whitelist_widgets = g_slist_prepend( page->whitelist_widgets, w );
         g_signal_connect( w, "clicked", G_CALLBACK( onAddWhitelistClicked ), page );
         gtk_box_pack_start( GTK_BOX( h ), w, TRUE, TRUE, 0 );
-        w = gtk_hbox_new( FALSE, 0 );
+        w = gtr_hbox_new( FALSE, 0 );
         gtk_box_pack_start( GTK_BOX( w ), gtk_alignment_new( 0, 0, 0, 0 ),
                             TRUE, TRUE, 0 );
         gtk_box_pack_start( GTK_BOX( w ), h, FALSE, FALSE, 0 );
@@ -933,7 +964,7 @@ onTimeComboChanged( GtkComboBox * w,
         int          val = 0;
         gtk_tree_model_get( gtk_combo_box_get_model(
                                 w ), &iter, 0, &val, -1 );
-        tr_core_set_pref_int( TR_CORE( core ), key, val );
+        gtr_core_set_pref_int( TR_CORE( core ), key, val );
     }
 }
 
@@ -1012,12 +1043,12 @@ bandwidthPageFree( gpointer gpage )
 static GtkWidget*
 bandwidthPage( GObject * core )
 {
-    int row = 0;
     const char * s;
     GtkWidget * t;
     GtkWidget * l;
     GtkWidget * w, * w2, * h;
     char buf[512];
+    guint row = 0;
     struct BandwidthPage * page = tr_new0( struct BandwidthPage, 1 );
 
     page->core = TR_CORE( core );
@@ -1040,7 +1071,7 @@ bandwidthPage( GObject * core )
         hig_workarea_add_row_w( t, &row, w, w2, NULL );
 
     hig_workarea_add_section_divider( t, &row );
-    h = gtk_hbox_new( FALSE, GUI_PAD );
+    h = gtr_hbox_new( FALSE, GUI_PAD );
     w = gtk_image_new_from_stock( "alt-speed-on", -1 );
     gtk_box_pack_start( GTK_BOX( h ), w, FALSE, FALSE, 0 );
     g_snprintf( buf, sizeof( buf ), "<b>%s</b>", _( "Alternative Speed Limits" ) );
@@ -1066,16 +1097,16 @@ bandwidthPage( GObject * core )
         hig_workarea_add_row( t, &row, buf, w, NULL );
 
         s = _( "_Scheduled times:" );
-        h = gtk_hbox_new( FALSE, 0 );
+        h = gtr_hbox_new( FALSE, 0 );
         w2 = new_time_combo( core, TR_PREFS_KEY_ALT_SPEED_TIME_BEGIN );
-        page->sched_widgets = g_slist_append( page->sched_widgets, w2 );
+        page->sched_widgets = g_slist_prepend( page->sched_widgets, w2 );
         gtk_box_pack_start( GTK_BOX( h ), w2, TRUE, TRUE, 0 );
         w2 = l = gtk_label_new_with_mnemonic ( _( " _to " ) );
-        page->sched_widgets = g_slist_append( page->sched_widgets, w2 );
+        page->sched_widgets = g_slist_prepend( page->sched_widgets, w2 );
         gtk_box_pack_start( GTK_BOX( h ), w2, FALSE, FALSE, 0 );
         w2 = new_time_combo( core, TR_PREFS_KEY_ALT_SPEED_TIME_END );
         gtk_label_set_mnemonic_widget( GTK_LABEL( l ), w2 );
-        page->sched_widgets = g_slist_append( page->sched_widgets, w2 );
+        page->sched_widgets = g_slist_prepend( page->sched_widgets, w2 );
         gtk_box_pack_start( GTK_BOX( h ), w2, TRUE, TRUE, 0 );
         w = new_check_button( s, TR_PREFS_KEY_ALT_SPEED_TIME_ENABLED, core );
         g_signal_connect( w, "toggled", G_CALLBACK( onSchedToggled ), page );
@@ -1083,9 +1114,9 @@ bandwidthPage( GObject * core )
 
         s = _( "_On days:" );
         w = new_week_combo( core, TR_PREFS_KEY_ALT_SPEED_TIME_DAY );
-        page->sched_widgets = g_slist_append( page->sched_widgets, w );
+        page->sched_widgets = g_slist_prepend( page->sched_widgets, w );
         w = hig_workarea_add_row( t, &row, s, w, NULL );
-        page->sched_widgets = g_slist_append( page->sched_widgets, w );
+        page->sched_widgets = g_slist_prepend( page->sched_widgets, w );
 
     hig_workarea_finish( t, &row );
     g_object_set_data_full( G_OBJECT( t ), "page", page, bandwidthPageFree );
@@ -1151,40 +1182,22 @@ onPortTest( GtkButton * button UNUSED, gpointer vdata )
     struct network_page_data * data = vdata;
     gtk_widget_set_sensitive( data->portButton, FALSE );
     gtk_widget_set_sensitive( data->portSpin, FALSE );
-    gtk_label_set_markup( GTK_LABEL( data->portLabel ), _( "<i>Testing...</i>" ) );
+    gtk_label_set_markup( GTK_LABEL( data->portLabel ), _( "<i>Testing TCP port...</i>" ) );
     if( !data->portTag )
         data->portTag = g_signal_connect( data->core, "port-tested", G_CALLBACK(onPortTested), data );
-    tr_core_port_test( data->core );
-}
-
-static void
-onGNOMEClicked( GtkButton * button, gpointer vdata UNUSED )
-{
-    GError * err = NULL;
-
-    if( !g_spawn_command_line_async( "gnome-network-properties", &err ) )
-    {
-        GtkWidget * d = gtk_message_dialog_new( GTK_WINDOW( gtk_widget_get_toplevel( GTK_WIDGET( button ) ) ),
-                                                GTK_DIALOG_DESTROY_WITH_PARENT,
-                                                GTK_MESSAGE_ERROR,
-                                                GTK_BUTTONS_CLOSE,
-                                                "%s", err->message );
-        g_signal_connect_swapped( d, "response", G_CALLBACK( gtk_widget_destroy ), d );
-        gtk_widget_show( d );
-        g_clear_error( &err );
-    }
+    gtr_core_port_test( data->core );
 }
 
 static GtkWidget*
 networkPage( GObject * core )
 {
-    int                        row = 0;
-    const char *               s;
-    GtkWidget *                t;
-    GtkWidget *                w;
-    GtkWidget *                h;
-    GtkWidget *                l;
+    GtkWidget * t;
+    GtkWidget * w;
+    GtkWidget * h;
+    GtkWidget * l;
+    const char * s;
     struct network_page_data * data;
+    guint row = 0;
 
     /* register to stop listening to core prefs changes when the page is destroyed */
     data = g_new0( struct network_page_data, 1 );
@@ -1198,7 +1211,7 @@ networkPage( GObject * core )
     w = data->portSpin = new_spin_button( TR_PREFS_KEY_PEER_PORT, core, 1, USHRT_MAX, 1 );
     hig_workarea_add_row( t, &row, s, w, NULL );
 
-    h = gtk_hbox_new( FALSE, GUI_PAD_BIG );
+    h = gtr_hbox_new( FALSE, GUI_PAD_BIG );
     l = data->portLabel = gtk_label_new( _( "Status unknown" ) );
     gtk_misc_set_alignment( GTK_MISC( l ), 0.0f, 0.5f );
     gtk_box_pack_start( GTK_BOX( h ), l, TRUE, TRUE, 0 );
@@ -1220,19 +1233,21 @@ networkPage( GObject * core )
     hig_workarea_add_section_divider( t, &row );
     hig_workarea_add_section_title( t, &row, _( "Peer Limits" ) );
 
-    w = new_spin_button( TR_PREFS_KEY_PEER_LIMIT_TORRENT, core, 1, 300, 5 );
+    w = new_spin_button( TR_PREFS_KEY_PEER_LIMIT_TORRENT, core, 1, FD_SETSIZE, 5 );
     hig_workarea_add_row( t, &row, _( "Maximum peers per _torrent:" ), w, NULL );
-    w = new_spin_button( TR_PREFS_KEY_PEER_LIMIT_GLOBAL, core, 1, 3000, 5 );
+    w = new_spin_button( TR_PREFS_KEY_PEER_LIMIT_GLOBAL, core, 1, FD_SETSIZE, 5 );
     hig_workarea_add_row( t, &row, _( "Maximum peers _overall:" ), w, NULL );
 
     hig_workarea_add_section_divider( t, &row );
     hig_workarea_add_section_title( t, &row, _( "Options" ) );
 
-    w = gtk_button_new_with_mnemonic( _( "Edit GNOME Proxy Settings" ) );
-    g_signal_connect( w, "clicked", G_CALLBACK( onGNOMEClicked ), data );
-    h = gtk_hbox_new( FALSE, 0 );
-    gtk_box_pack_start( GTK_BOX( h ), w, FALSE, FALSE, 0 );
-    hig_workarea_add_wide_control( t, &row, h );
+#ifdef WITH_UTP
+    s = _( "Enable _uTP for peer communication" );
+    w = new_check_button( s, TR_PREFS_KEY_UTP_ENABLED, core );
+    s = _( "uTP is a tool for reducing network congestion." );
+    gtk_widget_set_tooltip_text( w, s );
+    hig_workarea_add_wide_control( t, &row, w );
+#endif
 
     hig_workarea_finish( t, &row );
     return t;
@@ -1264,6 +1279,9 @@ gtr_prefs_dialog_new( GtkWindow * parent, GObject * core )
     gtk_notebook_append_page( GTK_NOTEBOOK( n ),
                               torrentPage( core ),
                               gtk_label_new ( _( "Torrents" ) ) );
+    gtk_notebook_append_page( GTK_NOTEBOOK( n ),
+                              downloadPage( core ),
+                              gtk_label_new ( _( "Downloading" ) ) );
     gtk_notebook_append_page( GTK_NOTEBOOK( n ),
                               bandwidthPage( core ),
                               gtk_label_new ( _( "Speed" ) ) );
