@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: tr-window.c 11439 2010-11-27 19:30:20Z charles $
+ * $Id: tr-window.c 12997 2011-10-20 00:37:39Z jordan $
  *
- * Copyright (c) 2005-2008 Transmission authors and contributors
+ * Copyright (c) Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -28,6 +28,7 @@
 #include <glib/gi18n.h>
 
 #include <libtransmission/transmission.h>
+#include <libtransmission/utils.h> /* tr_formatter_speed_KBps() */
 
 #include "actions.h"
 #include "conf.h"
@@ -35,22 +36,9 @@
 #include "hig.h"
 #include "torrent-cell-renderer.h"
 #include "tr-prefs.h"
-#include "tr-torrent.h"
 #include "tr-window.h"
 #include "util.h"
 
-#if !GTK_CHECK_VERSION( 2, 8, 0 )
-static void
-gtk_tree_view_column_queue_resize( GtkTreeViewColumn * column ) /* yuck */
-{
-    const int spacing = gtk_tree_view_column_get_spacing( column );
-
-    gtk_tree_view_column_set_spacing( column, spacing + 1 );
-    gtk_tree_view_column_set_spacing( column, spacing );
-}
-
-#endif
-
 typedef struct
 {
     GtkWidget *           speedlimit_on_item[2];
@@ -79,12 +67,18 @@ typedef struct
 }
 PrivateData;
 
-#define PRIVATE_DATA_KEY "private-data"
+static GQuark
+get_private_data_key( void )
+{
+    static GQuark q = 0;
+    if( !q ) q = g_quark_from_static_string( "private-data" );
+    return q;
+}
 
 static PrivateData*
 get_private_data( TrWindow * w )
 {
-    return g_object_get_data ( G_OBJECT( w ), PRIVATE_DATA_KEY );
+    return g_object_get_qdata ( G_OBJECT( w ), get_private_data_key( ) );
 }
 
 /***
@@ -95,7 +89,7 @@ static void
 on_popup_menu( GtkWidget * self UNUSED,
                GdkEventButton * event )
 {
-    GtkWidget * menu = action_get_widget ( "/main-window-popup" );
+    GtkWidget * menu = gtr_action_get_widget ( "/main-window-popup" );
 
     gtk_menu_popup ( GTK_MENU( menu ), NULL, NULL, NULL, NULL,
                     ( event ? event->button : 0 ),
@@ -108,7 +102,26 @@ view_row_activated( GtkTreeView       * tree_view UNUSED,
                     GtkTreeViewColumn * column    UNUSED,
                     gpointer            user_data UNUSED )
 {
-    action_activate( "show-torrent-properties" );
+    gtr_action_activate( "show-torrent-properties" );
+}
+
+static gboolean
+tree_view_search_equal_func( GtkTreeModel * model,
+                             gint           column UNUSED,
+                             const gchar  * key,
+                             GtkTreeIter  * iter,
+                             gpointer       search_data UNUSED )
+{
+    gboolean match;
+    char * lower;
+    const char * name = NULL;
+
+    lower = g_strstrip( g_utf8_strdown( key, -1 ) );
+    gtk_tree_model_get( model, iter, MC_NAME_COLLATED, &name, -1 );
+    match = strstr( name, lower ) != NULL;
+    g_free( lower );
+
+    return !match;
 }
 
 static GtkWidget*
@@ -118,12 +131,18 @@ makeview( PrivateData * p )
     GtkTreeViewColumn * col;
     GtkTreeSelection *  sel;
     GtkCellRenderer *   r;
+    GtkTreeView * tree_view;
 
     view = gtk_tree_view_new( );
-    gtk_tree_view_set_headers_visible( GTK_TREE_VIEW( view ), FALSE );
-    gtk_tree_view_set_fixed_height_mode( GTK_TREE_VIEW( view ), TRUE );
+    tree_view = GTK_TREE_VIEW( view );
+    gtk_tree_view_set_search_column( tree_view, MC_NAME_COLLATED );
+    gtk_tree_view_set_search_equal_func( tree_view,
+                                         tree_view_search_equal_func,
+                                         NULL, NULL );
+    gtk_tree_view_set_headers_visible( tree_view, FALSE );
+    gtk_tree_view_set_fixed_height_mode( tree_view, TRUE );
 
-    p->selection = gtk_tree_view_get_selection( GTK_TREE_VIEW( view ) );
+    p->selection = gtk_tree_view_get_selection( tree_view );
 
     p->column = col = GTK_TREE_VIEW_COLUMN (g_object_new (GTK_TYPE_TREE_VIEW_COLUMN,
         "title", _("Torrent"),
@@ -133,15 +152,15 @@ makeview( PrivateData * p )
 
     p->renderer = r = torrent_cell_renderer_new( );
     gtk_tree_view_column_pack_start( col, r, FALSE );
-    gtk_tree_view_column_add_attribute( col, r, "torrent", MC_TORRENT_RAW );
+    gtk_tree_view_column_add_attribute( col, r, "torrent", MC_TORRENT );
     gtk_tree_view_column_add_attribute( col, r, "piece-upload-speed", MC_SPEED_UP );
     gtk_tree_view_column_add_attribute( col, r, "piece-download-speed", MC_SPEED_DOWN );
 
-    gtk_tree_view_append_column( GTK_TREE_VIEW( view ), col );
+    gtk_tree_view_append_column( tree_view, col );
     g_object_set( r, "xpad", GUI_PAD_SMALL, "ypad", GUI_PAD_SMALL, NULL );
 
-    gtk_tree_view_set_rules_hint( GTK_TREE_VIEW( view ), TRUE );
-    sel = gtk_tree_view_get_selection( GTK_TREE_VIEW( view ) );
+    gtk_tree_view_set_rules_hint( tree_view, TRUE );
+    sel = gtk_tree_view_get_selection( tree_view );
     gtk_tree_selection_set_mode( GTK_TREE_SELECTION( sel ),
                                  GTK_SELECTION_MULTIPLE );
 
@@ -156,7 +175,8 @@ makeview( PrivateData * p )
                       G_CALLBACK( view_row_activated ), NULL );
 
 
-    gtk_tree_view_set_model( GTK_TREE_VIEW( view ), p->filter_model );
+    gtk_tree_view_set_model( tree_view, p->filter_model );
+    g_object_unref( p->filter_model );
 
     return view;
 }
@@ -172,30 +192,34 @@ prefsChanged( TrCore * core UNUSED,
 
     if( !strcmp( key, PREF_KEY_COMPACT_VIEW ) )
     {
-        g_object_set( p->renderer, "compact", pref_flag_get( key ), NULL );
+        g_object_set( p->renderer, "compact", gtr_pref_flag_get( key ), NULL );
         /* since the cell size has changed, we need gtktreeview to revalidate
-         * its fixed-height mode values.  Unfortunately there's not an API call
+         * its fixed-height mode values. Unfortunately there's not an API call
          * for that, but it *does* revalidate when it thinks the style's been tweaked */
+#if GTK_CHECK_VERSION( 3,0,0 )
+        g_signal_emit_by_name( p->view, "style-updated", NULL, NULL );
+#else
         g_signal_emit_by_name( p->view, "style-set", NULL, NULL );
+#endif
     }
     else if( !strcmp( key, PREF_KEY_STATUSBAR ) )
     {
-        const gboolean isEnabled = pref_flag_get( key );
+        const gboolean isEnabled = gtr_pref_flag_get( key );
         g_object_set( p->status, "visible", isEnabled, NULL );
     }
     else if( !strcmp( key, PREF_KEY_FILTERBAR ) )
     {
-        const gboolean isEnabled = pref_flag_get( key );
+        const gboolean isEnabled = gtr_pref_flag_get( key );
         g_object_set( p->filter, "visible", isEnabled, NULL );
     }
     else if( !strcmp( key, PREF_KEY_TOOLBAR ) )
     {
-        const gboolean isEnabled = pref_flag_get( key );
+        const gboolean isEnabled = gtr_pref_flag_get( key );
         g_object_set( p->toolbar, "visible", isEnabled, NULL );
     }
     else if( !strcmp( key, PREF_KEY_STATUSBAR_STATS ) )
     {
-        tr_window_update( (TrWindow*)wind );
+        gtr_window_refresh( (TrWindow*)wind );
     }
     else if( !strcmp( key, TR_PREFS_KEY_ALT_SPEED_ENABLED ) ||
              !strcmp( key, TR_PREFS_KEY_ALT_SPEED_UP_KBps ) ||
@@ -244,7 +268,7 @@ status_menu_toggled_cb( GtkCheckMenuItem * menu_item,
         PrivateData * p = vprivate;
         const char *  val = g_object_get_data( G_OBJECT(
                                                    menu_item ), STATS_MODE );
-        tr_core_set_pref( p->core, PREF_KEY_STATUSBAR_STATS, val );
+        gtr_core_set_pref( p->core, PREF_KEY_STATUSBAR_STATS, val );
     }
 }
 
@@ -255,20 +279,20 @@ syncAltSpeedButton( PrivateData * p )
     char d[32];
     char * str;
     const char * fmt;
-    const gboolean b = pref_flag_get( TR_PREFS_KEY_ALT_SPEED_ENABLED );
+    const gboolean b = gtr_pref_flag_get( TR_PREFS_KEY_ALT_SPEED_ENABLED );
     const char * stock = b ? "alt-speed-on" : "alt-speed-off";
     GtkWidget * w = p->alt_speed_button;
 
-    tr_formatter_speed_KBps( u, pref_int_get( TR_PREFS_KEY_ALT_SPEED_UP_KBps ), sizeof( u ) );
-    tr_formatter_speed_KBps( d, pref_int_get( TR_PREFS_KEY_ALT_SPEED_DOWN_KBps ), sizeof( d ) );
-    fmt = b ? _( "Click to disable Temporary Speed Limits\n(%1$s down, %2$s up)" )
-            : _( "Click to enable Temporary Speed Limits\n(%1$s down, %2$s up)" );
+    tr_formatter_speed_KBps( u, gtr_pref_int_get( TR_PREFS_KEY_ALT_SPEED_UP_KBps ), sizeof( u ) );
+    tr_formatter_speed_KBps( d, gtr_pref_int_get( TR_PREFS_KEY_ALT_SPEED_DOWN_KBps ), sizeof( d ) );
+    fmt = b ? _( "Click to disable Alternative Speed Limits\n(%1$s down, %2$s up)" )
+            : _( "Click to enable Alternative Speed Limits\n(%1$s down, %2$s up)" );
     str = g_strdup_printf( fmt, d, u );
 
     gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( w ), b );
     gtk_image_set_from_stock( GTK_IMAGE( p->alt_speed_image ), stock, -1 );
     gtk_button_set_alignment( GTK_BUTTON( w ), 0.5, 0.5 );
-    gtr_widget_set_tooltip_text( w, str );
+    gtk_widget_set_tooltip_text( w, str );
 
     g_free( str );
 }
@@ -278,15 +302,13 @@ alt_speed_toggled_cb( GtkToggleButton * button, gpointer vprivate )
 {
     PrivateData * p = vprivate;
     const gboolean b = gtk_toggle_button_get_active( button );
-    tr_core_set_pref_bool( p->core, TR_PREFS_KEY_ALT_SPEED_ENABLED,  b );
+    gtr_core_set_pref_bool( p->core, TR_PREFS_KEY_ALT_SPEED_ENABLED,  b );
 }
 
 /***
 ****  FILTER
 ***/
 
-#if GTK_CHECK_VERSION( 2, 12, 0 )
-
 static void
 findMaxAnnounceTime( GtkTreeModel *      model,
                      GtkTreePath  * path UNUSED,
@@ -297,7 +319,7 @@ findMaxAnnounceTime( GtkTreeModel *      model,
     const tr_stat * torStat;
     time_t *        maxTime = gmaxTime;
 
-    gtk_tree_model_get( model, iter, MC_TORRENT_RAW, &tor, -1 );
+    gtk_tree_model_get( model, iter, MC_TORRENT, &tor, -1 );
     torStat = tr_torrentStatCached( tor );
     *maxTime = MAX( *maxTime, torStat->manualAnnounceTime );
 }
@@ -335,22 +357,20 @@ onAskTrackerQueryTooltip( GtkWidget *            widget UNUSED,
     }
 }
 
-#endif
-
 static gboolean
 onAltSpeedToggledIdle( gpointer vp )
 {
     PrivateData * p = vp;
-    gboolean b = tr_sessionUsesAltSpeed( tr_core_session( p->core ) );
-    tr_core_set_pref_bool( p->core, TR_PREFS_KEY_ALT_SPEED_ENABLED, b );
+    gboolean b = tr_sessionUsesAltSpeed( gtr_core_session( p->core ) );
+    gtr_core_set_pref_bool( p->core, TR_PREFS_KEY_ALT_SPEED_ENABLED, b );
 
     return FALSE;
 }
 
 static void
-onAltSpeedToggled( tr_session * s UNUSED, tr_bool isEnabled UNUSED, tr_bool byUser UNUSED, void * p )
+onAltSpeedToggled( tr_session * s UNUSED, bool isEnabled UNUSED, bool byUser UNUSED, void * p )
 {
-    gtr_idle_add( onAltSpeedToggledIdle, p );
+    gdk_threads_add_idle( onAltSpeedToggledIdle, p );
 }
 
 /***
@@ -372,7 +392,7 @@ onSpeedToggled( GtkCheckMenuItem * check, gpointer vp )
                                     : TR_PREFS_KEY_DSPEED_ENABLED;
 
     if( gtk_check_menu_item_get_active( check ) )
-        tr_core_set_pref_bool( p->core, key, isEnabled );
+        gtr_core_set_pref_bool( p->core, key, isEnabled );
 }
 
 static void
@@ -385,10 +405,10 @@ onSpeedSet( GtkCheckMenuItem * check, gpointer vp )
     tr_direction dir = GPOINTER_TO_INT( g_object_get_data( o, DIRECTION_KEY ) );
 
     key = dir==TR_UP ? TR_PREFS_KEY_USPEED_KBps : TR_PREFS_KEY_DSPEED_KBps;
-    tr_core_set_pref_int( p->core, key, KBps );
+    gtr_core_set_pref_int( p->core, key, KBps );
 
     key = dir==TR_UP ? TR_PREFS_KEY_USPEED_ENABLED : TR_PREFS_KEY_DSPEED_ENABLED;
-    tr_core_set_pref_bool( p->core, key, TRUE );
+    gtr_core_set_pref_bool( p->core, key, TRUE );
 }
 
 static GtkWidget*
@@ -446,7 +466,7 @@ onRatioToggled( GtkCheckMenuItem * check, gpointer vp )
     if( gtk_check_menu_item_get_active( check ) )
     {
         gboolean f = g_object_get_data( G_OBJECT( check ), ENABLED_KEY ) != 0;
-        tr_core_set_pref_bool( p->core, TR_PREFS_KEY_RATIO_ENABLED, f );
+        gtr_core_set_pref_bool( p->core, TR_PREFS_KEY_RATIO_ENABLED, f );
     }
 }
 static void
@@ -455,8 +475,8 @@ onRatioSet( GtkCheckMenuItem * check, gpointer vp )
     PrivateData * p = vp;
     int i = GPOINTER_TO_INT( g_object_get_data( G_OBJECT( check ), RATIO_KEY ) );
     const double ratio = stockRatios[i];
-    tr_core_set_pref_double( p->core, TR_PREFS_KEY_RATIO, ratio );
-    tr_core_set_pref_bool  ( p->core, TR_PREFS_KEY_RATIO_ENABLED, TRUE );
+    gtr_core_set_pref_double( p->core, TR_PREFS_KEY_RATIO, ratio );
+    gtr_core_set_pref_bool  ( p->core, TR_PREFS_KEY_RATIO_ENABLED, TRUE );
 }
 
 static GtkWidget*
@@ -534,26 +554,26 @@ onOptionsClicked( GtkButton * button UNUSED, gpointer vp )
     PrivateData * p = vp;
 
     w = p->speedlimit_on_item[TR_DOWN];
-    tr_formatter_speed_KBps( buf1, pref_int_get( TR_PREFS_KEY_DSPEED_KBps ), sizeof( buf1 ) );
-    gtk_label_set_text( GTK_LABEL( gtk_bin_get_child( GTK_BIN( w ) ) ), buf1 );
+    tr_formatter_speed_KBps( buf1, gtr_pref_int_get( TR_PREFS_KEY_DSPEED_KBps ), sizeof( buf1 ) );
+    gtr_label_set_text( GTK_LABEL( gtk_bin_get_child( GTK_BIN( w ) ) ), buf1 );
 
-    b = pref_flag_get( TR_PREFS_KEY_DSPEED_ENABLED );
+    b = gtr_pref_flag_get( TR_PREFS_KEY_DSPEED_ENABLED );
     w = b ? p->speedlimit_on_item[TR_DOWN] : p->speedlimit_off_item[TR_DOWN];
     gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM( w ), TRUE );
 
     w = p->speedlimit_on_item[TR_UP];
-    tr_formatter_speed_KBps( buf1, pref_int_get( TR_PREFS_KEY_USPEED_KBps ), sizeof( buf1 ) );
-    gtk_label_set_text( GTK_LABEL( gtk_bin_get_child( GTK_BIN( w ) ) ), buf1 );
+    tr_formatter_speed_KBps( buf1, gtr_pref_int_get( TR_PREFS_KEY_USPEED_KBps ), sizeof( buf1 ) );
+    gtr_label_set_text( GTK_LABEL( gtk_bin_get_child( GTK_BIN( w ) ) ), buf1 );
 
-    b = pref_flag_get( TR_PREFS_KEY_USPEED_ENABLED );
+    b = gtr_pref_flag_get( TR_PREFS_KEY_USPEED_ENABLED );
     w = b ? p->speedlimit_on_item[TR_UP] : p->speedlimit_off_item[TR_UP];
     gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM( w ), TRUE );
 
-    tr_strlratio( buf1, pref_double_get( TR_PREFS_KEY_RATIO ), sizeof( buf1 ) );
+    tr_strlratio( buf1, gtr_pref_double_get( TR_PREFS_KEY_RATIO ), sizeof( buf1 ) );
     g_snprintf( buf2, sizeof( buf2 ), _( "Stop at Ratio (%s)" ), buf1 );
-    gtk_label_set_text( GTK_LABEL( gtk_bin_get_child( GTK_BIN( p->ratio_on_item ) ) ), buf2 );
+    gtr_label_set_text( GTK_LABEL( gtk_bin_get_child( GTK_BIN( p->ratio_on_item ) ) ), buf2 );
 
-    b = pref_flag_get( TR_PREFS_KEY_RATIO_ENABLED );
+    b = gtr_pref_flag_get( TR_PREFS_KEY_RATIO_ENABLED );
     gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM( b ? p->ratio_on_item : p->ratio_off_item ), TRUE );
 
     gtk_menu_popup ( GTK_MENU( p->options_menu ), NULL, NULL, NULL, NULL, 0, gtk_get_current_event_time( ) );
@@ -564,7 +584,7 @@ onOptionsClicked( GtkButton * button UNUSED, gpointer vp )
 ***/
 
 GtkWidget *
-tr_window_new( GtkUIManager * ui_mgr, TrCore * core )
+gtr_window_new( GtkUIManager * ui_mgr, TrCore * core )
 {
     int           i, n;
     const char  * pch;
@@ -578,47 +598,47 @@ tr_window_new( GtkUIManager * ui_mgr, TrCore * core )
 
     /* make the window */
     self = gtk_window_new ( GTK_WINDOW_TOPLEVEL );
-    g_object_set_data_full( G_OBJECT(
-                                self ), PRIVATE_DATA_KEY, p, privateFree );
+    g_object_set_qdata_full( G_OBJECT(self), get_private_data_key( ), p, privateFree );
     win = GTK_WINDOW( self );
     gtk_window_set_title( win, g_get_application_name( ) );
     gtk_window_set_role( win, "tr-main" );
     gtk_window_set_default_size( win,
-                                 pref_int_get( PREF_KEY_MAIN_WINDOW_WIDTH ),
-                                 pref_int_get( PREF_KEY_MAIN_WINDOW_HEIGHT ) );
-    gtk_window_move( win, pref_int_get( PREF_KEY_MAIN_WINDOW_X ),
-                          pref_int_get( PREF_KEY_MAIN_WINDOW_Y ) );
-    if( pref_flag_get( PREF_KEY_MAIN_WINDOW_IS_MAXIMIZED ) )
+                                 gtr_pref_int_get( PREF_KEY_MAIN_WINDOW_WIDTH ),
+                                 gtr_pref_int_get( PREF_KEY_MAIN_WINDOW_HEIGHT ) );
+    gtk_window_move( win, gtr_pref_int_get( PREF_KEY_MAIN_WINDOW_X ),
+                          gtr_pref_int_get( PREF_KEY_MAIN_WINDOW_Y ) );
+    if( gtr_pref_flag_get( PREF_KEY_MAIN_WINDOW_IS_MAXIMIZED ) )
         gtk_window_maximize( win );
     gtk_window_add_accel_group( win, gtk_ui_manager_get_accel_group( ui_mgr ) );
 
     /* window's main container */
-    vbox = gtk_vbox_new ( FALSE, 0 );
+    vbox = gtr_vbox_new ( FALSE, 0 );
     gtk_container_add ( GTK_CONTAINER( self ), vbox );
 
     /* main menu */
-    mainmenu = action_get_widget( "/main-window-menu" );
-    w = action_get_widget( "/main-window-menu/torrent-menu/update-tracker" );
-#if GTK_CHECK_VERSION( 2, 12, 0 )
-    g_signal_connect( w, "query-tooltip",
-                      G_CALLBACK( onAskTrackerQueryTooltip ), p );
-#endif
+    mainmenu = gtr_action_get_widget( "/main-window-menu" );
+    w = gtr_action_get_widget( "/main-window-menu/torrent-menu/torrent-reannounce" );
+    g_signal_connect( w, "query-tooltip", G_CALLBACK( onAskTrackerQueryTooltip ), p );
 
     /* toolbar */
-    toolbar = p->toolbar = action_get_widget( "/main-window-toolbar" );
-    action_set_important( "add-torrent-toolbar", TRUE );
-    action_set_important( "show-torrent-properties", TRUE );
+    toolbar = p->toolbar = gtr_action_get_widget( "/main-window-toolbar" );
+#if GTK_CHECK_VERSION( 3,0,0 )
+    gtk_style_context_add_class( gtk_widget_get_style_context( toolbar ),
+                                 GTK_STYLE_CLASS_PRIMARY_TOOLBAR );
+#endif
+    gtr_action_set_important( "open-torrent-toolbar", TRUE );
+    gtr_action_set_important( "show-torrent-properties", TRUE );
 
     /* filter */
-    h = filter = p->filter = gtr_filter_bar_new( tr_core_session( core ),
-                                                 tr_core_model( core ),
+    h = filter = p->filter = gtr_filter_bar_new( gtr_core_session( core ),
+                                                 gtr_core_model( core ),
                                                  &p->filter_model );
     gtk_container_set_border_width( GTK_CONTAINER( h ), GUI_PAD_SMALL );
 
     /* status menu */
     menu = p->status_menu = gtk_menu_new( );
     l = NULL;
-    pch = pref_string_get( PREF_KEY_STATUSBAR_STATS );
+    pch = gtr_pref_string_get( PREF_KEY_STATUSBAR_STATS );
     for( i = 0, n = G_N_ELEMENTS( stats_modes ); i < n; ++i )
     {
         const char * val = stats_modes[i].val;
@@ -636,12 +656,12 @@ tr_window_new( GtkUIManager * ui_mgr, TrCore * core )
     }
 
     /* status */
-    h = status = p->status = gtk_hbox_new( FALSE, GUI_PAD_BIG );
+    h = status = p->status = gtr_hbox_new( FALSE, GUI_PAD_BIG );
     gtk_container_set_border_width( GTK_CONTAINER( h ), GUI_PAD_SMALL );
 
         w = gtk_button_new( );
         gtk_container_add( GTK_CONTAINER( w ), gtk_image_new_from_stock( "utilities", -1 ) );
-        gtr_widget_set_tooltip_text( w, _( "Options" ) );
+        gtk_widget_set_tooltip_text( w, _( "Options" ) );
         gtk_box_pack_start( GTK_BOX( h ), w, 0, 0, 0 );
         gtk_button_set_relief( GTK_BUTTON( w ), GTK_RELIEF_NONE );
         p->options_menu = createOptionsMenu( p );
@@ -655,36 +675,40 @@ tr_window_new( GtkUIManager * ui_mgr, TrCore * core )
         gtk_box_pack_start( GTK_BOX( h ), w, 0, 0, 0 );
 
         w = p->gutter_lb = gtk_label_new( "N Torrents" );
+        gtk_label_set_single_line_mode( GTK_LABEL( w ), TRUE );
         gtk_box_pack_start( GTK_BOX( h ), w, 1, 1, GUI_PAD );
 
-        hbox = gtk_hbox_new( FALSE, GUI_PAD );
+        hbox = gtr_hbox_new( FALSE, GUI_PAD );
             w = gtk_alignment_new( 0.0f, 0.0f, 0.0f, 0.0f );
             gtk_widget_set_size_request( w, GUI_PAD, 0u );
             gtk_box_pack_start( GTK_BOX( hbox ), w, FALSE, FALSE, 0 );
             w = p->ul_lb = gtk_label_new( NULL );
+            gtk_label_set_single_line_mode( GTK_LABEL( w ), TRUE );
             gtk_box_pack_start( GTK_BOX( hbox ), w, FALSE, FALSE, 0 );
             w = gtk_image_new_from_stock( GTK_STOCK_GO_UP, GTK_ICON_SIZE_MENU );
             gtk_box_pack_start( GTK_BOX( hbox ), w, FALSE, FALSE, 0 );
         gtk_box_pack_end( GTK_BOX( h ), hbox, FALSE, FALSE, 0 );
 
-        hbox = gtk_hbox_new( FALSE, GUI_PAD );
+        hbox = gtr_hbox_new( FALSE, GUI_PAD );
             w = gtk_alignment_new( 0.0f, 0.0f, 0.0f, 0.0f );
             gtk_widget_set_size_request( w, GUI_PAD, 0u );
             gtk_box_pack_start( GTK_BOX( hbox ), w, FALSE, FALSE, 0 );
             w = p->dl_lb = gtk_label_new( NULL );
+            gtk_label_set_single_line_mode( GTK_LABEL( w ), TRUE );
             gtk_box_pack_start( GTK_BOX( hbox ), w, FALSE, FALSE, 0 );
             w = gtk_image_new_from_stock( GTK_STOCK_GO_DOWN, GTK_ICON_SIZE_MENU );
             gtk_box_pack_start( GTK_BOX( hbox ), w, FALSE, FALSE, 0 );
         gtk_box_pack_end( GTK_BOX( h ), hbox, FALSE, FALSE, 0 );
 
-        hbox = gtk_hbox_new( FALSE, GUI_PAD );
+        hbox = gtr_hbox_new( FALSE, GUI_PAD );
             w = gtk_button_new( );
-            gtr_widget_set_tooltip_text( w, _( "Statistics" ) );
+            gtk_widget_set_tooltip_text( w, _( "Statistics" ) );
             gtk_container_add( GTK_CONTAINER( w ), gtk_image_new_from_stock( "ratio", -1 ) );
             gtk_button_set_relief( GTK_BUTTON( w ), GTK_RELIEF_NONE );
             g_signal_connect( w, "clicked", G_CALLBACK( onYinYangReleased ), p );
             gtk_box_pack_start( GTK_BOX( hbox ), w, FALSE, FALSE, 0 );
             w = p->stats_lb = gtk_label_new( NULL );
+            gtk_label_set_single_line_mode( GTK_LABEL( w ), TRUE );
             gtk_box_pack_end( GTK_BOX( hbox ), w, FALSE, FALSE, 0 );
         gtk_box_pack_end( GTK_BOX( h ), hbox, FALSE, FALSE, 0 );
 
@@ -732,7 +756,7 @@ tr_window_new( GtkUIManager * ui_mgr, TrCore * core )
     p->pref_handler_id = g_signal_connect( core, "prefs-changed",
                                            G_CALLBACK( prefsChanged ), self );
 
-    tr_sessionSetAltSpeedFunc( tr_core_session( core ), onAltSpeedToggled, p );
+    tr_sessionSetAltSpeedFunc( gtr_core_session( core ), onAltSpeedToggled, p );
 
     return self;
 }
@@ -743,22 +767,22 @@ updateTorrentCount( PrivateData * p )
     if( p && p->core )
     {
         char      buf[512];
-        const int torrentCount = gtk_tree_model_iter_n_children( tr_core_model( p->core ), NULL );
+        const int torrentCount = gtk_tree_model_iter_n_children( gtr_core_model( p->core ), NULL );
         const int visibleCount = gtk_tree_model_iter_n_children( p->filter_model, NULL );
 
         if( !torrentCount )
             *buf = '\0';
         else if( torrentCount != visibleCount )
             g_snprintf( buf, sizeof( buf ),
-                        gtr_ngettext( "%1$'d of %2$'d Torrent",
-                                      "%1$'d of %2$'d Torrents",
-                                      torrentCount ),
+                        ngettext( "%1$'d of %2$'d Torrent",
+                                  "%1$'d of %2$'d Torrents",
+                                  torrentCount ),
                         visibleCount, torrentCount );
         else
             g_snprintf( buf, sizeof( buf ),
-                        gtr_ngettext( "%'d Torrent", "%'d Torrents", torrentCount ),
+                        ngettext( "%'d Torrent", "%'d Torrents", torrentCount ),
                         torrentCount );
-        gtk_label_set_text( GTK_LABEL( p->gutter_lb ), buf );
+        gtr_label_set_text( GTK_LABEL( p->gutter_lb ), buf );
     }
 }
 
@@ -768,10 +792,10 @@ updateStats( PrivateData * p )
     const char *            pch;
     char                    up[32], down[32], ratio[32], buf[512];
     struct tr_session_stats stats;
-    tr_session *            session = tr_core_session( p->core );
+    tr_session *            session = gtr_core_session( p->core );
 
     /* update the stats */
-    pch = pref_string_get( PREF_KEY_STATUSBAR_STATS );
+    pch = gtr_pref_string_get( PREF_KEY_STATUSBAR_STATS );
     if( !strcmp( pch, "session-ratio" ) )
     {
         tr_sessionGetStats( session, &stats );
@@ -783,18 +807,18 @@ updateStats( PrivateData * p )
         tr_sessionGetStats( session, &stats );
         tr_strlsize( up, stats.uploadedBytes, sizeof( up ) );
         tr_strlsize( down, stats.downloadedBytes, sizeof( down ) );
-        /* Translators: "size|" is here for disambiguation.  Please remove it from your translation.
+        /* Translators: "size|" is here for disambiguation. Please remove it from your translation.
            %1$s is the size of the data we've downloaded
            %2$s is the size of the data we've uploaded */
         g_snprintf( buf, sizeof( buf ), Q_(
-                        "size|Down: %1$s, Up: %2$s" ), down, up );
+                        "Down: %1$s, Up: %2$s" ), down, up );
     }
     else if( !strcmp( pch, "total-transfer" ) )
     {
         tr_sessionGetCumulativeStats( session, &stats );
         tr_strlsize( up, stats.uploadedBytes, sizeof( up ) );
         tr_strlsize( down, stats.downloadedBytes, sizeof( down ) );
-        /* Translators: "size|" is here for disambiguation.  Please remove it from your translation.
+        /* Translators: "size|" is here for disambiguation. Please remove it from your translation.
            %1$s is the size of the data we've downloaded
            %2$s is the size of the data we've uploaded */
         g_snprintf( buf, sizeof( buf ), Q_(
@@ -806,22 +830,22 @@ updateStats( PrivateData * p )
         tr_strlratio( ratio, stats.ratio, sizeof( ratio ) );
         g_snprintf( buf, sizeof( buf ), _( "Ratio: %s" ), ratio );
     }
-    gtk_label_set_text( GTK_LABEL( p->stats_lb ), buf );
+    gtr_label_set_text( GTK_LABEL( p->stats_lb ), buf );
 }
 
 static void
 updateSpeeds( PrivateData * p )
 {
-    tr_session * session = tr_core_session( p->core );
+    tr_session * session = gtr_core_session( p->core );
 
     if( session != NULL )
     {
         char buf[128];
         double up=0, down=0;
         GtkTreeIter iter;
-        GtkTreeModel * model = tr_core_model( p->core );
+        GtkTreeModel * model = gtr_core_model( p->core );
 
-        if( gtk_tree_model_get_iter_first( model, &iter ) ) do
+        if( gtk_tree_model_iter_nth_child( model, &iter, NULL, 0 ) ) do
         {
             double u, d;
             gtk_tree_model_get( model, &iter, MC_SPEED_UP, &u,
@@ -833,45 +857,49 @@ updateSpeeds( PrivateData * p )
         while( gtk_tree_model_iter_next( model, &iter ) );
 
         tr_formatter_speed_KBps( buf, down, sizeof( buf ) );
-        gtk_label_set_text( GTK_LABEL( p->dl_lb ), buf );
+        gtr_label_set_text( GTK_LABEL( p->dl_lb ), buf );
 
         tr_formatter_speed_KBps( buf, up, sizeof( buf ) );
-        gtk_label_set_text( GTK_LABEL( p->ul_lb ), buf );
+        gtr_label_set_text( GTK_LABEL( p->ul_lb ), buf );
     }
 }
 
 void
-tr_window_update( TrWindow * self )
+gtr_window_refresh( TrWindow * self )
 {
     PrivateData * p = get_private_data( self );
 
-    if( p && p->core && tr_core_session( p->core ) )
+    if( p && p->core && gtr_core_session( p->core ) )
     {
         updateSpeeds( p );
         updateTorrentCount( p );
         updateStats( p );
-        gtk_tree_model_filter_refilter( GTK_TREE_MODEL_FILTER( p->filter_model ) );
     }
 }
 
 GtkTreeSelection*
-tr_window_get_selection( TrWindow * w )
+gtr_window_get_selection( TrWindow * w )
 {
     return get_private_data( w )->selection;
 }
 
 void
-tr_window_set_busy( TrWindow * w, gboolean isBusy )
+gtr_window_set_busy( TrWindow * w, gboolean isBusy )
 {
-    if( w && gtr_widget_get_realized( GTK_WIDGET( w ) ) )
-    {    
+    if( w && gtk_widget_get_realized( GTK_WIDGET( w ) ) )
+    {
         GdkDisplay * display = gtk_widget_get_display( GTK_WIDGET( w ) );
         GdkCursor * cursor = isBusy ? gdk_cursor_new_for_display( display, GDK_WATCH ) : NULL;
 
-        gdk_window_set_cursor( GTK_WIDGET(w)->window, cursor );
+        gdk_window_set_cursor( gtk_widget_get_window( GTK_WIDGET( w ) ), cursor );
         gdk_display_flush( display );
 
-        if( cursor )
-            gdk_cursor_unref( cursor ); 
+        if( cursor ) {
+#if GTK_CHECK_VERSION( 3,0,0 )
+            g_object_unref( cursor );
+#else
+            gdk_cursor_unref( cursor );
+#endif
+        }
     }
 }
