@@ -1,42 +1,29 @@
 /*
- * This file Copyright (C) 2008-2010 Mnemosyne LLC
+ * This file Copyright (C) Mnemosyne LLC
  *
- * This file is licensed by the GPL version 2.  Works owned by the
+ * This file is licensed by the GPL version 2. Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
  * so that the bulk of its code can remain under the MIT license.
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: util.c 11314 2010-10-15 23:23:57Z charles $
+ * $Id: util.c 12999 2011-10-20 00:46:26Z jordan $
  */
 
 #include <ctype.h> /* isxdigit() */
 #include <errno.h>
-#include <math.h> /* pow() */
 #include <stdarg.h>
-#include <stdlib.h> /* free() */
-#include <string.h> /* strcmp() */
-
-#include <sys/types.h> /* for gtr_lockfile()'s open() */
-#include <sys/stat.h> /* for gtr_lockfile()'s open() */
-#include <fcntl.h> /* for gtr_lockfile()'s open() */
+#include <string.h> /* strchr(), strrchr(), strlen(), strncmp(), strstr() */
 
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
 #include <glib/gstdio.h> /* g_unlink() */
-#ifdef HAVE_GIO
- #include <gio/gio.h> /* g_file_trash() */
-#endif
-#ifdef HAVE_DBUS_GLIB
- #include <dbus/dbus-glib.h>
-#endif
-
-#include <evhttp.h>
+#include <gio/gio.h> /* g_file_trash() */
 
 #include <libtransmission/transmission.h> /* TR_RATIO_NA, TR_RATIO_INF */
-#include <libtransmission/utils.h> /* tr_inf */
+#include <libtransmission/utils.h> /* tr_strratio() */
 #include <libtransmission/web.h> /* tr_webResponseStr() */
-#include <libtransmission/version.h> /* tr_inf */
+#include <libtransmission/version.h> /* SHORT_VERSION_STRING */
 
 #include "conf.h"
 #include "hig.h"
@@ -69,76 +56,15 @@ const char * speed_T_str = N_("TiB/s");
 ****
 ***/
 
-gtr_lockfile_state_t
-gtr_lockfile( const char * filename )
-{
-    gtr_lockfile_state_t ret;
-
-#ifdef WIN32
-
-    HANDLE file = CreateFile( filename,
-                              GENERIC_READ | GENERIC_WRITE,
-                              FILE_SHARE_READ | FILE_SHARE_WRITE,
-                              NULL,
-                              OPEN_ALWAYS,
-                              FILE_ATTRIBUTE_NORMAL,
-                              NULL );
-    if( file == INVALID_HANDLE_VALUE )
-        ret = GTR_LOCKFILE_EOPEN;
-    else if( !LockFile( file, 0, 0, 1, 1 ) )
-        ret = GTR_LOCKFILE_ELOCK;
-    else
-        ret = GTR_LOCKFILE_SUCCESS;
-
-#else
-
-    int fd = open( filename, O_RDWR | O_CREAT, 0666 );
-    if( fd < 0 )
-        ret = GTR_LOCKFILE_EOPEN;
-    else {
-        struct flock lk;
-        memset( &lk, 0,  sizeof( lk ) );
-        lk.l_start = 0;
-        lk.l_len = 0;
-        lk.l_type = F_WRLCK;
-        lk.l_whence = SEEK_SET;
-        if( -1 == fcntl( fd, F_SETLK, &lk ) )
-            ret = GTR_LOCKFILE_ELOCK;
-        else
-            ret = GTR_LOCKFILE_SUCCESS;
-    }
-
-#endif
-
-    return ret;
-}
-
-/***
-****
-***/
-
-int
-gtr_compare_double( const double a, const double b, int decimal_places )
-{
-    const int64_t ia = (int64_t)(a * pow( 10, decimal_places ) );
-    const int64_t ib = (int64_t)(b * pow( 10, decimal_places ) );
-    if( ia < ib ) return -1;
-    if( ia > ib ) return  1; 
-    return 0;
-}
-
-/***
-****
-***/
-
 const char*
 gtr_get_unicode_string( int i )
 {
     switch( i ) {
-        case GTR_UNICODE_UP:   return "\xE2\x86\x91";
-        case GTR_UNICODE_DOWN: return "\xE2\x86\x93";
-        case GTR_UNICODE_INF:  return "\xE2\x88\x9E";
-        default:               return "err";
+        case GTR_UNICODE_UP:      return "\xE2\x86\x91";
+        case GTR_UNICODE_DOWN:    return "\xE2\x86\x93";
+        case GTR_UNICODE_INF:     return "\xE2\x88\x9E";
+        case GTR_UNICODE_BULLET:  return "\xE2\x88\x99";
+        default:                  return "err";
     }
 }
 
@@ -158,7 +84,7 @@ char*
 tr_strlsize( char * buf, guint64 bytes, size_t buflen )
 {
     if( !bytes )
-        g_strlcpy( buf, Q_( "size|None" ), buflen );
+        g_strlcpy( buf, Q_( "None" ), buflen );
     else
         tr_formatter_size_B( buf, bytes, buflen );
 
@@ -179,43 +105,31 @@ tr_strltime( char * buf, int seconds, size_t buflen )
     minutes = ( seconds % 3600 ) / 60;
     seconds = ( seconds % 3600 ) % 60;
 
-    g_snprintf( d, sizeof( d ), gtr_ngettext( "%'d day", "%'d days", days ), days );
-    g_snprintf( h, sizeof( h ), gtr_ngettext( "%'d hour", "%'d hours", hours ), hours );
-    g_snprintf( m, sizeof( m ), gtr_ngettext( "%'d minute", "%'d minutes", minutes ), minutes );
-    g_snprintf( s, sizeof( s ), gtr_ngettext( "%'d second", "%'d seconds", seconds ), seconds );
+    g_snprintf( d, sizeof( d ), ngettext( "%'d day", "%'d days", days ), days );
+    g_snprintf( h, sizeof( h ), ngettext( "%'d hour", "%'d hours", hours ), hours );
+    g_snprintf( m, sizeof( m ), ngettext( "%'d minute", "%'d minutes", minutes ), minutes );
+    g_snprintf( s, sizeof( s ), ngettext( "%'d second", "%'d seconds", seconds ), seconds );
 
     if( days )
     {
         if( days >= 4 || !hours )
-        {
             g_strlcpy( buf, d, buflen );
-        }
         else
-        {
             g_snprintf( buf, buflen, "%s, %s", d, h );
-        }
     }
     else if( hours )
     {
         if( hours >= 4 || !minutes )
-        {
             g_strlcpy( buf, h, buflen );
-        }
         else
-        {
             g_snprintf( buf, buflen, "%s, %s", h, m );
-        }
     }
     else if( minutes )
     {
         if( minutes >= 4 || !seconds )
-        {
             g_strlcpy( buf, m, buflen );
-        }
         else
-        {
             g_snprintf( buf, buflen, "%s, %s", m, s );
-        }
     }
     else
     {
@@ -225,52 +139,35 @@ tr_strltime( char * buf, int seconds, size_t buflen )
     return buf;
 }
 
-char *
-gtr_localtime( time_t time )
-{
-    const struct tm tm = *localtime( &time );
-    char            buf[256], *eoln;
-
-    g_strlcpy( buf, asctime( &tm ), sizeof( buf ) );
-    if( ( eoln = strchr( buf, '\n' ) ) )
-        *eoln = '\0';
-
-    return g_locale_to_utf8( buf, -1, NULL, NULL, NULL );
-}
-
-int
-gtr_mkdir_with_parents( const char * path, int mode )
-{
-#if GLIB_CHECK_VERSION( 2, 8, 0 )
-    return !g_mkdir_with_parents( path, mode );
-#else
-    return !tr_mkdirp( path, mode );
-#endif
-}
-
 /* pattern-matching text; ie, legaltorrents.com */
-char*
-gtr_get_host_from_url( const char * url )
+void
+gtr_get_host_from_url( char * buf, size_t buflen, const char * url )
 {
-    char * h = NULL;
-    char * name;
-    const char * first_dot;
-    const char * last_dot;
+    char host[1024];
+    const char * pch;
 
-    tr_urlParse( url, -1, NULL, &h, NULL, NULL );
-    first_dot = strchr( h, '.' );
-    last_dot = strrchr( h, '.' );
+    if(( pch = strstr( url, "://" ))) {
+        const size_t hostlen = strcspn( pch+3, ":/" );
+        const size_t copylen = MIN( hostlen, sizeof(host)-1 );
+        memcpy( host, pch+3, copylen );
+        host[copylen] = '\0';
+    } else {
+        *host = '\0';
+    }
 
-    if( ( first_dot ) && ( last_dot ) && ( first_dot != last_dot ) )
-        name = g_strdup( first_dot + 1 );
-    else
-        name = g_strdup( h );
-
-    tr_free( h );
-    return name;
+    if( tr_addressIsIP( host ) )
+        g_strlcpy( buf, url, buflen );
+    else {
+        const char * first_dot = strchr( host, '.' );
+        const char * last_dot = strrchr( host, '.' );
+        if( ( first_dot ) && ( last_dot ) && ( first_dot != last_dot ) )
+            g_strlcpy( buf, first_dot + 1, buflen );
+        else
+            g_strlcpy( buf, host, buflen );
+    }
 }
 
-gboolean
+static gboolean
 gtr_is_supported_url( const char * str )
 {
     return !strncmp( str, "ftp://", 6 )
@@ -312,9 +209,7 @@ getWindow( GtkWidget * w )
 }
 
 void
-addTorrentErrorDialog( GtkWidget *  child,
-                       int          err,
-                       const char * filename )
+gtr_add_torrent_error_dialog( GtkWidget * child, int err, const char * file )
 {
     char * secondary;
     const char * fmt;
@@ -327,7 +222,7 @@ addTorrentErrorDialog( GtkWidget *  child,
         case TR_PARSE_DUPLICATE: fmt = _( "The torrent file \"%s\" is already in use." ); break;
         default: fmt = _( "The torrent file \"%s\" encountered an unknown error." ); break;
     }
-    secondary = g_strdup_printf( fmt, filename );
+    secondary = g_strdup_printf( fmt, file );
 
     w = gtk_message_dialog_new( win,
                                 GTK_DIALOG_DESTROY_WITH_PARENT,
@@ -401,66 +296,31 @@ on_tree_view_button_released( GtkWidget *      view,
     return FALSE;
 }
 
-gpointer
-gtr_object_ref_sink( gpointer object )
-{
-#if GLIB_CHECK_VERSION( 2, 10, 0 )
-    g_object_ref_sink( object );
-#else
-    g_object_ref( object );
-    gtk_object_sink( GTK_OBJECT( object ) );
-#endif
-    return object;
-}
-
-int
-gtr_strcmp0( const char * str1, const char * str2 )
-{
-#if GLIB_CHECK_VERSION( 2, 16, 0 )
-    return g_strcmp0( str1, str2 );
-#else
-    if( str1 && str2 ) return strcmp( str1, str2 );
-    if( str1 ) return 1;
-    if( str2 ) return -1;
-    return 0;
-#endif
-}
-
-const gchar *
-gtr_ngettext( const gchar * msgid,
-              const gchar * msgid_plural,
-              gulong n )
-{
-#if GLIB_CHECK_VERSION( 2, 18, 0 )
-    return g_dngettext( NULL, msgid, msgid_plural, n );
-#else
-    return ngettext( msgid, msgid_plural, n );
-#endif
-}
-
 int
 gtr_file_trash_or_remove( const char * filename )
 {
-    if( filename && g_file_test( filename, G_FILE_TEST_EXISTS ) )
-    {
-        gboolean trashed = FALSE;
-#ifdef HAVE_GIO
-        GError * err = NULL;
-        GFile *  file = g_file_new_for_path( filename );
-        trashed = g_file_trash( file, NULL, &err );
-        if( err )
-            g_message( "Unable to trash file \"%s\": %s", filename, err->message );
-        g_clear_error( &err );
-        g_object_unref( G_OBJECT( file ) );
-#endif
+    gboolean trashed = FALSE;
+    GFile * file = g_file_new_for_path( filename );
 
-        if( !trashed && g_remove( filename ) )
-        {
-            const int err = errno;
-            g_message( "Unable to remove file \"%s\": %s", filename, g_strerror( err ) );
+    if( gtr_pref_flag_get( PREF_KEY_TRASH_CAN_ENABLED ) ) {
+        GError * err = NULL;
+        trashed = g_file_trash( file, NULL, &err );
+        if( err ) {
+            g_message( "Unable to trash file \"%s\": %s", filename, err->message );
+            g_clear_error( &err );
         }
     }
 
+    if( !trashed ) {
+        GError * err = NULL;
+        trashed = g_file_delete( file, NULL, &err );
+        if( err ) {
+            g_message( "Unable to delete file \"%s\": %s", filename, err->message );
+            g_clear_error( &err );
+        }
+    }
+
+    g_object_unref( G_OBJECT( file ) );
     return 0;
 }
 
@@ -485,11 +345,10 @@ gtr_open_file( const char * path )
 {
     char * uri = NULL;
 
-#ifdef HAVE_GIO
     GFile * file = g_file_new_for_path( path );
     uri = g_file_get_uri( file );
     g_object_unref( G_OBJECT( file ) );
-#else
+
     if( g_path_is_absolute( path ) )
         uri = g_strdup_printf( "file://%s", path );
     else {
@@ -497,7 +356,6 @@ gtr_open_file( const char * path )
         uri = g_strdup_printf( "file://%s/%s", cwd, path );
         g_free( cwd );
     }
-#endif
 
     gtr_open_uri( uri );
     g_free( uri );
@@ -510,10 +368,11 @@ gtr_open_uri( const char * uri )
     {
         gboolean opened = FALSE;
 
-#ifdef HAVE_GIO
+        if( !opened )
+            opened = gtk_show_uri( NULL, uri, GDK_CURRENT_TIME, NULL );
+
         if( !opened )
             opened = g_app_info_launch_default_for_uri( uri, NULL, NULL );
-#endif
 
         if( !opened ) {
             char * argv[] = { (char*)"xdg-open", (char*)uri, NULL };
@@ -526,95 +385,6 @@ gtr_open_uri( const char * uri )
     }
 }
 
-#define VALUE_SERVICE_NAME        "com.transmissionbt.Transmission"
-#define VALUE_SERVICE_OBJECT_PATH "/com/transmissionbt/Transmission"
-#define VALUE_SERVICE_INTERFACE   "com.transmissionbt.Transmission"
-
-gboolean
-gtr_dbus_add_torrent( const char * filename )
-{
-    /* FIXME: why is this static? */
-    static gboolean handled = FALSE;
-
-#ifdef HAVE_DBUS_GLIB
-    char * payload;
-    gsize file_length;
-    char * file_contents = NULL;
-
-    /* If it's a file, load its contents and send them over the wire...
-     * it might be a temporary file that's going to disappear. */
-    if( g_file_get_contents( filename, &file_contents, &file_length, NULL ) )
-        payload = tr_base64_encode( file_contents, file_length, NULL );
-    else if( gtr_is_supported_url( filename ) || gtr_is_magnet_link( filename ) )
-        payload = tr_strdup( filename );
-    else
-        payload = NULL;
-
-    if( payload != NULL )
-    {
-        GError * err = NULL;
-        DBusGConnection * conn;
-        DBusGProxy * proxy = NULL;
-
-        if(( conn = dbus_g_bus_get( DBUS_BUS_SESSION, &err )))
-            proxy = dbus_g_proxy_new_for_name (conn, VALUE_SERVICE_NAME,
-                                                     VALUE_SERVICE_OBJECT_PATH,
-                                                     VALUE_SERVICE_INTERFACE );
-        else if( err )
-           g_message( "err: %s", err->message );
-
-        if( proxy )
-            dbus_g_proxy_call( proxy, "AddMetainfo", &err,
-                               G_TYPE_STRING, payload,
-                               G_TYPE_INVALID,
-                               G_TYPE_BOOLEAN, &handled,
-                               G_TYPE_INVALID );
-        if( err )
-           g_message( "err: %s", err->message );
-
-        if( proxy )
-            g_object_unref( proxy );
-        if( conn )
-            dbus_g_connection_unref( conn );
-
-        tr_free( payload );
-    }
-
-    g_free( file_contents );
-
-#endif
-    return handled;
-}
-
-gboolean
-gtr_dbus_present_window( void )
-{
-    static gboolean   success = FALSE;
-
-#ifdef HAVE_DBUS_GLIB
-    DBusGProxy *      proxy = NULL;
-    GError *          err = NULL;
-    DBusGConnection * conn;
-    if( ( conn = dbus_g_bus_get( DBUS_BUS_SESSION, &err ) ) )
-        proxy = dbus_g_proxy_new_for_name ( conn, VALUE_SERVICE_NAME,
-                                            VALUE_SERVICE_OBJECT_PATH,
-                                            VALUE_SERVICE_INTERFACE );
-    else if( err )
-        g_message( "err: %s", err->message );
-    if( proxy )
-        dbus_g_proxy_call( proxy, "PresentWindow", &err,
-                           G_TYPE_INVALID,
-                           G_TYPE_BOOLEAN, &success,
-                           G_TYPE_INVALID );
-    if( err )
-        g_message( "err: %s", err->message );
-
-    g_object_unref( proxy );
-    dbus_g_connection_unref( conn );
-#endif
-    return success;
-}
-
 /***
 ****
 ***/
@@ -703,29 +473,28 @@ gtr_priority_combo_new( void )
 ****
 ***/
 
-void
-gtr_widget_set_tooltip_text( GtkWidget * w, const char * tip )
+GtkWidget*
+gtr_hbox_new( gboolean homogenous UNUSED, gint spacing )
 {
-#if GTK_CHECK_VERSION( 2,12,0 )
-    gtk_widget_set_tooltip_text( w, tip );
+#if GTK_CHECK_VERSION( 3,2,0 )
+    return gtk_box_new( GTK_ORIENTATION_HORIZONTAL, spacing );
 #else
-    static GtkTooltips * tips = NULL;
-    if( tips == NULL )
-        tips = gtk_tooltips_new( );
-    gtk_tooltips_set_tip( tips, w, tip, NULL );
+    return gtk_hbox_new( homogenous, spacing );
 #endif
 }
 
-gboolean
-gtr_widget_get_realized( GtkWidget * w )
+GtkWidget*
+gtr_vbox_new( gboolean homogenous UNUSED, gint spacing )
 {
-#if GTK_CHECK_VERSION( 2,20,0 )
-    return gtk_widget_get_realized( w );
+#if GTK_CHECK_VERSION( 3,2,0 )
+    return gtk_box_new( GTK_ORIENTATION_VERTICAL, spacing );
 #else
-    return GTK_WIDGET_REALIZED( w ) != 0;
+    return gtk_vbox_new( homogenous, spacing );
 #endif
 }
 
+#define GTR_CHILD_HIDDEN "gtr-child-hidden"
+
 void
 gtr_widget_set_visible( GtkWidget * w, gboolean b )
 {
@@ -737,112 +506,44 @@ gtr_widget_set_visible( GtkWidget * w, gboolean b )
         GtkWindow * window = GTK_WINDOW( w );
 
         for( l=windows; l!=NULL; l=l->next )
-            if( GTK_IS_WINDOW( l->data ) )
-                if( gtk_window_get_transient_for( GTK_WINDOW( l->data ) ) == window )
-                    gtr_widget_set_visible( GTK_WIDGET( l->data ), b );
+        {
+            if( !GTK_IS_WINDOW( l->data ) )
+                continue;
+            if( gtk_window_get_transient_for( GTK_WINDOW( l->data ) ) != window )
+                continue;
+            if( gtk_widget_get_visible( GTK_WIDGET( l->data ) ) == b )
+                continue;
+
+            if( b && g_object_get_data( G_OBJECT( l->data ), GTR_CHILD_HIDDEN ) != NULL )
+            {
+                g_object_steal_data( G_OBJECT( l->data ), GTR_CHILD_HIDDEN );
+                gtr_widget_set_visible( GTK_WIDGET( l->data ), TRUE );
+            }
+            else if( !b )
+            {
+                g_object_set_data( G_OBJECT( l->data ), GTR_CHILD_HIDDEN, GINT_TO_POINTER( 1 ) );
+                gtr_widget_set_visible( GTK_WIDGET( l->data ), FALSE );
+            }
+        }
 
         g_list_free( windows );
     }
 
-#if GTK_CHECK_VERSION( 2,18,0 )
     gtk_widget_set_visible( w, b );
-#else
-    if( b )
-        gtk_widget_show( w );
-    else
-        gtk_widget_hide( w );
-#endif
 }
 
 void
-gtr_toolbar_set_orientation( GtkToolbar      * toolbar,
-                             GtkOrientation    orientation )
+gtr_dialog_set_content( GtkDialog * dialog, GtkWidget * content )
 {
-#if GTK_CHECK_VERSION( 2,16,0 )
-    gtk_orientable_set_orientation( GTK_ORIENTABLE( toolbar ), orientation );
-#else
-    gtk_toolbar_set_orientation( toolbar, orientation );
-#endif
+    GtkWidget * vbox = gtk_dialog_get_content_area( dialog );
+    gtk_box_pack_start( GTK_BOX( vbox ), content, TRUE, TRUE, 0 );
+    gtk_widget_show_all( content );
 }
 
-
 /***
 ****
 ***/
 
-#if !GTK_CHECK_VERSION( 2,12,0 )
-struct gtr_func_data
-{
-    GSourceFunc function;
-    gpointer data;
-};
-
-static void
-gtr_func_data_free( gpointer data )
-{
-#if GTK_CHECK_VERSION( 2,10,0 )
-    g_slice_free( struct gtr_func_data, data );
-#else
-    g_free( data );
-#endif
-}
-
-static struct gtr_func_data *
-gtr_func_data_new( GSourceFunc function, gpointer data )
-{
-#if GTK_CHECK_VERSION( 2,10,0 )
-    struct gtr_func_data * d = g_slice_new( struct gtr_func_data );
-#else
-    struct gtr_func_data * d = g_new( struct gtr_func_data, 1 );
-#endif
-    d->function = function;
-    d->data = data;
-    return d;
-}
-
-static gboolean
-gtr_thread_func( gpointer data )
-{
-    gboolean more;
-    struct gtr_func_data * idle_data = data;
-
-    gdk_threads_enter( );
-    more = idle_data->function( idle_data->data );
-    gdk_threads_leave( );
-
-    return more;
-}
-#endif
-
-void
-gtr_idle_add( GSourceFunc function, gpointer data )
-{
-#if GTK_CHECK_VERSION( 2,12,0 )
-    gdk_threads_add_idle( function, data );
-#else
-    g_idle_add_full( G_PRIORITY_DEFAULT,
-                     gtr_thread_func,
-                     gtr_func_data_new( function, data ),
-                     gtr_func_data_free );
-#endif
-}
-
-guint
-gtr_timeout_add_seconds( guint seconds, GSourceFunc function, gpointer data )
-{
-#if GTK_CHECK_VERSION( 2,14,0 )
-    return gdk_threads_add_timeout_seconds( seconds, function, data );
-#elif GTK_CHECK_VERSION( 2,12,0 )
-    return gdk_threads_add_timeout( seconds*1000, function, data );
-#else
-    return g_timeout_add_full( G_PRIORITY_DEFAULT,
-                               seconds * 1000,
-                               gtr_thread_func,
-                               gtr_func_data_new( function, data ),
-                               gtr_func_data_free );
-#endif
-}
-
 void
 gtr_http_failure_dialog( GtkWidget * parent, const char * url, long response_code )
 {
@@ -881,7 +582,7 @@ gtr_unrecognized_url_dialog( GtkWidget * parent, const char * url )
     if( gtr_is_magnet_link( url ) && ( strstr( url, xt ) == NULL ) )
     {
         g_string_append_printf( gstr, "\n \n" );
-        g_string_append_printf( gstr, _( "This magnet link appears to be intended for something other than BitTorrent.  BitTorrent magnet links have a section containing \"%s\"." ), xt );
+        g_string_append_printf( gstr, _( "This magnet link appears to be intended for something other than BitTorrent. BitTorrent magnet links have a section containing \"%s\"." ), xt );
     }
 
     gtk_message_dialog_format_secondary_text( GTK_MESSAGE_DIALOG( w ), "%s", gstr->str );
@@ -889,3 +590,43 @@ gtr_unrecognized_url_dialog( GtkWidget * parent, const char * url )
     gtk_widget_show( w );
     g_string_free( gstr, TRUE );
 }
+
+/***
+****
+***/
+
+void
+gtr_paste_clipboard_url_into_entry( GtkWidget * e )
+{
+  size_t i;
+
+  char * text[] = {
+    gtk_clipboard_wait_for_text( gtk_clipboard_get( GDK_SELECTION_PRIMARY ) ),
+    gtk_clipboard_wait_for_text( gtk_clipboard_get( GDK_SELECTION_CLIPBOARD ) )
+  };
+
+  for( i=0; i<G_N_ELEMENTS(text); ++i ) {
+      char * s = text[i];
+      if( s && ( gtr_is_supported_url( s ) || gtr_is_magnet_link( s )
+                                           || gtr_is_hex_hashcode( s ) ) ) {
+          gtk_entry_set_text( GTK_ENTRY( e ), s );
+          break;
+      }
+  }
+
+  for( i=0; i<G_N_ELEMENTS(text); ++i )
+    g_free( text[i] );
+}
+
+/***
+****
+***/
+
+void
+gtr_label_set_text( GtkLabel * lb, const char * newstr )
+{
+    const char * oldstr = gtk_label_get_text( lb );
+
+    if( tr_strcmp0( oldstr, newstr ) )
+        gtk_label_set_text( lb, newstr );
+}
