@@ -1,23 +1,23 @@
 /*
- * This file Copyright (C) 2008-2010 Mnemosyne LLC
+ * This file Copyright (C) Mnemosyne LLC
  *
- * This file is licensed by the GPL version 2.  Works owned by the
+ * This file is licensed by the GPL version 2. Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
  * so that the bulk of its code can remain under the MIT license.
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: bandwidth.c 11349 2010-10-24 01:08:08Z charles $
+ * $Id: bandwidth.c 12653 2011-08-08 16:58:29Z jordan $
  */
 
 #include <assert.h>
 #include <limits.h>
+#include <string.h> /* memset() */
 
 #include "transmission.h"
 #include "bandwidth.h"
-#include "crypto.h"
+#include "crypto.h" /* tr_cryptoWeakRandInt() */
 #include "peer-io.h"
-#include "ptrarray.h"
 #include "utils.h"
 
 #define dbgmsg( ... ) \
@@ -33,22 +33,32 @@
 static unsigned int
 getSpeed_Bps( const struct bratecontrol * r, unsigned int interval_msec, uint64_t now )
 {
-    uint64_t       bytes = 0;
-    const uint64_t cutoff = (now?now:tr_time_msec()) - interval_msec;
-    int            i = r->newest;
+    if( !now )
+        now = tr_time_msec();
 
-    for( ;; )
+    if( now != r->cache_time )
     {
-        if( r->transfers[i].date <= cutoff )
-            break;
+        int i = r->newest;
+        uint64_t bytes = 0;
+        const uint64_t cutoff = now - interval_msec;
+        struct bratecontrol * rvolatile = (struct bratecontrol*) r;
 
-        bytes += r->transfers[i].size;
+        for( ;; )
+        {
+            if( r->transfers[i].date <= cutoff )
+                break;
 
-        if( --i == -1 ) i = HISTORY_SIZE - 1; /* circular history */
-        if( i == r->newest ) break; /* we've come all the way around */
+            bytes += r->transfers[i].size;
+
+            if( --i == -1 ) i = HISTORY_SIZE - 1; /* circular history */
+            if( i == r->newest ) break; /* we've come all the way around */
+        }
+
+        rvolatile->cache_val = (unsigned int)(( bytes * 1000u ) / interval_msec);
+        rvolatile->cache_time = now;
     }
 
-    return (unsigned int)(( bytes * 1000u ) / interval_msec);
+    return r->cache_val;
 }
 
 static void
@@ -62,6 +72,9 @@ bytesUsed( const uint64_t now, struct bratecontrol * r, size_t size )
         r->transfers[r->newest].date = now;
         r->transfers[r->newest].size = size;
     }
+
+    /* invalidate cache_val*/
+    r->cache_time = 0;
 }
 
 /******
@@ -69,32 +82,33 @@ bytesUsed( const uint64_t now, struct bratecontrol * r, size_t size )
 *******
 ******/
 
-static inline int
-comparePointers( const void * a, const void * b )
+static int
+compareBandwidth( const void * va, const void * vb )
 {
-    if( a != b )
-        return a < b ? -1 : 1;
-
-    return 0;
+    const tr_bandwidth * a = va;
+    const tr_bandwidth * b = vb;
+    return a->uniqueKey - b->uniqueKey;
 }
 
 /***
 ****
 ***/
 
-tr_bandwidth*
+void
 tr_bandwidthConstruct( tr_bandwidth * b, tr_session * session, tr_bandwidth * parent )
 {
+    static unsigned int uniqueKey = 0;
+
     b->session = session;
     b->children = TR_PTR_ARRAY_INIT;
-    b->magicNumber = MAGIC_NUMBER;
-    b->band[TR_UP].honorParentLimits = TRUE;
-    b->band[TR_DOWN].honorParentLimits = TRUE;
+    b->magicNumber = BANDWIDTH_MAGIC_NUMBER;
+    b->uniqueKey = uniqueKey++;
+    b->band[TR_UP].honorParentLimits = true;
+    b->band[TR_DOWN].honorParentLimits = true;
     tr_bandwidthSetParent( b, parent );
-    return b;
 }
 
-tr_bandwidth*
+void
 tr_bandwidthDestruct( tr_bandwidth * b )
 {
     assert( tr_isBandwidth( b ) );
@@ -103,7 +117,6 @@ tr_bandwidthDestruct( tr_bandwidth * b )
     tr_ptrArrayDestruct( &b->children, NULL );
 
     memset( b, ~0, sizeof( tr_bandwidth ) );
-    return b;
 }
 
 /***
@@ -123,9 +136,9 @@ tr_bandwidthSetParent( tr_bandwidth  * b,
 
         assert( tr_isBandwidth( b->parent ) );
 
-        removed = tr_ptrArrayRemoveSorted( &b->parent->children, b, comparePointers );
+        removed = tr_ptrArrayRemoveSorted( &b->parent->children, b, compareBandwidth );
         assert( removed == b );
-        assert( tr_ptrArrayFindSorted( &b->parent->children, b, comparePointers ) == NULL );
+        assert( tr_ptrArrayFindSorted( &b->parent->children, b, compareBandwidth ) == NULL );
 
         b->parent = NULL;
     }
@@ -135,8 +148,9 @@ tr_bandwidthSetParent( tr_bandwidth  * b,
         assert( tr_isBandwidth( parent ) );
         assert( parent->parent != b );
 
-        tr_ptrArrayInsertSorted( &parent->children, b, comparePointers );
-        assert( tr_ptrArrayFindSorted( &parent->children, b, comparePointers ) == b );
+        assert( tr_ptrArrayFindSorted( &parent->children, b, compareBandwidth ) == NULL );
+        tr_ptrArrayInsertSorted( &parent->children, b, compareBandwidth );
+        assert( tr_ptrArrayFindSorted( &parent->children, b, compareBandwidth ) == b );
         b->parent = parent;
     }
 }
@@ -144,10 +158,6 @@ tr_bandwidthSetParent( tr_bandwidth  * b,
 /***
 ****
 ***/
-#if 0
-#warning do not check the code in with this enabled
-#define DEBUG_DIRECTION TR_UP
-#endif
 
 static void
 allocateBandwidth( tr_bandwidth  * b,
@@ -156,7 +166,7 @@ allocateBandwidth( tr_bandwidth  * b,
                    unsigned int    period_msec,
                    tr_ptrArray   * peer_pool )
 {
-    tr_priority_t priority;
+    const tr_priority_t priority = MAX( parent_priority, b->priority );
 
     assert( tr_isBandwidth( b ) );
     assert( tr_isDirection( dir ) );
@@ -166,28 +176,14 @@ allocateBandwidth( tr_bandwidth  * b,
     {
         const unsigned int nextPulseSpeed = b->band[dir].desiredSpeed_Bps;
         b->band[dir].bytesLeft = ( nextPulseSpeed * period_msec ) / 1000u;
-
-#ifdef DEBUG_DIRECTION
-        if( dir == DEBUG_DIRECTION )
-                fprintf( stderr, "bandwidth %p currentPieceSpeed(%5.2f of %5.2f) desiredSpeed(%5.2f), allocating %d\n",
-                         b, currentSpeed, tr_bandwidthGetRawSpeed( b, dir ), desiredSpeed,
-                         b->band[dir].bytesLeft );
-#endif
     }
 
-    priority = MAX( parent_priority, b->priority );
-
     /* add this bandwidth's peer, if any, to the peer pool */
     if( b->peer != NULL ) {
         b->peer->priority = priority;
         tr_ptrArrayAppend( peer_pool, b->peer );
     }
 
-#ifdef DEBUG_DIRECTION
-if( ( dir == DEBUG_DIRECTION ) && ( n > 1 ) )
-fprintf( stderr, "bandwidth %p has %d peers\n", b, n );
-#endif
-
     /* traverse & repeat for the subtree */
     if( 1 ) {
         int i;
@@ -205,16 +201,20 @@ phaseOne( tr_ptrArray * peerArray, tr_direction dir )
     int peerCount = tr_ptrArraySize( peerArray );
     struct tr_peerIo ** peers = (struct tr_peerIo**) tr_ptrArrayBase( peerArray );
 
-    /* First phase of IO.  Tries to distribute bandwidth fairly to keep faster
-     * peers from starving the others.  Loop through the peers, giving each a
-     * small chunk of bandwidth.  Keep looping until we run out of bandwidth
+    /* First phase of IO. Tries to distribute bandwidth fairly to keep faster
+     * peers from starving the others. Loop through the peers, giving each a
+     * small chunk of bandwidth. Keep looping until we run out of bandwidth
      * and/or peers that can use it */
     n = peerCount;
     dbgmsg( "%d peers to go round-robin for %s", n, (dir==TR_UP?"upload":"download") );
     i = n ? tr_cryptoWeakRandInt( n ) : 0; /* pick a random starting point */
-    while( n > 1 )
+    while( n > 0 )
     {
-        const size_t increment = 1024;
+        /* value of 3000 bytes chosen so that when using uTP we'll send a full-size
+         * frame right away and leave enough buffered data for the next frame to go
+         * out in a timely manner. */
+        const size_t increment = 3000;
+
         const int bytesUsed = tr_peerIoFlush( peers[i], dir, increment );
 
         dbgmsg( "peer #%d of %d used %d bytes in this pass", i, n, bytesUsed );
@@ -229,7 +229,7 @@ phaseOne( tr_ptrArray * peerArray, tr_direction dir )
             --n;
         }
 
-        if( i == n )
+        if( i >= n )
             i = 0;
     }
 }
@@ -267,15 +267,15 @@ tr_bandwidthAllocate( tr_bandwidth  * b,
         }
     }
 
-    /* First phase of IO.  Tries to distribute bandwidth fairly to keep faster
-     * peers from starving the others.  Loop through the peers, giving each a
-     * small chunk of bandwidth.  Keep looping until we run out of bandwidth
+    /* First phase of IO. Tries to distribute bandwidth fairly to keep faster
+     * peers from starving the others. Loop through the peers, giving each a
+     * small chunk of bandwidth. Keep looping until we run out of bandwidth
      * and/or peers that can use it */
     phaseOne( &high, dir );
     phaseOne( &normal, dir );
     phaseOne( &low, dir );
 
-    /* Second phase of IO.  To help us scale in high bandwidth situations,
+    /* Second phase of IO. To help us scale in high bandwidth situations,
      * enable on-demand IO for peers with bandwidth left to burn.
      * This on-demand IO is enabled until (1) the peer runs out of bandwidth,
      * or (2) the next tr_bandwidthAllocate() call, when we start over again. */
@@ -305,10 +305,11 @@ tr_bandwidthSetPeer( tr_bandwidth * b, tr_peerIo * peer )
 ****
 ***/
 
-unsigned int
-tr_bandwidthClamp( const tr_bandwidth  * b,
-                   tr_direction          dir,
-                   unsigned int          byteCount )
+static unsigned int
+bandwidthClamp( const tr_bandwidth  * b,
+                uint64_t              now,
+                tr_direction          dir,
+                unsigned int          byteCount )
 {
     assert( tr_isBandwidth( b ) );
     assert( tr_isDirection( dir ) );
@@ -316,14 +317,44 @@ tr_bandwidthClamp( const tr_bandwidth  * b,
     if( b )
     {
         if( b->band[dir].isLimited )
+        {
             byteCount = MIN( byteCount, b->band[dir].bytesLeft );
 
-        if( b->parent && b->band[dir].honorParentLimits )
-            byteCount = tr_bandwidthClamp( b->parent, dir, byteCount );
+            /* if we're getting close to exceeding the speed limit,
+             * clamp down harder on the bytes available */
+            if( byteCount > 0 )
+            {
+                double current;
+                double desired;
+                double r;
+
+                if( now == 0 )
+                    now = tr_time_msec( );
+
+                current = tr_bandwidthGetRawSpeed_Bps( b, now, TR_DOWN );
+                desired = tr_bandwidthGetDesiredSpeed_Bps( b, TR_DOWN );
+                r = desired >= 1 ? current / desired : 0;
+
+                     if( r > 1.0 ) byteCount = 0;
+                else if( r > 0.9 ) byteCount *= 0.8;
+                else if( r > 0.8 ) byteCount *= 0.9;
+            }
+        }
+
+        if( b->parent && b->band[dir].honorParentLimits && ( byteCount > 0 ) )
+            byteCount = bandwidthClamp( b->parent, now, dir, byteCount );
     }
 
     return byteCount;
 }
+unsigned int
+tr_bandwidthClamp( const tr_bandwidth  * b,
+                   tr_direction          dir,
+                   unsigned int          byteCount )
+{
+    return bandwidthClamp( b, 0, dir, byteCount );
+}
+
 
 unsigned int
 tr_bandwidthGetRawSpeed_Bps( const tr_bandwidth * b, const uint64_t now, const tr_direction dir )
@@ -347,7 +378,7 @@ void
 tr_bandwidthUsed( tr_bandwidth  * b,
                   tr_direction    dir,
                   size_t          byteCount,
-                  tr_bool         isPieceData,
+                  bool         isPieceData,
                   uint64_t        now )
 {
     struct tr_band * band;
