@@ -1,13 +1,13 @@
 /*
- * This file Copyright (C) 2007-2010 Mnemosyne LLC
+ * This file Copyright (C) Mnemosyne LLC
  *
- * This file is licensed by the GPL version 2.  Works owned by the
+ * This file is licensed by the GPL version 2. Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
  * so that the bulk of its code can remain under the MIT license.
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: peer-mgr.h 11295 2010-10-08 13:33:50Z charles $
+ * $Id: peer-mgr.h 12539 2011-07-10 15:24:51Z jordan $
  */
 
 #ifndef __TRANSMISSION__
@@ -24,9 +24,8 @@
 #endif
 
 #include "bitfield.h"
-#include "bitset.h"
 #include "history.h"
-#include "net.h"
+#include "net.h" /* tr_address */
 #include "peer-common.h" /* struct peer_request */
 
 /**
@@ -34,17 +33,29 @@
  * @{
  */
 
+struct UTPSocket;
 struct tr_peer_stat;
 struct tr_torrent;
 typedef struct tr_peerMgr tr_peerMgr;
 
+/* added_f's bitwise-or'ed flags */
 enum
 {
-    /* corresponds to ut_pex's added.f flags */
+    /* true if the peer supports encryption */
     ADDED_F_ENCRYPTION_FLAG = 1,
 
-    /* corresponds to ut_pex's added.f flags */
+    /* true if the peer is a seed or partial seed */
     ADDED_F_SEED_FLAG = 2,
+
+    /* true if the peer supports uTP */
+    ADDED_F_UTP_FLAGS = 4,
+
+    /* true if the peer has holepunch support */
+    ADDED_F_HOLEPUNCH = 8,
+
+    /* true if the peer telling us about this peer
+     * initiated the connection (implying that it is connectible) */
+    ADDED_F_CONNECTABLE = 16
 };
 
 typedef struct tr_pex
@@ -56,7 +67,6 @@ typedef struct tr_pex
 tr_pex;
 
 
-struct tr_bandwidth;
 struct tr_peerIo;
 struct tr_peermsgs;
 
@@ -78,11 +88,11 @@ struct peer_atom;
  */
 typedef struct tr_peer
 {
-    tr_bool                  peerIsChoked;
-    tr_bool                  peerIsInterested;
-    tr_bool                  clientIsChoked;
-    tr_bool                  clientIsInterested;
-    tr_bool                  doPurge;
+    bool                     peerIsChoked;
+    bool                     peerIsInterested;
+    bool                     clientIsChoked;
+    bool                     clientIsInterested;
+    bool                     doPurge;
 
     /* number of bad pieces they've contributed to */
     uint8_t                  strikes;
@@ -99,8 +109,8 @@ typedef struct tr_peer
     struct tr_peerIo       * io;
     struct peer_atom       * atom;
 
-    struct tr_bitfield     * blame;
-    struct tr_bitset         have;
+    struct tr_bitfield       blame;
+    struct tr_bitfield       have;
 
     /** how complete the peer's copy of the torrent is. [0.0...1.0] */
     float                    progress;
@@ -110,16 +120,27 @@ typedef struct tr_peer
 
     time_t                   chokeChangedAt;
 
-    tr_recentHistory       * blocksSentToClient;
-    tr_recentHistory       * blocksSentToPeer;
+    tr_recentHistory         blocksSentToClient;
+    tr_recentHistory         blocksSentToPeer;
 
-    tr_recentHistory       * cancelsSentToClient;
-    tr_recentHistory       * cancelsSentToPeer;
+    tr_recentHistory         cancelsSentToClient;
+    tr_recentHistory         cancelsSentToPeer;
 
     struct tr_peermsgs     * msgs;
 }
 tr_peer;
 
+void tr_peerConstruct( struct tr_peer * peer );
+
+void tr_peerDestruct( tr_torrent * tor, struct tr_peer * peer );
+
+
+static inline bool
+tr_isPex( const tr_pex * pex )
+{
+    return pex && tr_address_is_valid( &pex->addr );
+}
+
 const tr_address * tr_peerAddress( const tr_peer * );
 
 int tr_pexCompare( const void * a, const void * b );
@@ -128,25 +149,34 @@ tr_peerMgr* tr_peerMgrNew( tr_session * );
 
 void tr_peerMgrFree( tr_peerMgr * manager );
 
-tr_bool tr_peerMgrPeerIsSeed( const tr_torrent * tor,
+bool tr_peerMgrPeerIsSeed( const tr_torrent * tor,
                               const tr_address * addr );
 
+void tr_peerMgrSetUtpSupported( tr_torrent       * tor,
+                                const tr_address * addr );
+
+void tr_peerMgrSetUtpFailed( tr_torrent *tor,
+                             const tr_address *addr,
+                             bool failed );
+
 void tr_peerMgrGetNextRequests( tr_torrent          * torrent,
                                 tr_peer             * peer,
                                 int                   numwant,
                                 tr_block_index_t    * setme,
-                                int                 * numgot );
+                                int                 * numgot,
+                                bool                  get_intervals );
 
-tr_bool tr_peerMgrDidPeerRequest( const tr_torrent  * torrent,
-                                  const tr_peer     * peer,
-                                  tr_block_index_t    block );
+bool tr_peerMgrDidPeerRequest( const tr_torrent  * torrent,
+                               const tr_peer     * peer,
+                               tr_block_index_t    block );
 
 void tr_peerMgrRebuildRequests( tr_torrent * torrent );
 
 void tr_peerMgrAddIncoming( tr_peerMgr  * manager,
                             tr_address  * addr,
                             tr_port       port,
-                            int           socket );
+                            int           socket,
+                            struct UTPSocket *utp_socket );
 
 tr_pex * tr_peerMgrCompactToPex( const void    * compact,
                                  size_t          compactLen,
@@ -174,14 +204,10 @@ void tr_peerMgrAddPex( tr_torrent     * tor,
 
 void tr_peerMgrMarkAllAsSeeds( tr_torrent * tor );
 
-void tr_peerMgrSetBlame( tr_torrent        * tor,
-                         tr_piece_index_t    pieceIndex,
-                         int                 success );
-
 enum
 {
     TR_PEERS_CONNECTED,
-    TR_PEERS_ALL
+    TR_PEERS_INTERESTING
 };
 
 int  tr_peerMgrGetPeers( tr_torrent      * tor,
@@ -203,14 +229,14 @@ void tr_peerMgrTorrentAvailability( const tr_torrent * tor,
                                     int8_t           * tab,
                                     unsigned int       tabCount );
 
-struct tr_bitfield* tr_peerMgrGetAvailable( const tr_torrent * tor );
+uint64_t tr_peerMgrGetDesiredAvailable( const tr_torrent * tor );
+
+void tr_peerMgrOnTorrentGotMetainfo( tr_torrent * tor );
 
 void tr_peerMgrOnBlocklistChanged( tr_peerMgr * manager );
 
 void tr_peerMgrTorrentStats( tr_torrent * tor,
-                             int * setmePeersKnown,
                              int * setmePeersConnected,
-                             int * setmeSeedsConnected,
                              int * setmeWebseedsSendingToUs,
                              int * setmePeersSendingToUs,
                              int * setmePeersGettingFromUs,
@@ -219,8 +245,6 @@ void tr_peerMgrTorrentStats( tr_torrent * tor,
 struct tr_peer_stat* tr_peerMgrPeerStats( const tr_torrent * tor,
                                           int              * setmeCount );
 
-int tr_peerMgrGetWebseedSpeed_Bps( const tr_torrent * tor, uint64_t now );
-
 double* tr_peerMgrWebSpeeds_KBps( const tr_torrent * tor );
 
 
