@@ -1,13 +1,13 @@
 /*
- * This file Copyright (C) 2008-2010 Mnemosyne LLC
+ * This file Copyright (C) Mnemosyne LLC
  *
- * This file is licensed by the GPL version 2.  Works owned by the
+ * This file is licensed by the GPL version 2. Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
  * so that the bulk of its code can remain under the MIT license.
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: rpc-server.c 11398 2010-11-11 15:31:11Z charles $
+ * $Id: rpc-server.c 12930 2011-09-27 22:34:52Z livings124 $
  */
 
 #include <assert.h>
@@ -15,27 +15,28 @@
 #include <string.h> /* memcpy */
 #include <limits.h> /* INT_MAX */
 
-#include <sys/types.h> /* open */
-#include <sys/stat.h>  /* open */
-#include <fcntl.h>     /* open */
 #include <unistd.h>    /* close */
 
 #ifdef HAVE_ZLIB
  #include <zlib.h>
 #endif
 
-#include <event.h>
-#include <evhttp.h>
+#include <event2/buffer.h>
+#include <event2/event.h>
+#include <event2/http.h>
+#include <event2/http_struct.h> /* TODO: eventually remove this */
 
 #include "transmission.h"
 #include "bencode.h"
-#include "crypto.h"
+#include "crypto.h" /* tr_cryptoRandBuf(), tr_ssha1_matches() */
+#include "fdlimit.h"
 #include "list.h"
 #include "net.h"
-#include "platform.h"
+#include "platform.h" /* tr_getWebClientDir() */
 #include "ptrarray.h"
 #include "rpcimpl.h"
 #include "rpc-server.h"
+#include "session.h"
 #include "trevent.h"
 #include "utils.h"
 #include "web.h"
@@ -51,16 +52,13 @@
 #define MY_REALM "Transmission"
 #define TR_N_ELEMENTS( ary ) ( sizeof( ary ) / sizeof( *ary ) )
 
-#ifdef WIN32
-#define strncasecmp _strnicmp
-#endif
-
 struct tr_rpc_server
 {
-    tr_bool            isEnabled;
-    tr_bool            isPasswordEnabled;
-    tr_bool            isWhitelistEnabled;
+    bool               isEnabled;
+    bool               isPasswordEnabled;
+    bool               isWhitelistEnabled;
     tr_port            port;
+    char *             url;
     struct in_addr     bindAddress;
     struct evhttp *    httpd;
     tr_session *       session;
@@ -73,7 +71,7 @@ struct tr_rpc_server
     time_t             sessionIdExpiresAt;
 
 #ifdef HAVE_ZLIB
-    tr_bool            isStreamInitialized;
+    bool               isStreamInitialized;
     z_stream           stream;
 #endif
 };
@@ -154,12 +152,12 @@ tr_mimepart_free( struct tr_mimepart * p )
 
 static void
 extract_parts_from_multipart( const struct evkeyvalq * headers,
-                              const struct evbuffer * body,
+                              struct evbuffer * body,
                               tr_ptrArray * setme_parts )
 {
     const char * content_type = evhttp_find_header( headers, "Content-Type" );
-    const char * in = (const char*) EVBUFFER_DATA( body );
-    size_t inlen = EVBUFFER_LENGTH( body );
+    const char * in = (const char*) evbuffer_pullup( body, -1 );
+    size_t inlen = evbuffer_get_length( body );
 
     const char * boundary_key = "boundary=";
     const char * boundary_key_begin = strstr( content_type, boundary_key );
@@ -209,11 +207,11 @@ handle_upload( struct evhttp_request * req,
     {
         int i;
         int n;
-        tr_bool hasSessionId = FALSE;
+        bool hasSessionId = false;
         tr_ptrArray parts = TR_PTR_ARRAY_INIT;
 
         const char * query = strchr( req->uri, '?' );
-        const tr_bool paused = query && strstr( query + 1, "paused=true" );
+        const bool paused = query && strstr( query + 1, "paused=true" );
 
         extract_parts_from_multipart( req->input_headers, req->input_buffer, &parts );
         n = tr_ptrArraySize( &parts );
@@ -246,7 +244,7 @@ handle_upload( struct evhttp_request * req,
             int body_len = p->body_len;
             tr_benc top, *args;
             tr_benc test;
-            tr_bool have_source = FALSE;
+            bool have_source = false;
             char * body = p->body;
 
             if( body_len >= 2 && !memcmp( &body[body_len - 2], "\r\n", 2 ) )
@@ -260,23 +258,22 @@ handle_upload( struct evhttp_request * req,
             if( tr_urlIsValid( body, body_len ) )
             {
                 tr_bencDictAddRaw( args, "filename", body, body_len );
-                have_source = TRUE;
+                have_source = true;
             }
             else if( !tr_bencLoad( body, body_len, &test, NULL ) )
             {
                 char * b64 = tr_base64_encode( body, body_len, NULL );
                 tr_bencDictAddStr( args, "metainfo", b64 );
                 tr_free( b64 );
-                have_source = TRUE;
+                have_source = true;
             }
 
             if( have_source )
             {
-                struct evbuffer * json = evbuffer_new( );
-                tr_bencToBuf( &top, TR_FMT_JSON, json );
+                struct evbuffer * json = tr_bencToBuf( &top, TR_FMT_JSON );
                 tr_rpc_request_exec_json( server->session,
-                                          EVBUFFER_DATA( json ),
-                                          EVBUFFER_LENGTH( json ),
+                                          evbuffer_pullup( json, -1 ),
+                                          evbuffer_get_length( json ),
                                           NULL, NULL );
                 evbuffer_free( json );
             }
@@ -326,14 +323,11 @@ mimetype_guess( const char * path )
 }
 
 static void
-add_response( struct evhttp_request * req,
-              struct tr_rpc_server *  server,
-              struct evbuffer *       out,
-              const void *            content,
-              size_t                  content_len )
+add_response( struct evhttp_request * req, struct tr_rpc_server * server,
+              struct evbuffer * out, struct evbuffer * content )
 {
 #ifndef HAVE_ZLIB
-    evbuffer_add( out, content, content_len );
+    evbuffer_add_buffer( out, content );
 #else
     const char * key = "Accept-Encoding";
     const char * encoding = evhttp_find_header( req->input_headers, key );
@@ -341,27 +335,27 @@ add_response( struct evhttp_request * req,
 
     if( !do_compress )
     {
-        evbuffer_add( out, content, content_len );
+        evbuffer_add_buffer( out, content );
     }
     else
     {
         int state;
-
-        /* FIXME(libevent2): this won't compile under libevent2.
-           but we can use evbuffer_reserve_space() + evbuffer_commit_space() instead */
+        struct evbuffer_iovec iovec[1];
+        void * content_ptr = evbuffer_pullup( content, -1 );
+        const size_t content_len = evbuffer_get_length( content );
 
         if( !server->isStreamInitialized )
         {
             int compressionLevel;
 
-            server->isStreamInitialized = TRUE;
+            server->isStreamInitialized = true;
             server->stream.zalloc = (alloc_func) Z_NULL;
             server->stream.zfree = (free_func) Z_NULL;
             server->stream.opaque = (voidpf) Z_NULL;
 
             /* zlib's manual says: "Add 16 to windowBits to write a simple gzip header
              * and trailer around the compressed data instead of a zlib wrapper." */
-#ifdef TR_EMBEDDED
+#ifdef TR_LIGHTWEIGHT
             compressionLevel = Z_DEFAULT_COMPRESSION;
 #else
             compressionLevel = Z_BEST_COMPRESSION;
@@ -369,36 +363,36 @@ add_response( struct evhttp_request * req,
             deflateInit2( &server->stream, compressionLevel, Z_DEFLATED, 15+16, 8, Z_DEFAULT_STRATEGY );
         }
 
-        server->stream.next_in = (Bytef*) content;
+        server->stream.next_in = content_ptr;
         server->stream.avail_in = content_len;
 
         /* allocate space for the raw data and call deflate() just once --
          * we won't use the deflated data if it's longer than the raw data,
          * so it's okay to let deflate() run out of output buffer space */
-        evbuffer_expand( out, content_len );
-        server->stream.next_out = EVBUFFER_DATA( out );
-        server->stream.avail_out = content_len;
-
+        evbuffer_reserve_space( out, content_len, iovec, 1 );
+        server->stream.next_out = iovec[0].iov_base;
+        server->stream.avail_out = iovec[0].iov_len;
         state = deflate( &server->stream, Z_FINISH );
 
         if( state == Z_STREAM_END )
         {
-            EVBUFFER_LENGTH( out ) = content_len - server->stream.avail_out;
+            iovec[0].iov_len -= server->stream.avail_out;
 
 #if 0
             fprintf( stderr, "compressed response is %.2f of original (raw==%zu bytes; compressed==%zu)\n",
-                             (double)EVBUFFER_LENGTH(out)/content_len,
-                             content_len, EVBUFFER_LENGTH(out) );
+                             (double)evbuffer_get_length(out)/content_len,
+                             content_len, evbuffer_get_length(out) );
 #endif
             evhttp_add_header( req->output_headers,
                                "Content-Encoding", "gzip" );
         }
         else
         {
-            evbuffer_drain( out, EVBUFFER_LENGTH( out ) );
-            evbuffer_add( out, content, content_len );
+            memcpy( iovec[0].iov_base, content_ptr, content_len );
+            iovec[0].iov_len = content_len;
         }
 
+        evbuffer_commit_space( out, iovec, 1 );
         deflateReset( &server->stream );
     }
 #endif
@@ -415,6 +409,12 @@ add_time_header( struct evkeyvalq * headers, const char * key, time_t value )
     evhttp_add_header( headers, key, buf );
 }
 
+static void
+evbuffer_ref_cleanup_tr_free( const void * data UNUSED, size_t datalen UNUSED, void * extra )
+{
+    tr_free( extra );
+}
+
 static void
 serve_file( struct evhttp_request * req,
             struct tr_rpc_server *  server,
@@ -427,13 +427,16 @@ serve_file( struct evhttp_request * req,
     }
     else
     {
-        size_t content_len;
-        uint8_t * content;
+        void * file;
+        size_t file_len;
+        struct evbuffer * content;
         const int error = errno;
 
         errno = 0;
-        content_len = 0;
-        content = tr_loadFile( filename, &content_len );
+        file_len = 0;
+        file = tr_loadFile( filename, &file_len );
+        content = evbuffer_new( );
+        evbuffer_add_reference( content, file, file_len, evbuffer_ref_cleanup_tr_free, file );
 
         if( errno )
         {
@@ -451,12 +454,13 @@ serve_file( struct evhttp_request * req,
             evhttp_add_header( req->output_headers, "Content-Type", mimetype_guess( filename ) );
             add_time_header( req->output_headers, "Date", now );
             add_time_header( req->output_headers, "Expires", now+(24*60*60) );
-            add_response( req, server, out, content, content_len );
+            add_response( req, server, out, content );
             evhttp_send_reply( req, HTTP_OK, "OK", out );
 
             evbuffer_free( out );
-            tr_free( content );
         }
+
+        evbuffer_free( content );
     }
 }
 
@@ -466,8 +470,6 @@ handle_web_client( struct evhttp_request * req,
 {
     const char * webClientDir = tr_getWebClientDir( server->session );
 
-    assert( !strncmp( req->uri, "/transmission/web/", 18 ) );
-
     if( !webClientDir || !*webClientDir )
     {
         send_simple_response( req, HTTP_NOTFOUND,
@@ -485,7 +487,7 @@ handle_web_client( struct evhttp_request * req,
         char * pch;
         char * subpath;
 
-        subpath = tr_strdup( req->uri + 18 );
+        subpath = tr_strdup( req->uri + strlen( server->url ) + 4 );
         if(( pch = strchr( subpath, '?' )))
             *pch = '\0';
 
@@ -513,17 +515,15 @@ struct rpc_response_data
     struct tr_rpc_server  * server;
 };
 
-/* FIXME(libevent2): make "response" an evbuffer and remove response_len */
 static void
 rpc_response_func( tr_session      * session UNUSED,
-                   const char      * response,
-                   size_t            response_len,
+                   struct evbuffer * response,
                    void            * user_data )
 {
     struct rpc_response_data * data = user_data;
     struct evbuffer * buf = evbuffer_new( );
 
-    add_response( data->req, data->server, buf, response, response_len );
+    add_response( data->req, data->server, buf, response );
     evhttp_add_header( data->req->output_headers,
                            "Content-Type", "application/json; charset=UTF-8" );
     evhttp_send_reply( data->req, HTTP_OK, "OK", buf );
@@ -551,35 +551,35 @@ handle_rpc( struct evhttp_request * req,
     else if( req->type == EVHTTP_REQ_POST )
     {
         tr_rpc_request_exec_json( server->session,
-                                  EVBUFFER_DATA( req->input_buffer ),
-                                  EVBUFFER_LENGTH( req->input_buffer ),
+                                  evbuffer_pullup( req->input_buffer, -1 ),
+                                  evbuffer_get_length( req->input_buffer ),
                                   rpc_response_func, data );
     }
 
 }
 
-static tr_bool
+static bool
 isAddressAllowed( const tr_rpc_server * server,
                   const char *          address )
 {
     tr_list * l;
 
     if( !server->isWhitelistEnabled )
-        return TRUE;
+        return true;
 
     for( l=server->whitelist; l!=NULL; l=l->next )
         if( tr_wildmat( address, l->data ) )
-            return TRUE;
+            return true;
 
-    return FALSE;
+    return false;
 }
 
-static tr_bool
+static bool
 test_session_id( struct tr_rpc_server * server, struct evhttp_request * req )
 {
     const char * ours = get_current_session_id( server );
     const char * theirs = evhttp_find_header( req->input_headers, TR_RPC_SESSION_ID_HEADER );
-    const tr_bool success =  theirs && !strcmp( theirs, ours );
+    const bool success =  theirs && !strcmp( theirs, ours );
     return success;
 }
 
@@ -597,7 +597,7 @@ handle_request( struct evhttp_request * req, void * arg )
         evhttp_add_header( req->output_headers, "Server", MY_REALM );
 
         auth = evhttp_find_header( req->input_headers, "Authorization" );
-        if( auth && !strncasecmp( auth, "basic ", 6 ) )
+        if( auth && !evutil_ascii_strncasecmp( auth, "basic ", 6 ) )
         {
             int    plen;
             char * p = tr_base64_decode( auth + 6, 0, &plen );
@@ -614,7 +614,7 @@ handle_request( struct evhttp_request * req, void * arg )
                 "<p>Unauthorized IP Address.</p>"
                 "<p>Either disable the IP address whitelist or add your address to it.</p>"
                 "<p>If you're editing settings.json, see the 'rpc-whitelist' and 'rpc-whitelist-enabled' entries.</p>"
-                "<p>If you're still using ACLs, use a whitelist instead.  See the transmission-daemon manpage for details.</p>" );
+                "<p>If you're still using ACLs, use a whitelist instead. See the transmission-daemon manpage for details.</p>" );
         }
         else if( server->isPasswordEnabled
                  && ( !pass || !user || strcmp( server->username, user )
@@ -626,18 +626,18 @@ handle_request( struct evhttp_request * req, void * arg )
                                "Basic realm=\"" MY_REALM "\"" );
             send_simple_response( req, 401, "Unauthorized User" );
         }
-        else if( !strcmp( req->uri, "/transmission/web" )
-               || !strcmp( req->uri, "/transmission/clutch" )
-               || !strcmp( req->uri, "/" ) )
+        else if( strncmp( req->uri, server->url, strlen( server->url ) ) )
         {
-            evhttp_add_header( req->output_headers, "Location", "/transmission/web/" );
+            char * location = tr_strdup_printf( "%sweb/", server->url );
+            evhttp_add_header( req->output_headers, "Location", location );
             send_simple_response( req, HTTP_MOVEPERM, NULL );
+            tr_free( location );
         }
-        else if( !strncmp( req->uri, "/transmission/web/", 18 ) )
+        else if( !strncmp( req->uri + strlen( server->url ), "web/", 4 ) )
         {
             handle_web_client( req, server );
         }
-        else if( !strncmp( req->uri, "/transmission/upload", 20 ) )
+        else if( !strncmp( req->uri + strlen( server->url ), "upload", 6 ) )
         {
             handle_upload( req, server );
         }
@@ -662,7 +662,7 @@ handle_request( struct evhttp_request * req, void * arg )
             tr_free( tmp );
         }
 #endif
-        else if( !strncmp( req->uri, "/transmission/rpc", 17 ) )
+        else if( !strncmp( req->uri + strlen( server->url ), "rpc", 3 ) )
         {
             handle_rpc( req, server );
         }
@@ -685,9 +685,8 @@ startServer( void * vserver )
     {
         addr.type = TR_AF_INET;
         addr.addr.addr4 = server->bindAddress;
-        server->httpd = evhttp_new( tr_eventGetBase( server->session ) );
-        evhttp_bind_socket( server->httpd, tr_ntop_non_ts( &addr ),
-                            server->port );
+        server->httpd = evhttp_new( server->session->event_base );
+        evhttp_bind_socket( server->httpd, tr_address_to_string( &addr ), server->port );
         evhttp_set_gencb( server->httpd, handle_request, server );
 
     }
@@ -716,14 +715,14 @@ onEnabledChanged( void * vserver )
 
 void
 tr_rpcSetEnabled( tr_rpc_server * server,
-                  tr_bool         isEnabled )
+                  bool            isEnabled )
 {
     server->isEnabled = isEnabled;
 
     tr_runInEventThread( server->session, onEnabledChanged, server );
 }
 
-tr_bool
+bool
 tr_rpcIsEnabled( const tr_rpc_server * server )
 {
     return server->isEnabled;
@@ -763,15 +762,30 @@ tr_rpcGetPort( const tr_rpc_server * server )
 }
 
 void
-tr_rpcSetWhitelist( tr_rpc_server * server,
-                    const char    * whitelistStr )
+tr_rpcSetUrl( tr_rpc_server * server, const char * url )
+{
+    char * tmp = server->url;
+    server->url = tr_strdup( url );
+    dbgmsg( "setting our URL to [%s]", server->url );
+    tr_free( tmp );
+}
+
+const char*
+tr_rpcGetUrl( const tr_rpc_server * server )
+{
+    return server->url ? server->url : "";
+}
+
+void
+tr_rpcSetWhitelist( tr_rpc_server * server, const char * whitelistStr )
 {
     void * tmp;
     const char * walk;
 
     /* keep the string */
-    tr_free( server->whitelistStr );
+    tmp = server->whitelistStr;
     server->whitelistStr = tr_strdup( whitelistStr );
+    tr_free( tmp );
 
     /* clear out the old whitelist entries */
     while(( tmp = tr_list_pop_front( &server->whitelist )))
@@ -802,12 +816,12 @@ tr_rpcGetWhitelist( const tr_rpc_server * server )
 
 void
 tr_rpcSetWhitelistEnabled( tr_rpc_server  * server,
-                           tr_bool          isEnabled )
+                           bool             isEnabled )
 {
     server->isWhitelistEnabled = isEnabled != 0;
 }
 
-tr_bool
+bool
 tr_rpcGetWhitelistEnabled( const tr_rpc_server * server )
 {
     return server->isWhitelistEnabled;
@@ -818,12 +832,12 @@ tr_rpcGetWhitelistEnabled( const tr_rpc_server * server )
 ****/
 
 void
-tr_rpcSetUsername( tr_rpc_server * server,
-                   const char *    username )
+tr_rpcSetUsername( tr_rpc_server * server, const char * username )
 {
-    tr_free( server->username );
+    char * tmp = server->username;
     server->username = tr_strdup( username );
     dbgmsg( "setting our Username to [%s]", server->username );
+    tr_free( tmp );
 }
 
 const char*
@@ -851,14 +865,13 @@ tr_rpcGetPassword( const tr_rpc_server * server )
 }
 
 void
-tr_rpcSetPasswordEnabled( tr_rpc_server * server,
-                          tr_bool          isEnabled )
+tr_rpcSetPasswordEnabled( tr_rpc_server * server, bool isEnabled )
 {
     server->isPasswordEnabled = isEnabled;
     dbgmsg( "setting 'password enabled' to %d", (int)isEnabled );
 }
 
-tr_bool
+bool
 tr_rpcIsPasswordEnabled( const tr_rpc_server * server )
 {
     return server->isPasswordEnabled;
@@ -870,7 +883,7 @@ tr_rpcGetBindAddress( const tr_rpc_server * server )
     tr_address addr;
     addr.type = TR_AF_INET;
     addr.addr.addr4 = server->bindAddress;
-    return tr_ntop_non_ts( &addr );
+    return tr_address_to_string( &addr );
 }
 
 /****
@@ -890,6 +903,7 @@ closeServer( void * vserver )
     if( s->isStreamInitialized )
         deflateEnd( &s->stream );
 #endif
+    tr_free( s->url );
     tr_free( s->sessionId );
     tr_free( s->whitelistStr );
     tr_free( s->username );
@@ -908,58 +922,79 @@ tr_rpc_server *
 tr_rpcInit( tr_session  * session, tr_benc * settings )
 {
     tr_rpc_server * s;
-    tr_bool found;
-    tr_bool boolVal;
+    bool boolVal;
     int64_t i;
-    const char *str;
+    const char * str;
+    const char * key;
     tr_address address;
 
     s = tr_new0( tr_rpc_server, 1 );
     s->session = session;
 
-    found = tr_bencDictFindBool( settings, TR_PREFS_KEY_RPC_ENABLED, &boolVal );
-    assert( found );
-    s->isEnabled = boolVal;
+    key = TR_PREFS_KEY_RPC_ENABLED;
+    if( !tr_bencDictFindBool( settings, key, &boolVal ) )
+        tr_nerr( MY_NAME, _( "Couldn't find settings key \"%s\"" ), key );
+    else
+        s->isEnabled = boolVal;
 
-    found = tr_bencDictFindInt( settings, TR_PREFS_KEY_RPC_PORT, &i );
-    assert( found );
-    s->port = i;
+    key = TR_PREFS_KEY_RPC_PORT;
+    if( !tr_bencDictFindInt( settings, key, &i ) )
+        tr_nerr( MY_NAME, _( "Couldn't find settings key \"%s\"" ), key );
+    else
+        s->port = i;
 
-    found = tr_bencDictFindBool( settings, TR_PREFS_KEY_RPC_WHITELIST_ENABLED, &boolVal );
-    assert( found );
-    tr_rpcSetWhitelistEnabled( s, boolVal );
+    key = TR_PREFS_KEY_RPC_URL;
+    if( !tr_bencDictFindStr( settings, TR_PREFS_KEY_RPC_URL, &str ) )
+        tr_nerr( MY_NAME, _( "Couldn't find settings key \"%s\"" ), key );
+    else
+        s->url = tr_strdup( str );
 
-    found = tr_bencDictFindBool( settings, TR_PREFS_KEY_RPC_AUTH_REQUIRED, &boolVal );
-    assert( found );
-    tr_rpcSetPasswordEnabled( s, boolVal );
+    key = TR_PREFS_KEY_RPC_WHITELIST_ENABLED;
+    if( !tr_bencDictFindBool( settings, key, &boolVal ) )
+        tr_nerr( MY_NAME, _( "Couldn't find settings key \"%s\"" ), key );
+    else
+        tr_rpcSetWhitelistEnabled( s, boolVal );
 
-    found = tr_bencDictFindStr( settings, TR_PREFS_KEY_RPC_WHITELIST, &str );
-    assert( found );
-    tr_rpcSetWhitelist( s, str ? str : "127.0.0.1" );
+    key = TR_PREFS_KEY_RPC_AUTH_REQUIRED;
+    if( !tr_bencDictFindBool( settings, key, &boolVal ) )
+        tr_nerr( MY_NAME, _( "Couldn't find settings key \"%s\"" ), key );
+    else
+        tr_rpcSetPasswordEnabled( s, boolVal );
 
-    found = tr_bencDictFindStr( settings, TR_PREFS_KEY_RPC_USERNAME, &str );
-    assert( found );
-    tr_rpcSetUsername( s, str );
+    key = TR_PREFS_KEY_RPC_WHITELIST;
+    if( !tr_bencDictFindStr( settings, key, &str ) && str )
+        tr_nerr( MY_NAME, _( "Couldn't find settings key \"%s\"" ), key );
+    else
+        tr_rpcSetWhitelist( s, str );
 
-    found = tr_bencDictFindStr( settings, TR_PREFS_KEY_RPC_PASSWORD, &str );
-    assert( found );
-    tr_rpcSetPassword( s, str );
+    key = TR_PREFS_KEY_RPC_USERNAME;
+    if( !tr_bencDictFindStr( settings, key, &str ) )
+        tr_nerr( MY_NAME, _( "Couldn't find settings key \"%s\"" ), key );
+    else
+        tr_rpcSetUsername( s, str );
 
-    found = tr_bencDictFindStr( settings, TR_PREFS_KEY_RPC_BIND_ADDRESS, &str );
-    assert( found );
-    if( tr_pton( str, &address ) == NULL ) {
-        tr_err( _( "%s is not a valid address" ), str );
+    key = TR_PREFS_KEY_RPC_PASSWORD;
+    if( !tr_bencDictFindStr( settings, key, &str ) )
+        tr_nerr( MY_NAME, _( "Couldn't find settings key \"%s\"" ), key );
+    else
+        tr_rpcSetPassword( s, str );
+
+    key = TR_PREFS_KEY_RPC_BIND_ADDRESS;
+    if( !tr_bencDictFindStr( settings, TR_PREFS_KEY_RPC_BIND_ADDRESS, &str ) ) {
+        tr_nerr( MY_NAME, _( "Couldn't find settings key \"%s\"" ), key );
+        address = tr_inaddr_any;
+    } else if( !tr_address_from_string( &address, str ) ) {
+        tr_nerr( MY_NAME, _( "%s is not a valid address" ), str );
         address = tr_inaddr_any;
     } else if( address.type != TR_AF_INET ) {
-        tr_err( _( "%s is not an IPv4 address. RPC listeners must be IPv4" ),
-                   str );
+        tr_nerr( MY_NAME, _( "%s is not an IPv4 address. RPC listeners must be IPv4" ), str );
         address = tr_inaddr_any;
     }
     s->bindAddress = address.addr.addr4;
 
     if( s->isEnabled )
     {
-        tr_ninf( MY_NAME, _( "Serving RPC and Web requests on port %d" ), (int) s->port );
+        tr_ninf( MY_NAME, _( "Serving RPC and Web requests on port 127.0.0.1:%d%s" ), (int) s->port, s->url );
         tr_runInEventThread( session, startServer, s );
 
         if( s->isWhitelistEnabled )
