@@ -1,13 +1,13 @@
 /*
- * This file Copyright (C) 2008-2010 Mnemosyne LLC
+ * This file Copyright (C) Mnemosyne LLC
  *
- * This file is licensed by the GPL version 2.  Works owned by the
+ * This file is licensed by the GPL version 2. Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
  * so that the bulk of its code can remain under the MIT license.
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: rpcimpl.c 11415 2010-11-14 18:17:52Z livings124 $
+ * $Id: rpcimpl.c 13001 2011-10-20 00:58:58Z jordan $
  */
 
 #include <assert.h>
@@ -21,7 +21,7 @@
  #include <zlib.h>
 #endif
 
-#include <event.h> /* evbuffer */
+#include <event2/buffer.h>
 
 #include "transmission.h"
 #include "bencode.h"
@@ -30,13 +30,12 @@
 #include "json.h"
 #include "rpcimpl.h"
 #include "session.h"
-#include "stats.h"
 #include "torrent.h"
 #include "utils.h"
 #include "version.h"
 #include "web.h"
 
-#define RPC_VERSION     11
+#define RPC_VERSION     14
 #define RPC_VERSION_MIN 1
 
 #define RECENTLY_ACTIVE_SECONDS 60
@@ -95,17 +94,16 @@ struct tr_rpc_idle_data
 static void
 tr_idle_function_done( struct tr_rpc_idle_data * data, const char * result )
 {
-    struct evbuffer * buf = evbuffer_new( );
+    struct evbuffer * buf;
 
     if( result == NULL )
         result = "success";
     tr_bencDictAddStr( data->response, "result", result );
 
-    tr_bencToBuf( data->response, TR_FMT_JSON_LEAN, buf );
-    (*data->callback)( data->session, (const char*)EVBUFFER_DATA(buf),
-                       EVBUFFER_LENGTH(buf), data->callback_user_data );
-
+    buf = tr_bencToBuf( data->response, TR_FMT_JSON_LEAN );
+    (*data->callback)( data->session, buf, data->callback_user_data );
     evbuffer_free( buf );
+
     tr_bencFree( data->response );
     tr_free( data->response );
     tr_free( data );
@@ -188,6 +186,71 @@ getTorrents( tr_session * session,
     return torrents;
 }
 
+static void
+notifyBatchQueueChange( tr_session * session, tr_torrent ** torrents, int n )
+{
+    int i;
+    for( i=0; i<n; ++i )
+        notify( session, TR_RPC_TORRENT_CHANGED, torrents[i] );
+    notify( session, TR_RPC_SESSION_QUEUE_POSITIONS_CHANGED, NULL );
+}
+
+static const char*
+queueMoveTop( tr_session               * session,
+              tr_benc                  * args_in,
+              tr_benc                  * args_out UNUSED,
+              struct tr_rpc_idle_data  * idle_data UNUSED )
+{
+    int n;
+    tr_torrent ** torrents = getTorrents( session, args_in, &n );
+    tr_torrentsQueueMoveTop( torrents, n );
+    notifyBatchQueueChange( session, torrents, n );
+    tr_free( torrents );
+    return NULL;
+}
+
+static const char*
+queueMoveUp( tr_session               * session,
+             tr_benc                  * args_in,
+             tr_benc                  * args_out UNUSED,
+             struct tr_rpc_idle_data  * idle_data UNUSED )
+{
+    int n;
+    tr_torrent ** torrents = getTorrents( session, args_in, &n );
+    tr_torrentsQueueMoveUp( torrents, n );
+    notifyBatchQueueChange( session, torrents, n );
+    tr_free( torrents );
+    return NULL;
+}
+
+static const char*
+queueMoveDown( tr_session               * session,
+               tr_benc                  * args_in,
+               tr_benc                  * args_out UNUSED,
+               struct tr_rpc_idle_data  * idle_data UNUSED )
+{
+    int n;
+    tr_torrent ** torrents = getTorrents( session, args_in, &n );
+    tr_torrentsQueueMoveDown( torrents, n );
+    notifyBatchQueueChange( session, torrents, n );
+    tr_free( torrents );
+    return NULL;
+}
+
+static const char*
+queueMoveBottom( tr_session               * session,
+                 tr_benc                  * args_in,
+                 tr_benc                  * args_out UNUSED,
+                 struct tr_rpc_idle_data  * idle_data UNUSED )
+{
+    int n;
+    tr_torrent ** torrents = getTorrents( session, args_in, &n );
+    tr_torrentsQueueMoveBottom( torrents, n );
+    notifyBatchQueueChange( session, torrents, n );
+    tr_free( torrents );
+    return NULL;
+}
+
 static const char*
 torrentStart( tr_session               * session,
               tr_benc                  * args_in,
@@ -212,6 +275,30 @@ torrentStart( tr_session               * session,
     return NULL;
 }
 
+static const char*
+torrentStartNow( tr_session               * session,
+                 tr_benc                  * args_in,
+                 tr_benc                  * args_out UNUSED,
+                 struct tr_rpc_idle_data  * idle_data UNUSED )
+{
+    int           i, torrentCount;
+    tr_torrent ** torrents = getTorrents( session, args_in, &torrentCount );
+
+    assert( idle_data == NULL );
+
+    for( i = 0; i < torrentCount; ++i )
+    {
+        tr_torrent * tor = torrents[i];
+        if( !tor->isRunning )
+        {
+            tr_torrentStartNow( tor );
+            notify( session, TR_RPC_TORRENT_STARTED, tor );
+        }
+    }
+    tr_free( torrents );
+    return NULL;
+}
+
 static const char*
 torrentStop( tr_session               * session,
              tr_benc                  * args_in,
@@ -227,9 +314,9 @@ torrentStop( tr_session               * session,
     {
         tr_torrent * tor = torrents[i];
 
-        if( tor->isRunning )
+        if( tor->isRunning || tr_torrentIsQueued( tor ) )
         {
-            tor->isStopping = TRUE;
+            tor->isStopping = true;
             notify( session, TR_RPC_TORRENT_STOPPED, tor );
         }
     }
@@ -245,19 +332,22 @@ torrentRemove( tr_session               * session,
 {
     int i;
     int torrentCount;
+    tr_rpc_callback_type type;
+    bool deleteFlag = false;
     tr_torrent ** torrents = getTorrents( session, args_in, &torrentCount );
 
     assert( idle_data == NULL );
 
+    tr_bencDictFindBool( args_in, "delete-local-data", &deleteFlag );
+    type = deleteFlag ? TR_RPC_TORRENT_TRASHING
+                      : TR_RPC_TORRENT_REMOVING;
+
     for( i=0; i<torrentCount; ++i )
     {
         tr_torrent * tor = torrents[i];
-        const tr_rpc_callback_status status = notify( session, TR_RPC_TORRENT_REMOVING, tor );
-        tr_bool deleteFlag;
-        if( tr_bencDictFindBool( args_in, "delete-local-data", &deleteFlag ) && deleteFlag )
-            tr_torrentDeleteLocalData( tor, NULL );
+        const tr_rpc_callback_status status = notify( session, type, tor );
         if( !( status & TR_RPC_NOREMOVE ) )
-            tr_torrentRemove( tor );
+            tr_torrentRemove( tor, deleteFlag, NULL );
     }
 
     tr_free( torrents );
@@ -444,6 +534,7 @@ addPeers( const tr_torrent * tor,
         tr_bencDictAddBool( d, "isEncrypted", peer->isEncrypted );
         tr_bencDictAddBool( d, "isIncoming", peer->isIncoming );
         tr_bencDictAddBool( d, "isUploadingTo", peer->isUploadingTo );
+        tr_bencDictAddBool( d, "isUTP", peer->isUTP );
         tr_bencDictAddBool( d, "peerIsChoked", peer->peerIsChoked );
         tr_bencDictAddBool( d, "peerIsInterested", peer->peerIsInterested );
         tr_bencDictAddInt ( d, "port", peer->port );
@@ -455,16 +546,18 @@ addPeers( const tr_torrent * tor,
     tr_torrentPeersFree( peers, peerCount );
 }
 
-/* faster-than-strcmp() optimization.  this is kind of clumsy,
+/* faster-than-strcmp() optimization. This is kind of clumsy,
    but addField() was in the profiler's top 10 list, and this
    makes it 4x faster... */
 #define tr_streq(a,alen,b) ((alen+1==sizeof(b)) && !memcmp(a,b,alen))
 
 static void
-addField( const tr_torrent * tor, tr_benc * d, const char * key )
+addField( const tr_torrent * const tor,
+          const tr_info    * const inf,
+          const tr_stat    * const st,
+          tr_benc          * const d,
+          const char       * const key )
 {
-    const tr_info * inf = tr_torrentInfo( tor );
-    const tr_stat * st = tr_torrentStat( (tr_torrent*)tor );
     const size_t keylen = strlen( key );
 
     if( tr_streq( key, keylen, "activityDate" ) )
@@ -517,6 +610,8 @@ addField( const tr_torrent * tor, tr_benc * d, const char * key )
         tr_bencDictAddBool( d, key, st->finished );
     else if( tr_streq( key, keylen, "isPrivate" ) )
         tr_bencDictAddBool( d, key, tr_torrentIsPrivate( tor ) );
+    else if( tr_streq( key, keylen, "isStalled" ) )
+        tr_bencDictAddBool( d, key, st->isStalled );
     else if( tr_streq( key, keylen, "leftUntilDone" ) )
         tr_bencDictAddInt( d, key, st->leftUntilDone );
     else if( tr_streq( key, keylen, "manualAnnounceTime" ) )
@@ -531,7 +626,7 @@ addField( const tr_torrent * tor, tr_benc * d, const char * key )
     else if( tr_streq( key, keylen, "metadataPercentComplete" ) )
         tr_bencDictAddReal( d, key, st->metadataPercentComplete );
     else if( tr_streq( key, keylen, "name" ) )
-        tr_bencDictAddStr( d, key, inf->name );
+        tr_bencDictAddStr( d, key, tr_torrentName( tor ) );
     else if( tr_streq( key, keylen, "percentDone" ) )
         tr_bencDictAddReal( d, key, st->percentDone );
     else if( tr_streq( key, keylen, "peer-limit" ) )
@@ -547,21 +642,22 @@ addField( const tr_torrent * tor, tr_benc * d, const char * key )
         tr_bencDictAddInt( tmp, "fromCache",    f[TR_PEER_FROM_RESUME] );
         tr_bencDictAddInt( tmp, "fromDht",      f[TR_PEER_FROM_DHT] );
         tr_bencDictAddInt( tmp, "fromIncoming", f[TR_PEER_FROM_INCOMING] );
+        tr_bencDictAddInt( tmp, "fromLpd",      f[TR_PEER_FROM_LPD] );
         tr_bencDictAddInt( tmp, "fromLtep",     f[TR_PEER_FROM_LTEP] );
         tr_bencDictAddInt( tmp, "fromPex",      f[TR_PEER_FROM_PEX] );
         tr_bencDictAddInt( tmp, "fromTracker",  f[TR_PEER_FROM_TRACKER] );
     }
     else if( tr_streq( key, keylen, "peersGettingFromUs" ) )
         tr_bencDictAddInt( d, key, st->peersGettingFromUs );
-    else if( tr_streq( key, keylen, "peersKnown" ) )
-        tr_bencDictAddInt( d, key, st->peersKnown );
     else if( tr_streq( key, keylen, "peersSendingToUs" ) )
         tr_bencDictAddInt( d, key, st->peersSendingToUs );
     else if( tr_streq( key, keylen, "pieces" ) ) {
-        const tr_bitfield * pieces = tr_cpPieceBitfield( &tor->completion );
-        char * str = tr_base64_encode( pieces->bits, pieces->byteCount, NULL );
+        size_t byte_count = 0;
+        void * bytes = tr_cpCreatePieceBitfield( &tor->completion, &byte_count );
+        char * str = tr_base64_encode( bytes, byte_count, NULL );
         tr_bencDictAddStr( d, key, str!=NULL ? str : "" );
         tr_free( str );
+        tr_free( bytes );
     }
     else if( tr_streq( key, keylen, "pieceCount" ) )
         tr_bencDictAddInt( d, key, inf->pieceCount );
@@ -574,6 +670,8 @@ addField( const tr_torrent * tor, tr_benc * d, const char * key )
         for( i = 0; i < inf->fileCount; ++i )
             tr_bencListAddInt( p, inf->files[i].priority );
     }
+    else if( tr_streq( key, keylen, "queuePosition" ) )
+        tr_bencDictAddInt( d, key, st->queuePosition );
     else if( tr_streq( key, keylen, "rateDownload" ) )
         tr_bencDictAddInt( d, key, toSpeedBytes( st->pieceDownloadSpeed_KBps ) );
     else if( tr_streq( key, keylen, "rateUpload" ) )
@@ -594,6 +692,10 @@ addField( const tr_torrent * tor, tr_benc * d, const char * key )
         tr_bencDictAddInt( d, key, st->startDate );
     else if( tr_streq( key, keylen, "status" ) )
         tr_bencDictAddInt( d, key, st->activity );
+    else if( tr_streq( key, keylen, "secondsDownloading" ) )
+        tr_bencDictAddInt( d, key, st->secondsDownloading );
+    else if( tr_streq( key, keylen, "secondsSeeding" ) )
+        tr_bencDictAddInt( d, key, st->secondsSeeding );
     else if( tr_streq( key, keylen, "trackers" ) )
         addTrackers( inf, tr_bencDictAddList( d, key, inf->trackerCount ) );
     else if( tr_streq( key, keylen, "trackerStats" ) ) {
@@ -628,19 +730,23 @@ addField( const tr_torrent * tor, tr_benc * d, const char * key )
 }
 
 static void
-addInfo( const tr_torrent * tor,
-         tr_benc *          d,
-         tr_benc *          fields )
+addInfo( const tr_torrent * tor, tr_benc * d, tr_benc * fields )
 {
-    int          i;
-    const int    n = tr_bencListSize( fields );
     const char * str;
+    const int n = tr_bencListSize( fields );
 
     tr_bencInitDict( d, n );
 
-    for( i = 0; i < n; ++i )
-        if( tr_bencGetStr( tr_bencListChild( fields, i ), &str ) )
-            addField( tor, d, str );
+    if( n > 0 )
+    {
+        int i;
+        const tr_info const * inf = tr_torrentInfo( tor );
+        const tr_stat const * st = tr_torrentStat( (tr_torrent*)tor );
+
+        for( i=0; i<n; ++i )
+            if( tr_bencGetStr( tr_bencListChild( fields, i ), &str ) )
+                addField( tor, inf, st, d, str );
+    }
 }
 
 static const char*
@@ -762,17 +868,17 @@ setFileDLs( tr_torrent * tor,
     return errmsg;
 }
 
-static tr_bool
+static bool
 findAnnounceUrl( const tr_tracker_info * t, int n, const char * url, int * pos )
 {
     int i;
-    tr_bool found = FALSE;
+    bool found = false;
 
     for( i=0; i<n; ++i )
     {
         if( !strcmp( t[i].announce, url ) )
         {
-            found = TRUE;
+            found = true;
             if( pos ) *pos = i;
             break;
         }
@@ -816,7 +922,7 @@ addTrackerUrls( tr_torrent * tor, tr_benc * urls )
     int tier;
     tr_benc * val;
     tr_tracker_info * trackers;
-    tr_bool changed = FALSE;
+    bool changed = false;
     const tr_info * inf = tr_torrentInfo( tor );
     const char * errmsg = NULL;
 
@@ -832,13 +938,13 @@ addTrackerUrls( tr_torrent * tor, tr_benc * urls )
         const char * announce = NULL;
 
         if(    tr_bencGetStr( val, &announce )
-            && tr_urlIsValid( announce, -1 )
+            && tr_urlIsValidTracker( announce )
             && !findAnnounceUrl( trackers, n, announce, NULL ) )
         {
             trackers[n].tier = ++tier; /* add a new tier */
             trackers[n].announce = tr_strdup( announce );
             ++n;
-            changed = TRUE;
+            changed = true;
         }
     }
 
@@ -857,7 +963,7 @@ replaceTrackers( tr_torrent * tor, tr_benc * urls )
     int i;
     tr_benc * pair[2];
     tr_tracker_info * trackers;
-    tr_bool changed = FALSE;
+    bool changed = false;
     const tr_info * inf = tr_torrentInfo( tor );
     const int n = inf->trackerCount;
     const char * errmsg = NULL;
@@ -876,13 +982,13 @@ replaceTrackers( tr_torrent * tor, tr_benc * urls )
 
         if(    tr_bencGetInt( pair[0], &pos )
             && tr_bencGetStr( pair[1], &newval )
-            && tr_urlIsValid( newval, -1 )
+            && tr_urlIsValidTracker( newval )
             && pos < n
             && pos >= 0 )
         {
             tr_free( trackers[pos].announce );
             trackers[pos].announce = tr_strdup( newval );
-            changed = TRUE;
+            changed = true;
         }
 
         i += 2;
@@ -907,7 +1013,7 @@ removeTrackers( tr_torrent * tor, tr_benc * ids )
     int * tids;
     tr_benc * val;
     tr_tracker_info * trackers;
-    tr_bool changed = FALSE;
+    bool changed = false;
     const tr_info * inf = tr_torrentInfo( tor );
     const char * errmsg = NULL;
 
@@ -938,7 +1044,7 @@ removeTrackers( tr_torrent * tor, tr_benc * ids )
             continue;
         tr_removeElementFromArray( trackers, tids[t], sizeof( tr_tracker_info ), n-- );
         dup = tids[t];
-        changed = TRUE;
+        changed = true;
     }
 
     if( !changed )
@@ -969,16 +1075,16 @@ torrentSet( tr_session               * session,
         double       d;
         tr_benc *    files;
         tr_benc *    trackers;
-        tr_bool      boolVal;
+        bool         boolVal;
         tr_torrent * tor = torrents[i];
 
         if( tr_bencDictFindInt( args_in, "bandwidthPriority", &tmp ) )
             if( tr_isPriority( tmp ) )
                 tr_torrentSetPriority( tor, tmp );
         if( !errmsg && tr_bencDictFindList( args_in, "files-unwanted", &files ) )
-            errmsg = setFileDLs( tor, FALSE, files );
+            errmsg = setFileDLs( tor, false, files );
         if( !errmsg && tr_bencDictFindList( args_in, "files-wanted", &files ) )
-            errmsg = setFileDLs( tor, TRUE, files );
+            errmsg = setFileDLs( tor, true, files );
         if( tr_bencDictFindInt( args_in, "peer-limit", &tmp ) )
             tr_torrentSetPeerLimit( tor, tmp );
         if( !errmsg &&  tr_bencDictFindList( args_in, "priority-high", &files ) )
@@ -1005,6 +1111,8 @@ torrentSet( tr_session               * session,
             tr_torrentSetRatioLimit( tor, d );
         if( tr_bencDictFindInt( args_in, "seedRatioMode", &tmp ) )
             tr_torrentSetRatioMode( tor, tmp );
+        if( tr_bencDictFindInt( args_in, "queuePosition", &tmp ) )
+            tr_torrentSetQueuePosition( tor, tmp );
         if( !errmsg && tr_bencDictFindList( args_in, "trackerAdd", &trackers ) )
             errmsg = addTrackerUrls( tor, trackers );
         if( !errmsg && tr_bencDictFindList( args_in, "trackerRemove", &trackers ) )
@@ -1035,7 +1143,7 @@ torrentSetLocation( tr_session               * session,
     }
     else
     {
-        tr_bool move = FALSE;
+        bool move = false;
         int i, torrentCount;
         tr_torrent ** torrents = getTorrents( session, args_in, &torrentCount );
 
@@ -1060,6 +1168,8 @@ torrentSetLocation( tr_session               * session,
 
 static void
 portTested( tr_session       * session UNUSED,
+            bool               did_connect UNUSED,
+            bool               did_timeout UNUSED,
             long               response_code,
             const void       * response,
             size_t             response_byte_count,
@@ -1075,7 +1185,7 @@ portTested( tr_session       * session UNUSED,
     }
     else /* success */
     {
-        const tr_bool isOpen = response_byte_count && *(char*)response == '1';
+        const bool isOpen = response_byte_count && *(char*)response == '1';
         tr_bencDictAddBool( data->args_out, "port-is-open", isOpen );
         tr_snprintf( result, sizeof( result ), "success" );
     }
@@ -1091,7 +1201,7 @@ portTest( tr_session               * session,
 {
     const int port = tr_sessionGetPeerPort( session );
     char * url = tr_strdup_printf( "http://portcheck.transmissionbt.com/%d", port );
-    tr_webRun( session, url, NULL, portTested, idle_data );
+    tr_webRun( session, url, NULL, NULL, portTested, idle_data );
     tr_free( url );
     return NULL;
 }
@@ -1102,6 +1212,8 @@ portTest( tr_session               * session,
 
 static void
 gotNewBlocklist( tr_session       * session,
+                 bool               did_connect UNUSED,
+                 bool               did_timeout UNUSED,
                  long               response_code,
                  const void       * response,
                  size_t             response_byte_count,
@@ -1110,6 +1222,8 @@ gotNewBlocklist( tr_session       * session,
     char result[1024];
     struct tr_rpc_idle_data * data = user_data;
 
+    *result = '\0';
+
     if( response_code != 200 )
     {
         tr_snprintf( result, sizeof( result ), "gotNewBlocklist: http error %ld: %s",
@@ -1118,66 +1232,59 @@ gotNewBlocklist( tr_session       * session,
     else /* successfully fetched the blocklist... */
     {
         int fd;
+        int err;
+        char * filename;
+        z_stream stream;
         const char * configDir = tr_sessionGetConfigDir( session );
-        char * filename = tr_buildPath( configDir, "blocklist.tmp", NULL );
+        const size_t buflen = 1024 * 128; /* 128 KiB buffer */
+        uint8_t * buf = tr_valloc( buflen );
 
-        errno = 0;
+        /* this is an odd Magic Number required by zlib to enable gz support.
+           See zlib's inflateInit2() documentation for a full description */
+        const int windowBits = 15 + 32;
 
-        if( !errno ) {
-            fd = tr_open_file_for_writing( filename );
-            if( fd < 0 )
-                tr_snprintf( result, sizeof( result ), _( "Couldn't save file \"%1$s\": %2$s" ), filename, tr_strerror( errno ) );
-        }
+        stream.zalloc = (alloc_func) Z_NULL;
+        stream.zfree = (free_func) Z_NULL;
+        stream.opaque = (voidpf) Z_NULL;
+        stream.next_in = (void*) response;
+        stream.avail_in = response_byte_count;
+        inflateInit2( &stream, windowBits );
 
-        if( !errno ) {
-            const char * buf = response;
-            size_t buflen = response_byte_count;
-            while( buflen > 0 ) {
-                int n = write( fd, buf, buflen );
-                if( n < 0 ) {
+        filename = tr_buildPath( configDir, "blocklist.tmp", NULL );
+        fd = tr_open_file_for_writing( filename );
+        if( fd < 0 )
+            tr_snprintf( result, sizeof( result ), _( "Couldn't save file \"%1$s\": %2$s" ), filename, tr_strerror( errno ) );
+
+        for( ;; )
+        {
+            stream.next_out = (void*) buf;
+            stream.avail_out = buflen;
+            err = inflate( &stream, Z_NO_FLUSH );
+
+            if( stream.avail_out < buflen ) {
+                const int e = write( fd, buf, buflen - stream.avail_out );
+                if( e < 0 ) {
                     tr_snprintf( result, sizeof( result ), _( "Couldn't save file \"%1$s\": %2$s" ), filename, tr_strerror( errno ) );
                     break;
                 }
-                buf += n;
-                buflen -= n;
-            }
-            tr_close_file( fd );
-        }
-
-#ifdef HAVE_ZLIB
-        if( !errno )
-        {
-            char * filename2 = tr_buildPath( configDir, "blocklist.txt.tmp", NULL );
-            fd = tr_open_file_for_writing( filename2 );
-            if( fd < 0 )
-                tr_snprintf( result, sizeof( result ), _( "Couldn't save file \"%1$s\": %2$s" ), filename2, tr_strerror( errno ) );
-            else {
-                gzFile gzf = gzopen( filename, "r" );
-                if( gzf ) {
-                    const size_t buflen = 1024 * 128; /* 128 KiB buffer */
-                    uint8_t * buf = tr_valloc( buflen );
-                    for( ;; ) {
-                        int n = gzread( gzf, buf, buflen );
-                        if( n < 0 ) /* error */
-                            tr_snprintf( result, sizeof( result ), _( "Error reading \"%1$s\": %2$s" ), filename, gzerror( gzf, NULL ) );
-                        if( n < 1 ) /* error or EOF */
-                            break;
-                        if( write( fd, buf, n ) < 0 )
-                            tr_snprintf( result, sizeof( result ), _( "Couldn't save file \"%1$s\": %2$s" ), filename2, tr_strerror( errno ) );
-                    }
-                    tr_free( buf );
-                    gzclose( gzf );
-                }
-                tr_close_file( fd );
             }
 
-            unlink( filename );
-            tr_free( filename );
-            filename = filename2;
+            if( err != Z_OK ) {
+                if( ( err != Z_STREAM_END ) && ( err != Z_DATA_ERROR ) )
+                    tr_snprintf( result, sizeof( result ), _( "Error uncompressing blocklist: %s (%d)" ), zError( err ), err );
+                break;
+            }
         }
-#endif
 
-        if( !errno ) {
+        inflateEnd( &stream );
+
+        if( err == Z_DATA_ERROR ) /* couldn't inflate it... it's probably already uncompressed */
+            if( write( fd, response, response_byte_count ) < 0 )
+                tr_snprintf( result, sizeof( result ), _( "Couldn't save file \"%1$s\": %2$s" ), filename, tr_strerror( errno ) );
+
+        if( *result )
+            tr_err( "%s", result );
+        else {
             /* feed it to the session and give the client a response */
             const int rule_count = tr_blocklistSetContent( session, filename );
             tr_bencDictAddInt( data->args_out, "blocklist-size", rule_count );
@@ -1186,6 +1293,7 @@ gotNewBlocklist( tr_session       * session,
 
         unlink( filename );
         tr_free( filename );
+        tr_free( buf );
     }
 
     tr_idle_function_done( data, result );
@@ -1197,7 +1305,7 @@ blocklistUpdate( tr_session               * session,
                  tr_benc                  * args_out UNUSED,
                  struct tr_rpc_idle_data  * idle_data )
 {
-    tr_webRun( session, session->blocklist_url, NULL, gotNewBlocklist, idle_data );
+    tr_webRun( session, session->blocklist_url, NULL, NULL, gotNewBlocklist, idle_data );
     return NULL;
 }
 
@@ -1246,6 +1354,8 @@ struct add_torrent_idle_data
 
 static void
 gotMetadataFromURL( tr_session       * session UNUSED,
+                    bool               did_connect UNUSED,
+                    bool               did_timeout UNUSED,
                     long               response_code,
                     const void       * response,
                     size_t             response_byte_count,
@@ -1272,11 +1382,11 @@ gotMetadataFromURL( tr_session       * session UNUSED,
     tr_free( data );
 }
 
-static tr_bool
+static bool
 isCurlURL( const char * filename )
 {
     if( filename == NULL )
-        return FALSE;
+        return false;
 
     return !strncmp( filename, "ftp://", 6 ) ||
            !strncmp( filename, "http://", 7 ) ||
@@ -1319,13 +1429,16 @@ torrentAdd( tr_session               * session,
     else
     {
         int64_t      i;
-        tr_bool      boolVal;
-        const char * str;
+        bool         boolVal;
         tr_benc    * l;
+        const char * str;
+        const char * cookies = NULL;
         tr_ctor    * ctor = tr_ctorNew( session );
 
         /* set the optional arguments */
 
+        tr_bencDictFindStr( args_in, "cookies", &cookies );
+
         if( tr_bencDictFindStr( args_in, TR_PREFS_KEY_DOWNLOAD_DIR, &str ) )
             tr_ctorSetDownloadDir( ctor, TR_FORCE, str );
 
@@ -1341,13 +1454,13 @@ torrentAdd( tr_session               * session,
         if( tr_bencDictFindList( args_in, "files-unwanted", &l ) ) {
             tr_file_index_t fileCount;
             tr_file_index_t * files = fileListFromList( l, &fileCount );
-            tr_ctorSetFilesWanted( ctor, files, fileCount, FALSE );
+            tr_ctorSetFilesWanted( ctor, files, fileCount, false );
             tr_free( files );
         }
         if( tr_bencDictFindList( args_in, "files-wanted", &l ) ) {
             tr_file_index_t fileCount;
             tr_file_index_t * files = fileListFromList( l, &fileCount );
-            tr_ctorSetFilesWanted( ctor, files, fileCount, TRUE );
+            tr_ctorSetFilesWanted( ctor, files, fileCount, true );
             tr_free( files );
         }
 
@@ -1377,7 +1490,7 @@ torrentAdd( tr_session               * session,
             struct add_torrent_idle_data * d = tr_new0( struct add_torrent_idle_data, 1 );
             d->data = idle_data;
             d->ctor = ctor;
-            tr_webRun( session, filename, NULL, gotMetadataFromURL, d );
+            tr_webRun( session, filename, NULL, cookies, gotMetadataFromURL, d );
         }
         else
         {
@@ -1421,7 +1534,7 @@ sessionSet( tr_session               * session,
 {
     int64_t      i;
     double       d;
-    tr_bool      boolVal;
+    bool         boolVal;
     const char * str;
 
     assert( idle_data == NULL );
@@ -1448,6 +1561,14 @@ sessionSet( tr_session               * session,
         tr_blocklistSetURL( session, str );
     if( tr_bencDictFindStr( args_in, TR_PREFS_KEY_DOWNLOAD_DIR, &str ) )
         tr_sessionSetDownloadDir( session, str );
+    if( tr_bencDictFindInt( args_in, TR_PREFS_KEY_QUEUE_STALLED_MINUTES, &i ) )
+        tr_sessionSetQueueStalledMinutes( session, i );
+    if( tr_bencDictFindBool( args_in, TR_PREFS_KEY_QUEUE_STALLED_ENABLED, &boolVal ) )
+        tr_sessionSetQueueStalledEnabled( session, boolVal );
+    if( tr_bencDictFindInt( args_in, TR_PREFS_KEY_DOWNLOAD_QUEUE_SIZE, &i ) )
+        tr_sessionSetQueueSize( session, TR_DOWN, i );
+    if( tr_bencDictFindBool( args_in, TR_PREFS_KEY_DOWNLOAD_QUEUE_ENABLED, &boolVal ) )
+        tr_sessionSetQueueEnabled ( session, TR_DOWN, boolVal );
     if( tr_bencDictFindStr( args_in, TR_PREFS_KEY_INCOMPLETE_DIR, &str ) )
         tr_sessionSetIncompleteDir( session, str );
     if( tr_bencDictFindBool( args_in, TR_PREFS_KEY_INCOMPLETE_DIR_ENABLED, &boolVal ) )
@@ -1460,6 +1581,8 @@ sessionSet( tr_session               * session,
         tr_sessionSetPexEnabled( session, boolVal );
     if( tr_bencDictFindBool( args_in, TR_PREFS_KEY_DHT_ENABLED, &boolVal ) )
         tr_sessionSetDHTEnabled( session, boolVal );
+    if( tr_bencDictFindBool( args_in, TR_PREFS_KEY_UTP_ENABLED, &boolVal ) )
+        tr_sessionSetUTPEnabled( session, boolVal );
     if( tr_bencDictFindBool( args_in, TR_PREFS_KEY_LPD_ENABLED, &boolVal ) )
         tr_sessionSetLPDEnabled( session, boolVal );
     if( tr_bencDictFindBool( args_in, TR_PREFS_KEY_PEER_PORT_RANDOM_ON_START, &boolVal ) )
@@ -1480,6 +1603,10 @@ sessionSet( tr_session               * session,
         tr_sessionSetIdleLimited( session, boolVal );
     if( tr_bencDictFindBool( args_in, TR_PREFS_KEY_START, &boolVal ) )
         tr_sessionSetPaused( session, !boolVal );
+    if( tr_bencDictFindBool( args_in, TR_PREFS_KEY_SEED_QUEUE_ENABLED, &boolVal ) )
+        tr_sessionSetQueueEnabled ( session, TR_UP, boolVal );
+    if( tr_bencDictFindInt( args_in, TR_PREFS_KEY_SEED_QUEUE_SIZE, &i ) )
+        tr_sessionSetQueueSize( session, TR_UP, i );
     if( tr_bencDictFindStr( args_in, TR_PREFS_KEY_SCRIPT_TORRENT_DONE_FILENAME, &str ) )
         tr_sessionSetTorrentDoneScript( session, str );
     if( tr_bencDictFindBool( args_in, TR_PREFS_KEY_SCRIPT_TORRENT_DONE_ENABLED, &boolVal ) )
@@ -1578,11 +1705,15 @@ sessionGet( tr_session               * s,
     tr_bencDictAddInt ( d, "blocklist-size", tr_blocklistGetRuleCount( s ) );
     tr_bencDictAddStr ( d, "config-dir", tr_sessionGetConfigDir( s ) );
     tr_bencDictAddStr ( d, TR_PREFS_KEY_DOWNLOAD_DIR, tr_sessionGetDownloadDir( s ) );
+    tr_bencDictAddBool( d, TR_PREFS_KEY_DOWNLOAD_QUEUE_ENABLED, tr_sessionGetQueueEnabled( s, TR_DOWN ) );
+    tr_bencDictAddInt ( d, TR_PREFS_KEY_DOWNLOAD_QUEUE_SIZE, tr_sessionGetQueueSize( s, TR_DOWN ) );
+    tr_bencDictAddInt ( d, "download-dir-free-space",  tr_sessionGetDownloadDirFreeSpace( s ) );
     tr_bencDictAddInt ( d, TR_PREFS_KEY_PEER_LIMIT_GLOBAL, tr_sessionGetPeerLimit( s ) );
     tr_bencDictAddInt ( d, TR_PREFS_KEY_PEER_LIMIT_TORRENT, tr_sessionGetPeerLimitPerTorrent( s ) );
     tr_bencDictAddStr ( d, TR_PREFS_KEY_INCOMPLETE_DIR, tr_sessionGetIncompleteDir( s ) );
     tr_bencDictAddBool( d, TR_PREFS_KEY_INCOMPLETE_DIR_ENABLED, tr_sessionIsIncompleteDirEnabled( s ) );
     tr_bencDictAddBool( d, TR_PREFS_KEY_PEX_ENABLED, tr_sessionIsPexEnabled( s ) );
+    tr_bencDictAddBool( d, TR_PREFS_KEY_UTP_ENABLED, tr_sessionIsUTPEnabled( s ) );
     tr_bencDictAddBool( d, TR_PREFS_KEY_DHT_ENABLED, tr_sessionIsDHTEnabled( s ) );
     tr_bencDictAddBool( d, TR_PREFS_KEY_LPD_ENABLED, tr_sessionIsLPDEnabled( s ) );
     tr_bencDictAddInt ( d, TR_PREFS_KEY_PEER_PORT, tr_sessionGetPeerPort( s ) );
@@ -1595,6 +1726,8 @@ sessionGet( tr_session               * s,
     tr_bencDictAddBool( d, "seedRatioLimited", tr_sessionIsRatioLimited( s ) );
     tr_bencDictAddInt ( d, TR_PREFS_KEY_IDLE_LIMIT, tr_sessionGetIdleLimit( s ) );
     tr_bencDictAddBool( d, TR_PREFS_KEY_IDLE_LIMIT_ENABLED, tr_sessionIsIdleLimited( s ) );
+    tr_bencDictAddBool( d, TR_PREFS_KEY_SEED_QUEUE_ENABLED, tr_sessionGetQueueEnabled( s, TR_UP ) );
+    tr_bencDictAddInt ( d, TR_PREFS_KEY_SEED_QUEUE_SIZE, tr_sessionGetQueueSize( s, TR_UP ) );
     tr_bencDictAddBool( d, TR_PREFS_KEY_START, !tr_sessionGetPaused( s ) );
     tr_bencDictAddBool( d, TR_PREFS_KEY_TRASH_ORIGINAL, tr_sessionGetDeleteSource( s ) );
     tr_bencDictAddInt ( d, TR_PREFS_KEY_USPEED_KBps, tr_sessionGetSpeedLimit_KBps( s, TR_UP ) );
@@ -1603,6 +1736,8 @@ sessionGet( tr_session               * s,
     tr_bencDictAddBool( d, TR_PREFS_KEY_DSPEED_ENABLED, tr_sessionIsSpeedLimited( s, TR_DOWN ) );
     tr_bencDictAddStr ( d, TR_PREFS_KEY_SCRIPT_TORRENT_DONE_FILENAME, tr_sessionGetTorrentDoneScript( s ) );
     tr_bencDictAddBool( d, TR_PREFS_KEY_SCRIPT_TORRENT_DONE_ENABLED, tr_sessionIsTorrentDoneScriptEnabled( s ) );
+    tr_bencDictAddInt ( d, TR_PREFS_KEY_QUEUE_STALLED_MINUTES, tr_sessionGetQueueStalledMinutes( s ) );
+    tr_bencDictAddBool( d, TR_PREFS_KEY_QUEUE_STALLED_ENABLED, tr_sessionGetQueueStalledEnabled( s ) );
     tr_formatter_get_units( tr_bencDictAddDict( d, "units", 0 ) );
     tr_bencDictAddStr ( d, "version", LONG_VERSION_STRING );
     switch( tr_sessionGetEncryption( s ) ) {
@@ -1619,37 +1754,56 @@ sessionGet( tr_session               * s,
 ****
 ***/
 
+static const char*
+sessionClose( tr_session               * session,
+              tr_benc                  * args_in UNUSED,
+              tr_benc                  * args_out UNUSED,
+              struct tr_rpc_idle_data  * idle_data UNUSED )
+{
+    notify( session, TR_RPC_SESSION_CLOSE, NULL );
+    return NULL;
+}
+
+/***
+****
+***/
+
 typedef const char* ( *handler )( tr_session*, tr_benc*, tr_benc*, struct tr_rpc_idle_data * );
 
 static struct method
 {
     const char *  name;
-    tr_bool       immediate;
+    bool          immediate;
     handler       func;
 }
 methods[] =
 {
-    { "port-test",             FALSE, portTest            },
-    { "blocklist-update",      FALSE, blocklistUpdate     },
-    { "session-get",           TRUE,  sessionGet          },
-    { "session-set",           TRUE,  sessionSet          },
-    { "session-stats",         TRUE,  sessionStats        },
-    { "torrent-add",           FALSE, torrentAdd          },
-    { "torrent-get",           TRUE,  torrentGet          },
-    { "torrent-remove",        TRUE,  torrentRemove       },
-    { "torrent-set",           TRUE,  torrentSet          },
-    { "torrent-set-location",  TRUE,  torrentSetLocation  },
-    { "torrent-start",         TRUE,  torrentStart        },
-    { "torrent-stop",          TRUE,  torrentStop         },
-    { "torrent-verify",        TRUE,  torrentVerify       },
-    { "torrent-reannounce",    TRUE,  torrentReannounce   }
+    { "port-test",             false, portTest            },
+    { "blocklist-update",      false, blocklistUpdate     },
+    { "session-close",         true,  sessionClose        },
+    { "session-get",           true,  sessionGet          },
+    { "session-set",           true,  sessionSet          },
+    { "session-stats",         true,  sessionStats        },
+    { "torrent-add",           false, torrentAdd          },
+    { "torrent-get",           true,  torrentGet          },
+    { "torrent-remove",        true,  torrentRemove       },
+    { "torrent-set",           true,  torrentSet          },
+    { "torrent-set-location",  true,  torrentSetLocation  },
+    { "torrent-start",         true,  torrentStart        },
+    { "torrent-start-now",     true,  torrentStartNow     },
+    { "torrent-stop",          true,  torrentStop         },
+    { "torrent-verify",        true,  torrentVerify       },
+    { "torrent-reannounce",    true,  torrentReannounce   },
+    { "queue-move-top",        true,  queueMoveTop        },
+    { "queue-move-up",         true,  queueMoveUp         },
+    { "queue-move-down",       true,  queueMoveDown       },
+    { "queue-move-bottom",     true,  queueMoveBottom     }
 };
 
 static void
-noop_response_callback( tr_session * session UNUSED,
-                        const char * response UNUSED,
-                        size_t       response_len UNUSED,
-                        void       * user_data UNUSED )
+noop_response_callback( tr_session       * session UNUSED,
+                        struct evbuffer  * response UNUSED,
+                        void             * user_data UNUSED )
 {
 }
 
@@ -1684,18 +1838,18 @@ request_exec( tr_session             * session,
     {
         int64_t tag;
         tr_benc response;
-        struct evbuffer * buf = evbuffer_new( );
+        struct evbuffer * buf;
 
         tr_bencInitDict( &response, 3 );
         tr_bencDictAddDict( &response, "arguments", 0 );
         tr_bencDictAddStr( &response, "result", result );
         if( tr_bencDictFindInt( request, "tag", &tag ) )
             tr_bencDictAddInt( &response, "tag", tag );
-        tr_bencToBuf( &response, TR_FMT_JSON_LEAN, buf );
-        (*callback)( session, (const char*)EVBUFFER_DATA(buf),
-                     EVBUFFER_LENGTH( buf ), callback_user_data );
 
+        buf = tr_bencToBuf( &response, TR_FMT_JSON_LEAN );
+        (*callback)( session, buf, callback_user_data );
         evbuffer_free( buf );
+
         tr_bencFree( &response );
     }
     else if( methods[i].immediate )
@@ -1703,7 +1857,7 @@ request_exec( tr_session             * session,
         int64_t tag;
         tr_benc response;
         tr_benc * args_out;
-        struct evbuffer * buf = evbuffer_new( );
+        struct evbuffer * buf;
 
         tr_bencInitDict( &response, 3 );
         args_out = tr_bencDictAddDict( &response, "arguments", 0 );
@@ -1713,11 +1867,11 @@ request_exec( tr_session             * session,
         tr_bencDictAddStr( &response, "result", result );
         if( tr_bencDictFindInt( request, "tag", &tag ) )
             tr_bencDictAddInt( &response, "tag", tag );
-        tr_bencToBuf( &response, TR_FMT_JSON_LEAN, buf );
-        (*callback)( session, (const char*)EVBUFFER_DATA(buf),
-                     EVBUFFER_LENGTH(buf), callback_user_data );
 
+        buf = tr_bencToBuf( &response, TR_FMT_JSON_LEAN );
+        (*callback)( session, buf, callback_user_data );
         evbuffer_free( buf );
+
         tr_bencFree( &response );
     }
     else
