@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: AddMagnetWindowController.m 11090 2010-08-01 20:18:44Z charles $
+ * $Id: AddMagnetWindowController.m 12612 2011-08-02 12:43:26Z livings124 $
  *
- * Copyright (c) 2010 Transmission authors and contributors
+ * Copyright (c) 2010-2011 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -199,9 +199,11 @@
 
 - (void) confirmAdd
 {
-    [fTorrent setWaitToStart: [fStartCheck state] == NSOnState];
     [fTorrent setGroupValue: fGroupValue];
     
+    if ([fStartCheck state] == NSOnState)
+        [fTorrent startTransfer];
+    
     [self close];
     [fController askOpenMagnetConfirmed: self add: YES]; //ensure last, since it releases this controller
 }
