@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: Controller.h 11178 2010-08-14 19:44:43Z livings124 $
+ * $Id: Controller.h 12772 2011-08-27 23:54:10Z livings124 $
  *
- * Copyright (c) 2005-2010 Transmission authors and contributors
+ * Copyright (c) 2005-2011 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -31,13 +31,14 @@
 @class AddWindowController;
 @class Badger;
 @class DragOverlayWindow;
-@class FilterButton;
+@class FilterBarController;
 @class InfoWindowController;
 @class MessageWindowController;
 @class PrefsController;
-@class StatusBarView;
+@class StatusBarController;
 @class Torrent;
 @class TorrentTableView;
+@class URLSheetWindowController;
 
 typedef enum
 {
@@ -49,7 +50,7 @@ typedef enum
 } addType;
 
 #warning uncomment
-@interface Controller : NSObject <GrowlApplicationBridgeDelegate> //, QLPreviewPanelDataSource, QLPreviewPanelDelegate>
+@interface Controller : NSObject <GrowlApplicationBridgeDelegate, NSPopoverDelegate, NSSoundDelegate, NSToolbarDelegate, NSWindowDelegate> //, QLPreviewPanelDataSource, QLPreviewPanelDelegate>
 {
     tr_session                      * fLib;
     
@@ -69,22 +70,18 @@ typedef enum
     NSTimer                         * fTimer;
     
     IBOutlet NSMenuItem             * fOpenIgnoreDownloadFolder;
-    IBOutlet NSButton               * fActionButton, * fSpeedLimitButton;
+    IBOutlet NSButton               * fActionButton, * fSpeedLimitButton, * fClearCompletedButton;
     IBOutlet NSTextField            * fTotalTorrentsField;
     
-    IBOutlet StatusBarView          * fStatusBar;
-    IBOutlet NSButton               * fStatusButton;
-    IBOutlet NSTextField            * fTotalDLField, * fTotalULField;
-    IBOutlet NSImageView            * fTotalDLImageView;
+    StatusBarController             * fStatusBar;
     
-    IBOutlet StatusBarView          * fFilterBar;
-    IBOutlet FilterButton           * fNoFilterButton, * fActiveFilterButton, * fDownloadFilterButton,
-                                    * fSeedFilterButton, * fPauseFilterButton;
-    IBOutlet NSSearchField          * fSearchFilterField;
-    IBOutlet NSMenuItem             * fNextFilterItem, * fPrevFilterItem;
+    FilterBarController             * fFilterBar;
+    IBOutlet NSMenuItem             * fNextFilterItem;
                                 
     IBOutlet NSMenuItem             * fNextInfoTabItem, * fPrevInfoTabItem;
     
+    IBOutlet NSMenu                 * fActionMenu;
+    
     IBOutlet NSMenu                 * fUploadMenu, * fDownloadMenu;
     IBOutlet NSMenuItem             * fUploadLimitItem, * fUploadNoLimitItem,
                                     * fDownloadLimitItem, * fDownloadNoLimitItem;
@@ -92,28 +89,22 @@ typedef enum
     IBOutlet NSMenu                 * fRatioStopMenu;
     IBOutlet NSMenuItem             * fCheckRatioItem, * fNoCheckRatioItem;
     
-    IBOutlet NSMenu                 * fGroupsSetMenu, * fGroupsSetContextMenu, * fGroupFilterMenu;
-    IBOutlet NSPopUpButton          * fGroupsButton;
-    
-    IBOutlet NSWindow               * fURLSheetWindow;
-    IBOutlet NSTextField            * fURLSheetTextField;
-    IBOutlet NSButton               * fURLSheetOpenButton;
+    IBOutlet NSMenu                 * fGroupsSetMenu, * fGroupsSetContextMenu;
     
     #warning change to QLPreviewPanel
     id                              fPreviewPanel;
     BOOL                            fQuitting;
-    
-    BOOL                            fUpdateInProgress;
+    BOOL                            fQuitRequested;
     BOOL                            fPauseOnLaunch;
     
     Badger                          * fBadger;
-    IBOutlet NSMenu                 * fDockMenu;
     
     NSMutableArray                  * fAutoImportedNames;
     NSTimer                         * fAutoImportTimer;
     
     NSMutableDictionary             * fPendingTorrentDownloads;
     
+    BOOL                            fGlobalPopoverShown;
     BOOL                            fSoundPlaying;
 }
 
@@ -132,10 +123,9 @@ typedef enum
 - (void) duplicateOpenAlert: (NSString *) name;
 - (void) duplicateOpenMagnetAlert: (NSString *) address transferName: (NSString *) name;
 
-- (void) openURL:               (NSString *) urlString;
-- (void) openURLEndSheet:       (id) sender;
-- (void) openURLCancelEndSheet: (id) sender;
-- (void) openURLShowSheet:      (id) sender;
+- (void) openURL: (NSString *) urlString;
+- (void) openURLShowSheet: (id) sender;
+- (void) urlSheetDidEnd: (URLSheetWindowController *) controller url: (NSString *) urlString returnCode: (NSInteger) returnCode;
 
 - (void) quitSheetDidEnd: (NSWindow *) sheet returnCode: (NSInteger) returnCode contextInfo: (void *) contextInfo;
 
@@ -160,6 +150,8 @@ typedef enum
 - (void) removeNoDelete:                (id) sender;
 - (void) removeDeleteData:              (id) sender;
 
+- (void) clearCompleted: (id) sender;
+
 - (void) moveDataFilesSelected: (id) sender;
 - (void) moveDataFiles: (NSArray *) torrents;
 - (void) moveDataFileChoiceClosed: (NSOpenPanel *) panel returnCode: (NSInteger) code contextInfo: (NSArray *) torrents;
@@ -188,22 +180,17 @@ typedef enum
 - (void) showStatsWindow: (id) sender;
 
 - (void) updateUI;
+- (void) fullUpdateUI;
 
-- (void) resizeStatusButton;
 - (void) setBottomCountText: (BOOL) filtering;
 
-- (void) updateSpeedFieldsToolTips;
-
-- (void) updateTorrentsInQueue;
-- (NSUInteger) numToStartFromQueue: (BOOL) downloadQueue;
-
 - (void) torrentFinishedDownloading: (NSNotification *) notification;
 - (void) torrentRestartedDownloading: (NSNotification *) notification;
 - (void) torrentFinishedSeeding: (NSNotification *) notification;
 
 - (void) updateTorrentHistory;
 
-- (void) applyFilter: (id) sender;
+- (void) applyFilter;
 
 - (void) sortTorrents;
 - (void) sortTorrentsIgnoreSelected;
@@ -211,16 +198,11 @@ typedef enum
 - (void) setSortByGroup: (id) sender;
 - (void) setSortReverse: (id) sender;
 
-- (void) setFilter: (id) sender;
-- (void) setFilterSearchType: (id) sender;
 - (void) switchFilter: (id) sender;
 
-- (void) setStatusLabel: (id) sender;
+- (IBAction) showGlobalPopover: (id) sender;
 
 - (void) setGroup: (id) sender; //used by delegate-generated menu items
-- (void) setGroupFilter: (id) sender;
-- (void) updateGroupsFilterButton;
-- (void) updateGroupsFilters: (NSNotification *) notification;
 
 - (void) toggleSpeedLimit: (id) sender;
 - (void) speedLimitChanged: (id) sender;
@@ -257,8 +239,9 @@ typedef enum
 
 - (void) setWindowSizeToFit;
 - (NSRect) sizedWindowFrame;
-
-- (void) resizeFilterBar;
+- (void) updateForAutoSize;
+- (void) setWindowMinMaxToCurrent;
+- (CGFloat) minWindowContentSizeAllowed;
 
 - (void) updateForExpandCollape;
 
@@ -274,8 +257,10 @@ typedef enum
 - (void) rpcCallback: (tr_rpc_callback_type) type forTorrentStruct: (struct tr_torrent *) torrentStruct;
 - (void) rpcAddTorrentStruct: (NSValue *) torrentStructPtr;
 - (void) rpcRemoveTorrent: (Torrent *) torrent;
+- (void) rpcRemoveTorrentDeleteData: (Torrent *) torrent;
 - (void) rpcStartedStoppedTorrent: (Torrent *) torrent;
 - (void) rpcChangedTorrent: (Torrent *) torrent;
 - (void) rpcMovedTorrent: (Torrent *) torrent;
+- (void) rpcUpdateQueue;
 
 @end
