@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: FileListNode.h 11348 2010-10-23 20:20:31Z livings124 $
+ * $Id: FileListNode.h 12483 2011-05-31 22:26:04Z livings124 $
  *
- * Copyright (c) 2008-2010 Transmission authors and contributors
+ * Copyright (c) 2008-2011 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -24,6 +24,8 @@
 
 #import <Cocoa/Cocoa.h>
 
+@class Torrent;
+
 @interface FileListNode : NSObject <NSCopying>
 {
     NSString * fName, * fPath;
@@ -34,14 +36,18 @@
     NSImage * fIcon;
     
     NSMutableArray * fChildren;
+    
+    Torrent * fTorrent;
 }
 
-- (id) initWithFolderName: (NSString *) name path: (NSString *) path;
-- (id) initWithFileName: (NSString *) name path: (NSString *) path size: (uint64_t) size index: (NSUInteger) index;
+- (id) initWithFolderName: (NSString *) name path: (NSString *) path torrent: (Torrent *) torrent;
+- (id) initWithFileName: (NSString *) name path: (NSString *) path size: (uint64_t) size index: (NSUInteger) index torrent: (Torrent *) torrent;
 
 - (void) insertChild: (FileListNode *) child;
 - (void) insertIndex: (NSUInteger) index withSize: (uint64_t) size;
 
+- (NSString *) description;
+
 - (BOOL) isFolder;
 - (NSString *) name;
 - (NSString *) path;
@@ -52,4 +58,6 @@
 
 - (NSMutableArray *) children;
 
+- (Torrent *) torrent;
+
 @end
