@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: InfoOptionsViewController.m 11617 2011-01-01 20:42:14Z livings124 $
+ * $Id: InfoOptionsViewController.m 12489 2011-06-04 13:48:56Z livings124 $
  *
  * Copyright (c) 2010-2011 Transmission authors and contributors
  *
@@ -23,6 +23,7 @@
  *****************************************************************************/
 
 #import "InfoOptionsViewController.h"
+#import "NSStringAdditions.h"
 #import "Torrent.h"
 
 #define OPTION_POPUP_GLOBAL 0
@@ -38,6 +39,8 @@
 @interface InfoOptionsViewController (Private)
 
 - (void) setupInfo;
+- (void) setGlobalLabels;
+- (void) updateOptionsNotification: (NSNotification *) notification;
 
 @end
 
@@ -53,8 +56,18 @@
     return self;
 }
 
+- (void) awakeFromNib
+{
+    [self setGlobalLabels];
+    
+    [[NSNotificationCenter defaultCenter] addObserver: self selector: @selector(setGlobalLabels) name: @"UpdateGlobalOptions" object: nil];
+    [[NSNotificationCenter defaultCenter] addObserver: self selector: @selector(updateOptionsNotification:) name: @"UpdateOptionsNotification" object: nil];
+}
+
 - (void) dealloc
 {
+    [[NSNotificationCenter defaultCenter] removeObserver: self];
+    
     [fTorrents release];
     
     [super dealloc];
@@ -182,6 +195,8 @@
     else
         [fRatioLimitField setStringValue: @""];
     
+    [fRatioLimitGlobalLabel setHidden: checkRatio != TR_RATIOLIMIT_GLOBAL];
+    
     //set idle view
     if (checkIdle == TR_IDLELIMIT_SINGLE)
         index = OPTION_POPUP_LIMIT;
@@ -201,6 +216,8 @@
         [fIdleLimitField setStringValue: @""];
     [fIdleLimitLabel setHidden: checkIdle != TR_IDLELIMIT_SINGLE];
     
+    [fIdleLimitGlobalLabel setHidden: checkIdle != TR_IDLELIMIT_GLOBAL];
+    
     //get priority info
     enumerator = [fTorrents objectEnumerator];
     torrent = [enumerator nextObject]; //first torrent
@@ -270,6 +287,8 @@
     
     NSTextField * label = upload ? fUploadLimitLabel : fDownloadLimitLabel;
     [label setEnabled: limit];
+    
+    [[NSNotificationCenter defaultCenter] postNotificationName: @"UpdateOptionsNotification" object: self];
 }
 
 - (void) setUseGlobalSpeedLimit: (id) sender
@@ -280,6 +299,8 @@
     
     for (Torrent * torrent in fTorrents)
         [torrent setUseGlobalSpeedLimit: limit];
+    
+    [[NSNotificationCenter defaultCenter] postNotificationName: @"UpdateOptionsNotification" object: self];
 }
 
 - (void) setSpeedLimit: (id) sender
@@ -289,12 +310,14 @@
     
     for (Torrent * torrent in fTorrents)
         [torrent setSpeedLimit: limit upload: upload];
+    
+    [[NSNotificationCenter defaultCenter] postNotificationName: @"UpdateOptionsNotification" object: self];
 }
 
 - (void) setRatioSetting: (id) sender
 {
     NSInteger setting;
-    bool single = NO;
+    BOOL single = NO;
     switch ([sender indexOfSelectedItem])
     {
         case OPTION_POPUP_LIMIT:
@@ -321,6 +344,10 @@
         [fRatioLimitField selectText: self];
         [[[self view] window] makeKeyAndOrderFront: self];
     }
+    
+    [fRatioLimitGlobalLabel setHidden: setting != TR_RATIOLIMIT_GLOBAL];
+    
+    [[NSNotificationCenter defaultCenter] postNotificationName: @"UpdateOptionsNotification" object: self];
 }
 
 - (void) setRatioLimit: (id) sender
@@ -329,12 +356,14 @@
     
     for (Torrent * torrent in fTorrents)
         [torrent setRatioLimit: limit];
+    
+    [[NSNotificationCenter defaultCenter] postNotificationName: @"UpdateOptionsNotification" object: self];
 }
 
 - (void) setIdleSetting: (id) sender
 {
     NSInteger setting;
-    bool single = NO;
+    BOOL single = NO;
     switch ([sender indexOfSelectedItem])
     {
         case OPTION_POPUP_LIMIT:
@@ -362,6 +391,10 @@
         [fIdleLimitField selectText: self];
         [[[self view] window] makeKeyAndOrderFront: self];
     }
+    
+    [fIdleLimitGlobalLabel setHidden: setting != TR_IDLELIMIT_GLOBAL];
+    
+    [[NSNotificationCenter defaultCenter] postNotificationName: @"UpdateOptionsNotification" object: self];
 }
 
 - (void) setIdleLimit: (id) sender
@@ -370,6 +403,8 @@
     
     for (Torrent * torrent in fTorrents)
         [torrent setIdleLimitMinutes: limit];
+    
+    [[NSNotificationCenter defaultCenter] postNotificationName: @"UpdateOptionsNotification" object: self];
 }
 
 - (void) setPriority: (id) sender
@@ -395,6 +430,8 @@
         [torrent setPriority: priority];
     
     [[NSNotificationCenter defaultCenter] postNotificationName: @"UpdateUI" object: nil];
+    
+    [[NSNotificationCenter defaultCenter] postNotificationName: @"UpdateOptionsNotification" object: self];
 }
 
 - (void) setPeersConnectLimit: (id) sender
@@ -403,6 +440,8 @@
     
     for (Torrent * torrent in fTorrents)
         [torrent setMaxPeerConnect: limit];
+    
+    [[NSNotificationCenter defaultCenter] postNotificationName: @"UpdateOptionsNotification" object: self];
 }
 
 - (BOOL) control: (NSControl *) control textShouldBeginEditing: (NSText *) fieldEditor
@@ -455,12 +494,14 @@
         [fRatioPopUp selectItemAtIndex: -1];
         [fRatioLimitField setHidden: YES];
         [fRatioLimitField setStringValue: @""];
+        [fRatioLimitGlobalLabel setHidden: YES];
         
         [fIdlePopUp setEnabled: NO];
         [fIdlePopUp selectItemAtIndex: -1];
         [fIdleLimitField setHidden: YES];
         [fIdleLimitField setStringValue: @""];
         [fIdleLimitLabel setHidden: YES];
+        [fIdleLimitGlobalLabel setHidden: YES];
         
         [fPeersConnectField setEnabled: NO];
         [fPeersConnectField setStringValue: @""];
@@ -470,4 +511,30 @@
         [self updateOptions];
 }
 
+- (void) setGlobalLabels
+{
+    NSString * global = [[NSUserDefaults standardUserDefaults] boolForKey: @"RatioCheck"]
+        ? [NSString stringForRatio: [[NSUserDefaults standardUserDefaults] floatForKey: @"RatioLimit"]]
+        : NSLocalizedString(@"disabled", "Info options -> global setting");
+    [fRatioLimitGlobalLabel setStringValue: global];
+    
+    //idle field
+    NSString * globalIdle;
+    if ([[NSUserDefaults standardUserDefaults] boolForKey: @"IdleLimitCheck"])
+    {
+        const NSInteger globalMin = [[NSUserDefaults standardUserDefaults] integerForKey: @"IdleLimitMinutes"];
+        globalIdle = globalMin == 1 ? NSLocalizedString(@"1 minute", "Info options -> global setting")
+            : [NSString stringWithFormat: NSLocalizedString(@"%d minutes", "Info options -> global setting"), globalMin];
+    }
+    else
+        globalIdle = NSLocalizedString(@"disabled", "Info options -> global setting");
+    [fIdleLimitGlobalLabel setStringValue: globalIdle];
+}
+
+- (void) updateOptionsNotification: (NSNotification *) notification
+{
+    if ([notification object] != self)
+        [self updateOptions];
+}
+
 @end
