@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: InfoPeersViewController.m 11409 2010-11-14 15:16:39Z livings124 $
+ * $Id: InfoPeersViewController.m 12081 2011-03-04 02:58:08Z livings124 $
  *
- * Copyright (c) 2010 Transmission authors and contributors
+ * Copyright (c) 2010-2011 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -138,7 +138,7 @@
     else
         [fWebSeeds removeAllObjects];
     
-    NSUInteger known = 0, connected = 0, tracker = 0, incoming = 0, cache = 0, lpd = 0, pex = 0, dht = 0, ltep = 0,
+    NSUInteger connected = 0, tracker = 0, incoming = 0, cache = 0, lpd = 0, pex = 0, dht = 0, ltep = 0,
                 toUs = 0, fromUs = 0;
     BOOL anyActive = false;
     for (Torrent * torrent in fTorrents)
@@ -146,8 +146,6 @@
         if ([torrent webSeedCount] > 0)
             [fWebSeeds addObjectsFromArray: [torrent webSeeds]];
         
-        known += [torrent totalPeersKnown];
-        
         if ([torrent isActive])
         {
             anyActive = YES;
@@ -177,7 +175,6 @@
     [fWebSeeds sortUsingDescriptors: [fWebSeedTable sortDescriptors]];
     [fWebSeedTable reloadData];
     
-    NSString * knownString = [NSString stringWithFormat: NSLocalizedString(@"%d known", "Inspector -> Peers tab -> peers"), known];
     if (anyActive)
     {
         NSString * connectedText = [NSString stringWithFormat: NSLocalizedString(@"%d Connected", "Inspector -> Peers tab -> peers"),
@@ -185,7 +182,17 @@
         
         if (connected > 0)
         {
-            NSMutableArray * fromComponents = [NSMutableArray arrayWithCapacity: 6];
+            NSMutableArray * upDownComponents = [NSMutableArray arrayWithCapacity: 2];
+            if (toUs > 0)
+                [upDownComponents addObject: [NSString stringWithFormat:
+                                        NSLocalizedString(@"DL from %d", "Inspector -> Peers tab -> peers"), toUs]];
+            if (fromUs > 0)
+                [upDownComponents addObject: [NSString stringWithFormat:
+                                        NSLocalizedString(@"UL to %d", "Inspector -> Peers tab -> peers"), fromUs]];
+            if ([upDownComponents count] > 0)
+                connectedText = [connectedText stringByAppendingFormat: @": %@", [upDownComponents componentsJoinedByString: @", "]];
+            
+            NSMutableArray * fromComponents = [NSMutableArray arrayWithCapacity: 7];
             if (tracker > 0)
                 [fromComponents addObject: [NSString stringWithFormat:
                                         NSLocalizedString(@"%d tracker", "Inspector -> Peers tab -> peers"), tracker]];
@@ -208,33 +215,20 @@
                 [fromComponents addObject: [NSString stringWithFormat:
                                         NSLocalizedString(@"%d LTEP", "Inspector -> Peers tab -> peers"), ltep]];
             
-            NSMutableArray * upDownComponents = [NSMutableArray arrayWithCapacity: 3];
-            if (toUs > 0)
-                [upDownComponents addObject: [NSString stringWithFormat:
-                                        NSLocalizedString(@"DL from %d", "Inspector -> Peers tab -> peers"), toUs]];
-            if (fromUs > 0)
-                [upDownComponents addObject: [NSString stringWithFormat:
-                                        NSLocalizedString(@"UL to %d", "Inspector -> Peers tab -> peers"), fromUs]];
-            [upDownComponents addObject: knownString];
-            
-            connectedText = [connectedText stringByAppendingFormat: @": %@\n%@", [fromComponents componentsJoinedByString: @", "],
-                                [upDownComponents componentsJoinedByString: @", "]];
+            connectedText = [connectedText stringByAppendingFormat: @"\n%@", [fromComponents componentsJoinedByString: @", "]];
         }
-        else
-            connectedText = [connectedText stringByAppendingFormat: @"\n%@", knownString];
         
         [fConnectedPeersField setStringValue: connectedText];
     }
     else
     {
-        NSString * activeString;
+        NSString * notActiveString;
         if ([fTorrents count] == 1)
-            activeString = NSLocalizedString(@"Transfer Not Active", "Inspector -> Peers tab -> peers");
+            notActiveString = NSLocalizedString(@"Transfer Not Active", "Inspector -> Peers tab -> peers");
         else
-            activeString = NSLocalizedString(@"Transfers Not Active", "Inspector -> Peers tab -> peers");
+            notActiveString = NSLocalizedString(@"Transfers Not Active", "Inspector -> Peers tab -> peers");
         
-        NSString * connectedText = [activeString stringByAppendingFormat: @"\n%@", knownString];
-        [fConnectedPeersField setStringValue: connectedText];
+        [fConnectedPeersField setStringValue: notActiveString];
     }
 }
 
@@ -366,8 +360,13 @@
                                 "Inspector -> Peers tab -> table row tooltip")];
         [components addObject: progressString];
         
+        NSString * protocolString = [[peer objectForKey: @"uTP"] boolValue] ? @"\u00b5TP" : @"TCP";
         if ([[peer objectForKey: @"Encryption"] boolValue])
-            [components addObject: NSLocalizedString(@"Encrypted Connection", "Inspector -> Peers tab -> table row tooltip")];
+            protocolString = [protocolString stringByAppendingFormat: @" (%@)",
+                                NSLocalizedString(@"encrypted", "Inspector -> Peers tab -> table row tooltip")];
+        [components addObject: [NSString stringWithFormat:
+                                NSLocalizedString(@"Protocol: %@", "Inspector -> Peers tab -> table row tooltip"),
+                                protocolString]];
         
         NSString * portString;
         NSInteger port;
