@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: NSApplicationAdditions.h 4423 2008-01-02 16:55:05Z livings124 $
+ * $Id: NSApplicationAdditions.h 12570 2011-07-21 02:35:03Z livings124 $
  *
- * Copyright (c) 2007-2008 Transmission authors and contributors
+ * Copyright (c) 2009-2011 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -22,8 +22,13 @@
  * DEALINGS IN THE SOFTWARE.
  *****************************************************************************/
 
+#define QLPreviewPanelSL NSClassFromString(@"QLPreviewPanel")
+#define NSRunningApplicationSL NSClassFromString(@"NSRunningApplication")
+#define NSPopoverLion NSClassFromString(@"NSPopover")
+
 @interface NSApplication (NSApplicationAdditions)
 
-- (BOOL) isOnLeopardOrBetter;
+- (BOOL) isOnSnowLeopardOrBetter;
+- (BOOL) isOnLionOrBetter;
 
 @end
