@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: NSStringAdditions.m 11417 2010-11-14 20:26:58Z livings124 $
+ * $Id: NSStringAdditions.m 12910 2011-09-24 19:38:04Z livings124 $
  *
- * Copyright (c) 2005-2010 Transmission authors and contributors
+ * Copyright (c) 2005-2011 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -30,6 +30,9 @@
 
 @interface NSString (Private)
 
++ (NSString *) stringForFileSize: (uint64_t) size showUnitUnless: (NSString *) notAllowedUnit
+    unitsUsed: (NSString **) unitUsed;
+
 + (NSString *) stringForSpeed: (CGFloat) speed kb: (NSString *) kb mb: (NSString *) mb gb: (NSString *) gb;
 
 @end
@@ -57,43 +60,16 @@
 
 + (NSString *) stringForFileSize: (uint64_t) size
 {
-    const CGFloat baseFloat = [NSApp isOnSnowLeopardOrBetter] ? 1000.0 : 1024.0;
-    const NSUInteger baseInt = [NSApp isOnSnowLeopardOrBetter] ? 1000 : 1024;
-    
-    if (size < baseInt)
-    {
-        if (size != 1)
-            return [NSString stringWithFormat: @"%lld %@", size, NSLocalizedString(@"bytes", "File size - bytes")];
-        else
-            return NSLocalizedString(@"1 byte", "File size - bytes");
-    }
+    return [self stringForFileSize: size showUnitUnless: nil unitsUsed: nil];
+}
 
-    CGFloat convertedSize;
-    NSString * unit;
-    if (size < pow(baseInt, 2))
-    {
-        convertedSize = size / baseFloat;
-        unit = NSLocalizedString(@"KB", "File size - kilobytes");
-    }
-    else if (size < pow(baseInt, 3))
-    {
-        convertedSize = size / powf(baseFloat, 2);
-        unit = NSLocalizedString(@"MB", "File size - megabytes");
-    }
-    else if (size < pow(baseInt, 4))
-    {
-        convertedSize = size / powf(baseFloat, 3);
-        unit = NSLocalizedString(@"GB", "File size - gigabytes");
-    }
-    else
-    {
-        convertedSize = size / powf(baseFloat, 4);
-        unit = NSLocalizedString(@"TB", "File size - terabytes");
-    }
++ (NSString *) stringForFilePartialSize: (uint64_t) partialSize fullSize: (uint64_t) fullSize
+{
+    NSString * units;
+    NSString * fullString = [self stringForFileSize: fullSize showUnitUnless: nil unitsUsed: &units];
+    NSString * partialString = [self stringForFileSize: partialSize showUnitUnless: units unitsUsed: nil];
     
-    //attempt to have minimum of 3 digits with at least 1 decimal
-    return convertedSize <= 9.995 ? [NSString localizedStringWithFormat: @"%.2f %@", convertedSize, unit]
-                                : [NSString localizedStringWithFormat: @"%.1f %@", convertedSize, unit];
+    return [NSString stringWithFormat: NSLocalizedString(@"%@ of %@", "file size string"), partialString, fullString];
 }
 
 + (NSString *) stringForSpeed: (CGFloat) speed
@@ -146,10 +122,20 @@
 {
     NSAssert(max > 0, @"Cannot generate a time string with no fields");
     
-    NSMutableArray * timeArray = [NSMutableArray arrayWithCapacity: MIN(max, 4)];
+    NSMutableArray * timeArray = [NSMutableArray arrayWithCapacity: MIN(max, 5)];
     NSUInteger remaining = seconds; //causes problems for some users when it's a uint64_t
     
-    if (seconds >= (24 * 60 * 60))
+    if (seconds >= 31557600) //official amount of seconds in one year
+    {
+        const NSUInteger years = remaining / 31557600;
+        if (years == 1)
+            [timeArray addObject: NSLocalizedString(@"1 year", "time string")];
+        else
+            [timeArray addObject: [NSString stringWithFormat: NSLocalizedString(@"%u years", "time string"), years]];
+        remaining %= 31557600;
+        --max;
+    }
+    if (max > 0 && seconds >= (24 * 60 * 60))
     {
         const NSUInteger days = remaining / (24 * 60 * 60);
         if (days == 1)
@@ -157,19 +143,19 @@
         else
             [timeArray addObject: [NSString stringWithFormat: NSLocalizedString(@"%u days", "time string"), days]];
         remaining %= (24 * 60 * 60);
-        max--;
+        --max;
     }
     if (max > 0 && seconds >= (60 * 60))
     {
         [timeArray addObject: [NSString stringWithFormat: NSLocalizedString(@"%u hr", "time string"), remaining / (60 * 60)]];
         remaining %= (60 * 60);
-        max--;
+        --max;
     }
     if (max > 0 && (!showSeconds || seconds >= 60))
     {
         [timeArray addObject: [NSString stringWithFormat: NSLocalizedString(@"%u min", "time string"), remaining / 60]];
         remaining %= 60;
-        max--;
+        --max;
     }
     if (max > 0 && showSeconds)
         [timeArray addObject: [NSString stringWithFormat: NSLocalizedString(@"%u sec", "time string"), remaining]];
@@ -196,10 +182,90 @@
     return [self compare: string options: comparisonOptions range: NSMakeRange(0, [self length]) locale: [NSLocale currentLocale]];
 }
 
+- (NSArray *) betterComponentsSeparatedByCharactersInSet: (NSCharacterSet *) separator
+{
+    NSMutableArray * components = [NSMutableArray array];
+    
+    NSUInteger i = 0;
+    while (i < [self length])
+    {
+        const NSRange range = [self rangeOfCharacterFromSet: separator options: 0 range: NSMakeRange(i, [self length]-i)];
+        
+        if (range.location == NSNotFound)
+        {
+            [components addObject: [self substringFromIndex: i]];
+            break;
+        }
+        else if (range.location != i)
+        {
+            const NSUInteger length = range.location - i;
+            [components addObject: [self substringWithRange: NSMakeRange(i, length)]];
+            
+            i += length;
+        }
+        
+        i += range.length;
+    }
+    
+    return components;
+}
+
 @end
 
 @implementation NSString (Private)
 
++ (NSString *) stringForFileSize: (uint64_t) size showUnitUnless: (NSString *) notAllowedUnit
+    unitsUsed: (NSString **) unitUsed
+{
+    const float baseFloat = [NSApp isOnSnowLeopardOrBetter] ? 1000.0 : 1024.0;
+    const NSUInteger baseInt = [NSApp isOnSnowLeopardOrBetter] ? 1000 : 1024;
+    
+    double convertedSize;
+    NSString * unit;
+    NSUInteger decimals;
+    if (size < pow(baseInt, 2))
+    {
+        convertedSize = size / baseFloat;
+        unit = NSLocalizedString(@"KB", "File size - kilobytes");
+        decimals = convertedSize >= 10.0 ? 0 : 1;
+    }
+    else if (size < pow(baseInt, 3))
+    {
+        convertedSize = size / powf(baseFloat, 2);
+        unit = NSLocalizedString(@"MB", "File size - megabytes");
+        decimals = 1;
+    }
+    else if (size < pow(baseInt, 4))
+    {
+        convertedSize = size / powf(baseFloat, 3);
+        unit = NSLocalizedString(@"GB", "File size - gigabytes");
+        decimals = 2;
+    }
+    else
+    {
+        convertedSize = size / powf(baseFloat, 4);
+        unit = NSLocalizedString(@"TB", "File size - terabytes");
+        decimals = 3; //guessing on this one
+    }
+    
+    //match Finder's behavior
+    NSNumberFormatter * numberFormatter = [[NSNumberFormatter alloc] init];
+    [numberFormatter setNumberStyle: NSNumberFormatterDecimalStyle];
+    [numberFormatter setMinimumFractionDigits: 0];
+    [numberFormatter setMaximumFractionDigits: decimals];
+    
+    NSString * fileSizeString = [numberFormatter stringFromNumber: [NSNumber numberWithFloat: convertedSize]];
+    [numberFormatter release];
+    
+    if (!notAllowedUnit || ![unit isEqualToString: notAllowedUnit])
+        fileSizeString = [fileSizeString stringByAppendingFormat: @" %@", unit];
+    
+    if (unitUsed)
+        *unitUsed = unit;
+    
+    return fileSizeString;
+}
+
 + (NSString *) stringForSpeed: (CGFloat) speed kb: (NSString *) kb mb: (NSString *) mb gb: (NSString *) gb
 {
     const CGFloat baseFloat = [NSApp isOnSnowLeopardOrBetter] ? 1000.0 : 1024.0;
