@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: PrefsController.m 11757 2011-01-23 18:26:35Z livings124 $
+ * $Id: PrefsController.m 12693 2011-08-17 01:49:55Z livings124 $
  *
  * Copyright (c) 2005-2011 Transmission authors and contributors
  *
@@ -53,7 +53,7 @@
 #define RPC_KEYCHAIN_SERVICE    "Transmission:Remote"
 #define RPC_KEYCHAIN_NAME       "Remote"
 
-#define WEBUI_URL   @"http://localhost:%d/transmission/web/"
+#define WEBUI_URL   @"http://localhost:%d/"
 
 @interface PrefsController (Private)
 
@@ -223,6 +223,15 @@ tr_session * fHandle;
     [self updateBlocklistButton];
     [self updateBlocklistFields];
     
+    [[NSNotificationCenter defaultCenter] addObserver: self selector: @selector(updateLimitFields)
+                                                 name: @"UpdateSpeedLimitValuesOutsidePrefs" object: nil];
+    
+    [[NSNotificationCenter defaultCenter] addObserver: self selector: @selector(updateRatioStopField)
+                                                 name: @"UpdateRatioStopValueOutsidePrefs" object: nil];
+    
+    [[NSNotificationCenter defaultCenter] addObserver: self selector: @selector(updateLimitStopField)
+                                                 name: @"UpdateIdleStopValueOutsidePrefs" object: nil];
+    
     [[NSNotificationCenter defaultCenter] addObserver: self selector: @selector(updateBlocklistFields)
         name: @"BlocklistUpdated" object: nil];
     
@@ -452,6 +461,11 @@ tr_session * fHandle;
         [sound play];
 }
 
+- (void) setUTP: (id) sender
+{
+    tr_sessionSetUTPEnabled(fHandle, [fDefaults boolForKey: @"UTPGlobal"]);
+}
+
 - (void) setPeersGlobal: (id) sender
 {
     const int count = [sender intValue];
@@ -597,6 +611,12 @@ tr_session * fHandle;
 {
     tr_sessionSetRatioLimited(fHandle, [fDefaults boolForKey: @"RatioCheck"]);
     tr_sessionSetRatioLimit(fHandle, [fDefaults floatForKey: @"RatioLimit"]);
+    
+    //reload main table for seeding progress
+    [[NSNotificationCenter defaultCenter] postNotificationName: @"UpdateUI" object: nil];
+    
+    //reload global settings in inspector
+    [[NSNotificationCenter defaultCenter] postNotificationName: @"UpdateGlobalOptions" object: nil];
 }
 
 - (void) setRatioStop: (id) sender
@@ -610,6 +630,11 @@ tr_session * fHandle;
 {
     if (fHasLoaded)
         [fRatioStopField setFloatValue: [fDefaults floatForKey: @"RatioLimit"]];
+}
+
+- (void) updateRatioStopFieldOld
+{
+    [self updateRatioStopField];
     
     [self applyRatioSetting: nil];
 }
@@ -618,6 +643,12 @@ tr_session * fHandle;
 {
     tr_sessionSetIdleLimited(fHandle, [fDefaults boolForKey: @"IdleLimitCheck"]);
     tr_sessionSetIdleLimit(fHandle, [fDefaults integerForKey: @"IdleLimitMinutes"]);
+    
+    //reload main table for remaining seeding time
+    [[NSNotificationCenter defaultCenter] postNotificationName: @"UpdateUI" object: nil];
+    
+    //reload global settings in inspector
+    [[NSNotificationCenter defaultCenter] postNotificationName: @"UpdateGlobalOptions" object: nil];
 }
 
 - (void) setIdleStop: (id) sender
@@ -627,6 +658,12 @@ tr_session * fHandle;
     [self applyIdleStopSetting: nil];
 }
 
+- (void) updateLimitStopField
+{
+    if (fHasLoaded)
+        [fIdleStopField setIntegerValue: [fDefaults integerForKey: @"IdleLimitMinutes"]];
+}
+
 - (void) updateLimitFields
 {
     if (!fHasLoaded)
@@ -716,6 +753,7 @@ tr_session * fHandle;
     [fDefaults removeObjectForKey: @"WarningCreatorPrivateBlankAddress"];
     [fDefaults removeObjectForKey: @"WarningRemoveTrackers"];
     [fDefaults removeObjectForKey: @"WarningInvalidOpen"];
+    [fDefaults removeObjectForKey: @"WarningRemoveCompleted"];
     [fDefaults removeObjectForKey: @"WarningDonate"];
     //[fDefaults removeObjectForKey: @"WarningLegal"];
 }
@@ -730,24 +768,40 @@ tr_session * fHandle;
 
 - (void) setQueue: (id) sender
 {
+    //let's just do both - easier that way
+    tr_sessionSetQueueEnabled(fHandle, TR_DOWN, [fDefaults boolForKey: @"Queue"]);
+    tr_sessionSetQueueEnabled(fHandle, TR_UP, [fDefaults boolForKey: @"QueueSeed"]);
+    
+    //handle if any transfers switch from queued to paused
     [[NSNotificationCenter defaultCenter] postNotificationName: @"UpdateQueue" object: self];
 }
 
 - (void) setQueueNumber: (id) sender
 {
-    [fDefaults setInteger: [sender intValue] forKey: sender == fQueueDownloadField ? @"QueueDownloadNumber" : @"QueueSeedNumber"];
-    [self setQueue: nil];
+    const NSInteger number = [sender intValue];
+    const BOOL seed = sender == fQueueSeedField;
+    
+    [fDefaults setInteger: number forKey: seed ? @"QueueSeedNumber" : @"QueueDownloadNumber"];
+    
+    tr_sessionSetQueueSize(fHandle, seed ? TR_UP : TR_DOWN, number);
 }
 
 - (void) setStalled: (id) sender
 {
-    [[NSNotificationCenter defaultCenter] postNotificationName: @"UpdateQueue" object: self];
+    tr_sessionSetQueueStalledEnabled(fHandle, [fDefaults boolForKey: @"CheckStalled"]);
+    
+    //reload main table for stalled status
+    [[NSNotificationCenter defaultCenter] postNotificationName: @"UpdateUI" object: nil];
 }
 
 - (void) setStalledMinutes: (id) sender
 {
-    [fDefaults setInteger: [sender intValue] forKey: @"StalledMinutes"];
-    [self setStalled: nil];
+    const NSInteger min = [sender intValue];
+    [fDefaults setInteger: min forKey: @"StalledMinutes"];
+    tr_sessionSetQueueStalledMinutes(fHandle, min);
+    
+    //reload main table for stalled status
+    [[NSNotificationCenter defaultCenter] postNotificationName: @"UpdateUI" object: self];
 }
 
 - (void) setDownloadLocation: (id) sender
@@ -1088,6 +1142,10 @@ tr_session * fHandle;
     const BOOL usePartialFileRanaming = tr_sessionIsIncompleteFileNamingEnabled(fHandle);
     [fDefaults setBool: usePartialFileRanaming forKey: @"RenamePartialFiles"];
     
+    //utp
+    const BOOL utp = tr_sessionIsUTPEnabled(fHandle);
+    [fDefaults setBool: utp forKey: @"UTPGlobal"];
+    
     //peers
     const uint16_t peersTotal = tr_sessionGetPeerLimit(fHandle);
     [fDefaults setInteger: peersTotal forKey: @"PeersTotal"];
@@ -1103,7 +1161,7 @@ tr_session * fHandle;
     const BOOL dht = tr_sessionIsDHTEnabled(fHandle);
     [fDefaults setBool: dht forKey: @"DHTGlobal"];
     
-    //dht
+    //lpd
     const BOOL lpd = tr_sessionIsLPDEnabled(fHandle);
     [fDefaults setBool: lpd forKey: @"LocalPeerDiscoveryGlobal"];
     
@@ -1178,13 +1236,32 @@ tr_session * fHandle;
     const float ratioLimit = tr_sessionGetRatioLimit(fHandle);
     [fDefaults setFloat: ratioLimit forKey: @"RatioLimit"];
     
-    //Idle seed limit
+    //idle seed limit
     const BOOL idleLimited = tr_sessionIsIdleLimited(fHandle);
     [fDefaults setBool: idleLimited forKey: @"IdleLimitCheck"];
     
     const NSUInteger idleLimitMin = tr_sessionGetIdleLimit(fHandle);
     [fDefaults setInteger: idleLimitMin forKey: @"IdleLimitMinutes"];
     
+    //queue
+    const BOOL downloadQueue = tr_sessionGetQueueEnabled(fHandle, TR_DOWN);
+    [fDefaults setBool: downloadQueue forKey: @"Queue"];
+    
+    const int downloadQueueNum = tr_sessionGetQueueSize(fHandle, TR_DOWN);
+    [fDefaults setInteger: downloadQueueNum forKey: @"QueueDownloadNumber"];
+    
+    const BOOL seedQueue = tr_sessionGetQueueEnabled(fHandle, TR_UP);
+    [fDefaults setBool: seedQueue forKey: @"QueueSeed"];
+    
+    const int seedQueueNum = tr_sessionGetQueueSize(fHandle, TR_UP);
+    [fDefaults setInteger: seedQueueNum forKey: @"QueueSeedNumber"];
+    
+    const BOOL checkStalled = tr_sessionGetQueueStalledEnabled(fHandle);
+    [fDefaults setBool: checkStalled forKey: @"CheckStalled"];
+    
+    const int stalledMinutes = tr_sessionGetQueueStalledMinutes(fHandle);
+    [fDefaults setInteger: stalledMinutes forKey: @"StalledMinutes"];
+    
     //done script
     const BOOL doneScriptEnabled = tr_sessionIsTorrentDoneScriptEnabled(fHandle);
     [fDefaults setBool: doneScriptEnabled forKey: @"DoneScriptEnabled"];
@@ -1199,6 +1276,8 @@ tr_session * fHandle;
         
         //download directory handled by bindings
         
+        //utp handled by bindings
+        
         [fPeersGlobalField setIntValue: peersTotal];
         [fPeersTorrentField setIntValue: peersTorrent];
         
@@ -1235,9 +1314,19 @@ tr_session * fHandle;
         
         //idle limit enabled handled by bindings
         [fIdleStopField setIntegerValue: idleLimitMin];
+        
+        //queues enabled handled by bindings
+        [fQueueDownloadField setIntValue: downloadQueueNum];
+        [fQueueSeedField setIntValue: seedQueueNum];
+        
+        //check stalled handled by bindings
+        [fStalledField setIntValue: stalledMinutes];
     }
     
     [[NSNotificationCenter defaultCenter] postNotificationName: @"SpeedLimitUpdate" object: nil];
+    
+    //reload global settings in inspector
+    [[NSNotificationCenter defaultCenter] postNotificationName: @"UpdateGlobalOptions" object: nil];
 }
 
 @end
