@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: Torrent.h 11175 2010-08-14 17:32:52Z livings124 $
+ * $Id: Torrent.h 12773 2011-08-28 00:07:30Z livings124 $
  *
- * Copyright (c) 2006-2010 Transmission authors and contributors
+ * Copyright (c) 2006-2011 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -47,8 +47,6 @@
     NSIndexSet * fPreviousFinishedIndexes;
     NSDate * fPreviousFinishedIndexesDate;
     
-    BOOL fWaitToStart, fStalled;
-    
     NSInteger fGroupValue;
     
     BOOL fResumeOnWake;
@@ -64,7 +62,7 @@
 
 - (NSDictionary *) history;
 
-- (void) closeRemoveTorrent;
+- (void) closeRemoveTorrent: (BOOL) trashFiles;
 
 - (void) changeDownloadFolderBeforeUsing: (NSString *) folder;
 
@@ -73,15 +71,20 @@
 - (void) getAvailability: (int8_t *) tab size: (NSInteger) size;
 - (void) getAmountFinished: (float *) tab size: (NSInteger) size;
 - (NSIndexSet *) previousFinishedPieces;
--(void) setPreviousFinishedPieces: (NSIndexSet *) indexes;
+- (void) setPreviousFinishedPieces: (NSIndexSet *) indexes;
 
 - (void) update;
 
+- (void) startTransferIgnoringQueue: (BOOL) ignoreQueue;
+- (void) startTransferNoQueue;
 - (void) startTransfer;
 - (void) stopTransfer;
 - (void) sleep;
 - (void) wakeUp;
 
+- (NSInteger) queuePosition;
+- (void) setQueuePosition: (NSUInteger) index;
+
 - (void) manualAnnounce;
 - (BOOL) canManualAnnounce;
 
@@ -112,14 +115,12 @@
 - (void) setMaxPeerConnect: (uint16_t) count;
 - (uint16_t) maxPeerConnect;
 
-- (void) setWaitToStart: (BOOL) wait;
 - (BOOL) waitingToStart;
 
 - (tr_priority_t) priority;
 - (void) setPriority: (tr_priority_t) priority;
 
 + (void) trashFile: (NSString *) path;
-- (void) trashData;
 - (void) moveTorrentDataFileTo: (NSString *) folder;
 - (void) copyTorrentFileTo: (NSString *) path;
 
@@ -135,7 +136,7 @@
 - (NSMutableArray *) allTrackerStats;
 - (NSArray *) allTrackersFlat; //used by GroupRules
 - (BOOL) addTrackerToNewTier: (NSString *) tracker;
-- (void) removeTrackersWithIdentifiers: (NSIndexSet *) removeIdentifiers;
+- (void) removeTrackers: (NSSet *) trackers;
 
 - (NSString *) comment;
 - (NSString *) creator;
@@ -187,7 +188,6 @@
 - (NSInteger) totalPeersDHT;
 - (NSInteger) totalPeersLocal;
 - (NSInteger) totalPeersLTEP;
-- (NSInteger) totalPeersKnown;
 
 - (NSInteger) peersSendingToUs;
 - (NSInteger) peersGettingFromUs;
@@ -227,6 +227,9 @@
 - (NSDate *) dateActivity;
 - (NSDate *) dateActivityOrAdd;
 
+- (NSInteger) secondsDownloading;
+- (NSInteger) secondsSeeding;
+
 - (NSInteger) stalledMinutes;
 - (BOOL) isStalled;
 
