@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: Torrent.m 11787 2011-01-30 02:01:05Z livings124 $
+ * $Id: Torrent.m 12774 2011-08-28 00:26:39Z livings124 $
  *
  * Copyright (c) 2006-2011 Transmission authors and contributors
  *
@@ -37,7 +37,6 @@
 
 - (id) initWithPath: (NSString *) path hash: (NSString *) hashString torrentStruct: (tr_torrent *) torrentStruct
         magnetAddress: (NSString *) magnetAddress lib: (tr_session *) lib
-        waitToStart: (NSNumber *) waitToStart
         groupValue: (NSNumber *) groupValue
         timeMachineExcludeLocation: (NSString *) timeMachineExclude
         downloadFolder: (NSString *) downloadFolder
@@ -48,6 +47,7 @@
     index: (NSInteger) index flatList: (NSMutableArray *) flatFileList;
 - (void) sortFileList: (NSMutableArray *) fileNodes;
 
+- (void) startQueue;
 - (void) completenessChange: (NSDictionary *) statusInfo;
 - (void) ratioLimitHit;
 - (void) idleLimitHit;
@@ -60,7 +60,12 @@
 
 @end
 
-void completenessChangeCallback(tr_torrent * torrent, tr_completeness status, tr_bool wasRunning, void * torrentData)
+void startQueueCallback(tr_torrent * torrent, void * torrentData)
+{
+    [(Torrent *)torrentData performSelectorOnMainThread: @selector(startQueue) withObject: nil waitUntilDone: NO];
+}
+
+void completenessChangeCallback(tr_torrent * torrent, tr_completeness status, bool wasRunning, void * torrentData)
 {
     NSAutoreleasePool * pool = [[NSAutoreleasePool alloc] init];
     
@@ -88,8 +93,12 @@ void metadataCallback(tr_torrent * torrent, void * torrentData)
 
 int trashDataFile(const char * filename)
 {
+    NSAutoreleasePool * pool = [[NSAutoreleasePool alloc] init];
+    
     if (filename != NULL)
         [Torrent trashFile: [NSString stringWithUTF8String: filename]];
+    
+    [pool drain];
     return 0;
 }
 
@@ -99,7 +108,7 @@ int trashDataFile(const char * filename)
         lib: (tr_session *) lib
 {
     self = [self initWithPath: path hash: nil torrentStruct: NULL magnetAddress: nil lib: lib
-            waitToStart: nil groupValue: nil
+            groupValue: nil
             timeMachineExcludeLocation: nil
             downloadFolder: location
             legacyIncompleteFolder: nil];
@@ -115,7 +124,7 @@ int trashDataFile(const char * filename)
 - (id) initWithTorrentStruct: (tr_torrent *) torrentStruct location: (NSString *) location lib: (tr_session *) lib
 {
     self = [self initWithPath: nil hash: nil torrentStruct: torrentStruct magnetAddress: nil lib: lib
-            waitToStart: nil groupValue: nil
+            groupValue: nil
             timeMachineExcludeLocation: nil
             downloadFolder: location
             legacyIncompleteFolder: nil];
@@ -126,7 +135,7 @@ int trashDataFile(const char * filename)
 - (id) initWithMagnetAddress: (NSString *) address location: (NSString *) location lib: (tr_session *) lib
 {
     self = [self initWithPath: nil hash: nil torrentStruct: nil magnetAddress: address
-            lib: lib waitToStart: nil groupValue: nil
+            lib: lib groupValue: nil
             timeMachineExcludeLocation: nil
             downloadFolder: location legacyIncompleteFolder: nil];
     
@@ -140,7 +149,6 @@ int trashDataFile(const char * filename)
                 torrentStruct: NULL
                 magnetAddress: nil
                 lib: lib
-                waitToStart: [history objectForKey: @"WaitToStart"]
                 groupValue: [history objectForKey: @"GroupValue"]
                 timeMachineExcludeLocation: [history objectForKey: @"TimeMachineExcludeLocation"]
                 downloadFolder: [history objectForKey: @"DownloadFolder"] //upgrading from versions < 1.80
@@ -154,7 +162,7 @@ int trashDataFile(const char * filename)
         if (!pause && (active = [history objectForKey: @"Active"]) && [active boolValue])
         {
             fStat = tr_torrentStat(fHandle);
-            [self startTransfer];
+            [self startTransferNoQueue];
         }
         
         //upgrading from versions < 1.30: get old added, activity, and done dates
@@ -190,7 +198,7 @@ int trashDataFile(const char * filename)
                                         [self torrentLocation], @"InternalTorrentPath",
                                         [self hashString], @"TorrentHash",
                                         [NSNumber numberWithBool: [self isActive]], @"Active",
-                                        [NSNumber numberWithBool: fWaitToStart], @"WaitToStart",
+                                        [NSNumber numberWithBool: [self waitingToStart]], @"WaitToStart",
                                         [NSNumber numberWithInt: fGroupValue], @"GroupValue", nil];
     
     if (fTimeMachineExclude)
@@ -274,7 +282,7 @@ int trashDataFile(const char * filename)
         return nil;
 }
 
--(void) setPreviousFinishedPieces: (NSIndexSet *) indexes
+- (void) setPreviousFinishedPieces: (NSIndexSet *) indexes
 {
     [fPreviousFinishedIndexes release];
     fPreviousFinishedIndexes = [indexes retain];
@@ -285,38 +293,18 @@ int trashDataFile(const char * filename)
 
 - (void) update
 {
-    //get previous status values before update
-    BOOL wasChecking = NO, wasError = NO, wasStalled = NO;
-    if (fStat != NULL)
-    {
-        wasChecking = [self isChecking];
-        wasError = [self isError];
-        wasStalled = fStalled;
-    }
-    
     fStat = tr_torrentStat(fHandle);
     
-    //check if stalled (stored because based on time and needs to check if it was previously stalled)
-    fStalled = [self isActive] && [fDefaults boolForKey: @"CheckStalled"]
-                && [self stalledMinutes] > [fDefaults integerForKey: @"StalledMinutes"];
-    
-    //update queue for checking (from downloading to seeding), stalled, or error
-    if ((wasChecking && ![self isChecking]) || (wasStalled != fStalled) || (!wasError && [self isError] && [self isActive]))
-        [[NSNotificationCenter defaultCenter] postNotificationName: @"UpdateQueue" object: self];
-    
     //when the data first appears, update time machine exclusion
     if (!fTimeMachineExclude)
         [self updateTimeMachineExclude];
 }
 
-- (void) startTransfer
+- (void) startTransferIgnoringQueue: (BOOL) ignoreQueue
 {
-    fWaitToStart = NO;
-    
-    if (![self isActive] && [self alertForRemainingDiskSpace])
+    if ([self alertForRemainingDiskSpace])
     {
-        tr_ninf( fInfo->name, "restarting via startTransfer" );
-        tr_torrentStart(fHandle);
+        ignoreQueue ? tr_torrentStartNow(fHandle) : tr_torrentStart(fHandle);
         [self update];
         
         //capture, specifically, stop-seeding settings changing to unlimited
@@ -324,17 +312,20 @@ int trashDataFile(const char * filename)
     }
 }
 
+- (void) startTransferNoQueue
+{
+    [self startTransferIgnoringQueue: YES];
+}
+
+- (void) startTransfer
+{
+    [self startTransferIgnoringQueue: NO];
+}
+
 - (void) stopTransfer
 {
-    fWaitToStart = NO;
-    
-    if ([self isActive])
-    {
-        tr_torrentStop(fHandle);
-        [self update];
-        
-        [[NSNotificationCenter defaultCenter] postNotificationName: @"UpdateQueue" object: self];
-    }
+    tr_torrentStop(fHandle);
+    [self update];
 }
 
 - (void) sleep
@@ -352,6 +343,16 @@ int trashDataFile(const char * filename)
     }
 }
 
+- (NSInteger) queuePosition
+{
+    return fStat->queuePosition;
+}
+
+- (void) setQueuePosition: (NSUInteger) index
+{
+    tr_torrentSetQueuePosition(fHandle, index);
+}
+
 - (void) manualAnnounce
 {
     tr_torrentManualUpdate(fHandle);
@@ -471,15 +472,9 @@ int trashDataFile(const char * filename)
 {
     return tr_torrentGetPeerLimit(fHandle);
 }
-
-- (void) setWaitToStart: (BOOL) wait
-{
-    fWaitToStart = wait;
-}
-
 - (BOOL) waitingToStart
 {
-    return fWaitToStart;
+    return fStat->activity == TR_STATUS_DOWNLOAD_WAIT || fStat->activity == TR_STATUS_SEED_WAIT;
 }
 
 - (tr_priority_t) priority
@@ -568,15 +563,17 @@ int trashDataFile(const char * filename)
     if ([self allDownloaded] || ![fDefaults boolForKey: @"WarningRemainingSpace"])
         return YES;
     
-    NSString * downloadFolder = [self currentDirectory], * volumeName;
-    if ((volumeName = [[[NSFileManager defaultManager] componentsToDisplayForPath: downloadFolder] objectAtIndex: 0]))
+    NSString * downloadFolder = [self currentDirectory];
+    NSDictionary * systemAttributes;
+    if ((systemAttributes = [[NSFileManager defaultManager] attributesOfFileSystemForPath: downloadFolder error: NULL]))
     {
-        NSDictionary * systemAttributes = [[NSFileManager defaultManager] attributesOfFileSystemForPath: downloadFolder error: NULL];
-        uint64_t remainingSpace = [[systemAttributes objectForKey: NSFileSystemFreeSize] unsignedLongLongValue];
+        const uint64_t remainingSpace = [[systemAttributes objectForKey: NSFileSystemFreeSize] unsignedLongLongValue];
         
         //if the remaining space is greater than the size left, then there is enough space regardless of preallocation
         if (remainingSpace < [self sizeLeft] && remainingSpace < tr_torrentGetBytesLeftToAllocate(fHandle))
         {
+            NSString * volumeName = [[[NSFileManager defaultManager] componentsToDisplayForPath: downloadFolder] objectAtIndex: 0];
+            
             NSAlert * alert = [[NSAlert alloc] init];
             [alert setMessageText: [NSString stringWithFormat:
                                     NSLocalizedString(@"Not enough remaining disk space to download \"%@\" completely.",
@@ -693,7 +690,7 @@ int trashDataFile(const char * filename)
     return success;
 }
 
-- (void) removeTrackersWithIdentifiers: (NSIndexSet *) removeIdentifiers
+- (void) removeTrackers: (NSSet *) trackers
 {
     //recreate the tracker structure
     tr_tracker_info * trackerStructs = tr_new(tr_tracker_info, fInfo->trackerCount);
@@ -701,7 +698,7 @@ int trashDataFile(const char * filename)
     NSUInteger newCount = 0;
     for (NSUInteger i = 0; i < fInfo->trackerCount; i++)
     {
-        if (![removeIdentifiers containsIndex: fInfo->trackers[i].id])
+        if (![trackers containsObject: [NSString stringWithUTF8String: fInfo->trackers[i].announce]])
             trackerStructs[newCount++] = fInfo->trackers[i];
     }
     
@@ -834,7 +831,7 @@ int trashDataFile(const char * filename)
 
 - (BOOL) isActive
 {
-    return fStat->activity != TR_STATUS_STOPPED;
+    return fStat->activity != TR_STATUS_STOPPED && fStat->activity != TR_STATUS_DOWNLOAD_WAIT && fStat->activity != TR_STATUS_SEED_WAIT;
 }
 
 - (BOOL) isSeeding
@@ -903,7 +900,7 @@ int trashDataFile(const char * filename)
     for (int i = 0; i < totalPeers; i++)
     {
         tr_peer_stat * peer = &peers[i];
-        NSMutableDictionary * dict = [NSMutableDictionary dictionaryWithCapacity: 11];
+        NSMutableDictionary * dict = [NSMutableDictionary dictionaryWithCapacity: 12];
         
         [dict setObject: [self name] forKey: @"Name"];
         [dict setObject: [NSNumber numberWithInt: peer->from] forKey: @"From"];
@@ -912,6 +909,7 @@ int trashDataFile(const char * filename)
         [dict setObject: [NSNumber numberWithFloat: peer->progress] forKey: @"Progress"];
         [dict setObject: [NSNumber numberWithBool: peer->isSeed] forKey: @"Seed"];
         [dict setObject: [NSNumber numberWithBool: peer->isEncrypted] forKey: @"Encryption"];
+        [dict setObject: [NSNumber numberWithBool: peer->isUTP] forKey: @"uTP"];
         [dict setObject: [NSString stringWithUTF8String: peer->client] forKey: @"Client"];
         [dict setObject: [NSString stringWithUTF8String: peer->flagStr] forKey: @"Flags"];
         
@@ -1043,18 +1041,20 @@ int trashDataFile(const char * filename)
         switch (fStat->activity)
         {
             case TR_STATUS_STOPPED:
-                if (fWaitToStart)
-                {
-                    string = ![self allDownloaded]
-                            ? [NSLocalizedString(@"Waiting to download", "Torrent -> status string") stringByAppendingEllipsis]
-                            : [NSLocalizedString(@"Waiting to seed", "Torrent -> status string") stringByAppendingEllipsis];
-                }
-                else if ([self isFinishedSeeding])
+                if ([self isFinishedSeeding])
                     string = NSLocalizedString(@"Seeding complete", "Torrent -> status string");
                 else
                     string = NSLocalizedString(@"Paused", "Torrent -> status string");
                 break;
-
+            
+            case TR_STATUS_DOWNLOAD_WAIT:
+                string = [NSLocalizedString(@"Waiting to download", "Torrent -> status string") stringByAppendingEllipsis];
+                break;
+                
+            case TR_STATUS_SEED_WAIT:
+                string = [NSLocalizedString(@"Waiting to seed", "Torrent -> status string") stringByAppendingEllipsis];
+                break;
+            
             case TR_STATUS_CHECK_WAIT:
                 string = [NSLocalizedString(@"Waiting to check existing data", "Torrent -> status string") stringByAppendingEllipsis];
                 break;
@@ -1097,7 +1097,7 @@ int trashDataFile(const char * filename)
                                                     [self peersGettingFromUs]];
         }
         
-        if (fStalled)
+        if ([self isStalled])
             string = [NSLocalizedString(@"Stalled", "Torrent -> status string") stringByAppendingFormat: @", %@", string];
     }
     
@@ -1123,17 +1123,19 @@ int trashDataFile(const char * filename)
     switch (fStat->activity)
     {
         case TR_STATUS_STOPPED:
-            if (fWaitToStart)
-            {
-                string = ![self allDownloaded]
-                        ? [NSLocalizedString(@"Waiting to download", "Torrent -> status string") stringByAppendingEllipsis]
-                        : [NSLocalizedString(@"Waiting to seed", "Torrent -> status string") stringByAppendingEllipsis];
-            }
-            else if ([self isFinishedSeeding])
+            if ([self isFinishedSeeding])
                 string = NSLocalizedString(@"Seeding complete", "Torrent -> status string");
             else
                 string = NSLocalizedString(@"Paused", "Torrent -> status string");
             break;
+        
+        case TR_STATUS_DOWNLOAD_WAIT:
+            string = [NSLocalizedString(@"Waiting to download", "Torrent -> status string") stringByAppendingEllipsis];
+            break;
+            
+        case TR_STATUS_SEED_WAIT:
+            string = [NSLocalizedString(@"Waiting to seed", "Torrent -> status string") stringByAppendingEllipsis];
+            break;
 
         case TR_STATUS_CHECK_WAIT:
             string = [NSLocalizedString(@"Waiting to check existing data", "Torrent -> status string") stringByAppendingEllipsis];
@@ -1173,13 +1175,15 @@ int trashDataFile(const char * filename)
     switch (fStat->activity)
     {
         case TR_STATUS_STOPPED:
+        case TR_STATUS_DOWNLOAD_WAIT:
+        case TR_STATUS_SEED_WAIT:
         {
             NSString * string = NSLocalizedString(@"Paused", "Torrent -> status string");
             
             NSString * extra = nil;
-            if (fWaitToStart)
+            if ([self waitingToStart])
             {
-                extra = ![self allDownloaded]
+                extra = fStat->activity == TR_STATUS_DOWNLOAD_WAIT 
                         ? NSLocalizedString(@"Waiting to download", "Torrent -> status string")
                         : NSLocalizedString(@"Waiting to seed", "Torrent -> status string");
             }
@@ -1203,10 +1207,6 @@ int trashDataFile(const char * filename)
 
         case TR_STATUS_SEED:
             return NSLocalizedString(@"Seeding", "Torrent -> status string");
-        
-        default:
-            NSAssert1(NO, @"Unknown activity %d for state string", fStat->activity);
-            return nil;
     }
 }
 
@@ -1250,11 +1250,6 @@ int trashDataFile(const char * filename)
     return fStat->peersFrom[TR_PEER_FROM_LTEP];
 }
 
-- (NSInteger) totalPeersKnown
-{
-    return fStat->peersKnown;
-}
-
 - (NSInteger) peersSendingToUs
 {
     return fStat->peersSendingToUs;
@@ -1534,7 +1529,7 @@ int trashDataFile(const char * filename)
 
 - (BOOL) isStalled
 {
-    return fStalled;
+    return fStat->isStalled;
 }
 
 - (void) updateTimeMachineExclude
@@ -1565,7 +1560,7 @@ int trashDataFile(const char * filename)
 {
     if (![self isActive]) //paused
     {
-        if (fWaitToStart)
+        if ([self waitingToStart])
             return 1;
         else
             return 0;
@@ -1611,7 +1606,6 @@ int trashDataFile(const char * filename)
 
 - (id) initWithPath: (NSString *) path hash: (NSString *) hashString torrentStruct: (tr_torrent *) torrentStruct
         magnetAddress: (NSString *) magnetAddress lib: (tr_session *) lib
-        waitToStart: (NSNumber *) waitToStart
         groupValue: (NSNumber *) groupValue
         timeMachineExcludeLocation: (NSString *) timeMachineExclude
         downloadFolder: (NSString *) downloadFolder
@@ -1660,6 +1654,7 @@ int trashDataFile(const char * filename)
 
     fInfo = tr_torrentInfo(fHandle);
     
+    tr_torrentSetQueueStartCallback(fHandle, startQueueCallback, self);
     tr_torrentSetCompletenessCallback(fHandle, completenessChangeCallback, self);
     tr_torrentSetRatioLimitHitCallback(fHandle, ratioLimitHitCallback, self);
     tr_torrentSetIdleLimitHitCallback(fHandle, idleLimitHitCallback, self);
@@ -1667,7 +1662,6 @@ int trashDataFile(const char * filename)
     
     fHashString = [[NSString alloc] initWithUTF8String: fInfo->hashString];
     
-    fWaitToStart = waitToStart && [waitToStart boolValue];
     fResumeOnWake = NO;
     
     //don't do after this point - it messes with auto-group functionality
@@ -1716,7 +1710,7 @@ int trashDataFile(const char * filename)
                 
                 if (!node)
                 {
-                    node = [[FileListNode alloc] initWithFolderName: name path: path];
+                    node = [[FileListNode alloc] initWithFolderName: name path: path torrent: self];
                     [fileList addObject: node];
                     [node release];
                 }
@@ -1729,7 +1723,7 @@ int trashDataFile(const char * filename)
             }
             else
             {
-                FileListNode * node = [[FileListNode alloc] initWithFileName: name path: path size: file->length index: i];
+                FileListNode * node = [[FileListNode alloc] initWithFileName: name path: path size: file->length index: i torrent: self];
                 [fileList addObject: node];
                 [flatFileList addObject: node];
                 [node release];
@@ -1744,7 +1738,7 @@ int trashDataFile(const char * filename)
     }
     else
     {
-        FileListNode * node = [[FileListNode alloc] initWithFileName: [self name] path: @"" size: [self size] index: 0];
+        FileListNode * node = [[FileListNode alloc] initWithFileName: [self name] path: @"" size: [self size] index: 0 torrent: self];
         fFileList = [[NSArray arrayWithObject: node] retain];
         fFlatFileList = [fFileList retain];
         [node release];
@@ -1770,10 +1764,10 @@ int trashDataFile(const char * filename)
     {
         NSString * path = [[parent path] stringByAppendingPathComponent: [parent name]];
         if (isFolder)
-            node = [[FileListNode alloc] initWithFolderName: name path: path];
+            node = [[FileListNode alloc] initWithFolderName: name path: path torrent: self];
         else
         {
-            node = [[FileListNode alloc] initWithFileName: name path: path size: size index: index];
+            node = [[FileListNode alloc] initWithFileName: name path: path size: size index: index torrent: self];
             [flatFileList addObject: node];
         }
         
@@ -1801,6 +1795,11 @@ int trashDataFile(const char * filename)
             [self sortFileList: [node children]];
 }
 
+- (void) startQueue
+{
+    [[NSNotificationCenter defaultCenter] postNotificationName: @"UpdateQueue" object: self];
+}
+
 //status has been retained
 - (void) completenessChange: (NSDictionary *) statusInfo
 {
