@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: TorrentTableView.m 11617 2011-01-01 20:42:14Z livings124 $
+ * $Id: TorrentTableView.m 12769 2011-08-27 21:32:45Z livings124 $
  *
  * Copyright (c) 2005-2011 Transmission authors and contributors
  *
@@ -25,6 +25,7 @@
 #import "TorrentTableView.h"
 #import "Controller.h"
 #import "FileListNode.h"
+#import "InfoOptionsViewController.h"
 #import "NSApplicationAdditions.h"
 #import "NSStringAdditions.h"
 #import "Torrent.h"
@@ -71,8 +72,11 @@
         fMouseControlRow = -1;
         fMouseRevealRow = -1;
         fMouseActionRow = -1;
+        #warning we can get rid of the on 10.7
         fActionPushedRow = -1;
         
+        fActionPopoverShown = NO;
+        
         [self setDelegate: self];
         
         fPiecesBarPercent = [fDefaults boolForKey: @"PiecesBar"] ? 1.0 : 0.0;
@@ -83,6 +87,8 @@
 
 - (void) dealloc
 {
+    [[NSNotificationCenter defaultCenter] removeObserver: self];
+    
     [fCollapsedGroups release];
     
     [fPiecesBarAnimation release];
@@ -99,6 +105,9 @@
 {
     //set group columns to show ratio, needs to be in awakeFromNib to size columns correctly
     [self setGroupStatusColumns];
+    
+    [[NSNotificationCenter defaultCenter] addObserver: self selector: @selector(reloadData)
+                                                 name: @"ReloadTorrentTable" object: nil];
 }
 
 - (BOOL) isGroupCollapsed: (NSInteger) value
@@ -151,14 +160,17 @@
 {
     if ([item isKindOfClass: [Torrent class]])
     {
-        [cell setRepresentedObject: item];
-        
-        const NSInteger row = [self rowForItem: item];
-        [cell setHover: row == fMouseRow];
-        [cell setControlHover: row == fMouseControlRow];
-        [cell setRevealHover: row == fMouseRevealRow];
-        [cell setActionHover: row == fMouseActionRow];
-        [cell setActionPushed: row == fActionPushedRow];
+        if (!tableColumn)
+        {
+            [cell setRepresentedObject: item];
+            
+            const NSInteger row = [self rowForItem: item];
+            [cell setHover: row == fMouseRow];
+            [cell setControlHover: row == fMouseControlRow];
+            [cell setRevealHover: row == fMouseRevealRow];
+            [cell setActionHover: row == fMouseActionRow];
+            [cell setActionPushed: row == fActionPushedRow];
+        }
     }
     else
     {
@@ -400,13 +412,20 @@
     //avoid weird behavior when showing menu by doing this after mouse down
     if (row != -1 && fMouseActionRow == row)
     {
-        fActionPushedRow = row;
-        [self setNeedsDisplayInRect: [self rectOfRow: row]]; //ensure button is pushed down
+        if (![NSApp isOnLionOrBetter])
+        {
+            fActionPushedRow = row;
+            [self setNeedsDisplayInRect: [self rectOfRow: row]]; //ensure button is pushed down
+        }
         
-        [self displayTorrentMenuForEvent: event];
+        #warning maybe make appear on mouse down
+        [self displayTorrentActionPopoverForEvent: event];
         
-        fActionPushedRow = -1;
-        [self setNeedsDisplayInRect: [self rectOfRow: row]];
+        if (![NSApp isOnLionOrBetter])
+        {
+            fActionPushedRow = -1;
+            [self setNeedsDisplayInRect: [self rectOfRow: row]];
+        }
     }
     else if (!pushed && [event clickCount] == 2) //double click
     {
@@ -522,6 +541,8 @@
         [fController focusFilterField];
     else if (firstChar == ' ')
         [fController toggleQuickLook: nil];
+    else if ([event keyCode] == 53) //esc key
+        [self deselectAll: nil];
     else
         [super keyDown: event];
 }
@@ -564,40 +585,75 @@
     }
 }
 
-- (void) displayTorrentMenuForEvent: (NSEvent *) event
+- (void) displayTorrentActionPopoverForEvent: (NSEvent *) event
 {
     const NSInteger row = [self rowAtPoint: [self convertPoint: [event locationInWindow] fromView: nil]];
     if (row < 0)
         return;
     
-    //update file action menu
-    fMenuTorrent = [[self itemAtRow: row] retain];
+    const NSRect rect = [fTorrentCell iconRectForBounds: [self rectOfRow: row]];
     
-    //update global limit check
-    [fGlobalLimitItem setState: [fMenuTorrent usesGlobalSpeedLimit] ? NSOnState : NSOffState];
-    
-    //place menu below button
-    NSRect rect = [fTorrentCell iconRectForBounds: [self rectOfRow: row]];
-    NSPoint location = rect.origin;
-    location.y += rect.size.height + 5.0;
-    
-    if ([NSApp isOnSnowLeopardOrBetter])
+    if ([NSApp isOnLionOrBetter])
     {
-        location = [self convertPoint: location toView: self];
-        [fActionMenu popUpMenuPositioningItem: nil atLocation: location inView: self];
+        if (fActionPopoverShown)
+            return;
+        
+        Torrent * torrent = [self itemAtRow: row];
+        
+        NSPopover * popover = [[NSPopoverLion alloc] init];
+        [popover setBehavior: NSPopoverBehaviorTransient];
+        InfoOptionsViewController * infoViewController = [[InfoOptionsViewController alloc] init];
+        [popover setContentViewController: infoViewController];
+        [popover setDelegate: self];
+        
+        [popover showRelativeToRect: rect ofView: self preferredEdge: NSMaxYEdge];
+        [infoViewController setInfoForTorrents: [NSArray arrayWithObject: torrent]];
+        [infoViewController updateInfo];
+        
+        [infoViewController release];
+        [popover release];
     }
     else
     {
-        location = [self convertPoint: location toView: nil];
-        NSEvent * newEvent = [NSEvent mouseEventWithType: [event type] location: location
-            modifierFlags: [event modifierFlags] timestamp: [event timestamp] windowNumber: [event windowNumber]
-            context: [event context] eventNumber: [event eventNumber] clickCount: [event clickCount] pressure: [event pressure]];
+        //update file action menu
+        fMenuTorrent = [[self itemAtRow: row] retain];
         
-        [NSMenu popUpContextMenu: fActionMenu withEvent: newEvent forView: self];
+        //update global limit check
+        [fGlobalLimitItem setState: [fMenuTorrent usesGlobalSpeedLimit] ? NSOnState : NSOffState];
+        
+        //place menu below button
+        NSPoint location = rect.origin;
+        location.y += NSHeight(rect) + 5.0;
+        
+        if ([NSApp isOnSnowLeopardOrBetter])
+        {
+            location = [self convertPoint: location toView: self];
+            [fActionMenu popUpMenuPositioningItem: nil atLocation: location inView: self];
+        }
+        else
+        {
+            location = [self convertPoint: location toView: nil];
+            NSEvent * newEvent = [NSEvent mouseEventWithType: [event type] location: location
+                modifierFlags: [event modifierFlags] timestamp: [event timestamp] windowNumber: [event windowNumber]
+                context: [event context] eventNumber: [event eventNumber] clickCount: [event clickCount] pressure: [event pressure]];
+            
+            [NSMenu popUpContextMenu: fActionMenu withEvent: newEvent forView: self];
+        }
+        
+        [fMenuTorrent release];
+        fMenuTorrent = nil;
     }
-    
-    [fMenuTorrent release];
-    fMenuTorrent = nil;
+}
+
+//don't show multiple popovers when clicking the gear button repeatedly
+- (void) popoverWillShow: (NSNotification *) notification
+{
+    fActionPopoverShown = YES;
+}
+
+- (void) popoverWillClose: (NSNotification *) notification
+{
+    fActionPopoverShown = NO;
 }
 
 - (void) menuNeedsUpdate: (NSMenu *) menu
@@ -683,80 +739,6 @@
     }
 }
 
-//alternating rows - first row after group row is white
-- (void) highlightSelectionInClipRect: (NSRect) clipRect
-{
-    if (![fDefaults boolForKey: @"SmallView"])
-    {
-        NSRect visibleRect = clipRect;
-        NSRange rows = [self rowsInRect: visibleRect];
-        BOOL start = YES;
-        
-        const CGFloat totalRowHeight = [self rowHeight] + [self intercellSpacing].height;
-        
-        NSRect gridRects[(NSInteger)(ceil(visibleRect.size.height / totalRowHeight / 2.0)) + 1]; //add one if partial rows at top and bottom
-        NSInteger rectNum = 0;
-        
-        if (rows.length > 0)
-        {
-            //determine what the first row color should be
-            if ([[self itemAtRow: rows.location] isKindOfClass: [Torrent class]])
-            {
-                for (NSInteger i = rows.location-1; i>=0; i--)
-                {
-                    if (![[self itemAtRow: i] isKindOfClass: [Torrent class]])
-                        break;
-                    start = !start;
-                }
-            }
-            else
-            {
-                rows.location++;
-                rows.length--;
-            }
-            
-            NSInteger i;
-            for (i = rows.location; i < NSMaxRange(rows); i++)
-            {
-                if (![[self itemAtRow: i] isKindOfClass: [Torrent class]])
-                {
-                    start = YES;
-                    continue;
-                }
-                
-                if (!start && ![self isRowSelected: i])
-                    gridRects[rectNum++] = [self rectOfRow: i];
-                
-                start = !start;
-            }
-            
-            const CGFloat newY = NSMaxY([self rectOfRow: i-1]);
-            visibleRect.size.height -= newY - visibleRect.origin.y;
-            visibleRect.origin.y = newY;
-        }
-        
-        const NSInteger numberBlankRows = ceil(visibleRect.size.height / totalRowHeight);
-        
-        //remaining visible rows continue alternating
-        visibleRect.size.height = totalRowHeight;
-        if (start)
-            visibleRect.origin.y += totalRowHeight;
-        
-        for (NSInteger i = start ? 1 : 0; i < numberBlankRows; i += 2)
-        {
-            gridRects[rectNum++] = visibleRect;
-            visibleRect.origin.y += 2.0 * totalRowHeight;
-        }
-        
-        NSAssert([[NSColor controlAlternatingRowBackgroundColors] count] >= 2, @"There should be 2 alternating row colors");
-        
-        [[[NSColor controlAlternatingRowBackgroundColors] objectAtIndex: 1] set];
-        NSRectFillList(gridRects, rectNum);
-    }
-    
-    [super highlightSelectionInClipRect: clipRect];
-}
-
 - (void) setQuickLimitMode: (id) sender
 {
     const BOOL limit = [sender tag] == ACTION_MENU_LIMIT_TAG;
@@ -835,14 +817,6 @@
     [[NSNotificationCenter defaultCenter] postNotificationName: @"UpdateUI" object: nil];
 }
 
-- (void) checkFile: (id) sender
-{
-    NSIndexSet * indexSet = [(FileListNode *)[sender representedObject] indexes];
-    [fMenuTorrent setFileCheckState: [sender state] != NSOnState ? NSOnState : NSOffState forIndexes: indexSet];
-    
-    [[NSNotificationCenter defaultCenter] postNotificationName: @"UpdateStats" object: nil];
-}
-
 - (void) togglePiecesBar
 {
     //stop previous animation
