@@ -7,7 +7,7 @@
  *
  * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
  *
- * $Id: about.cc 11095 2010-08-02 16:29:23Z charles $
+ * $Id: about.cc 12697 2011-08-20 05:19:27Z jordan $
  */
 
 #include <QDialogButtonBox>
@@ -37,21 +37,21 @@ AboutDialog :: AboutDialog( QWidget * parent ):
     QVBoxLayout * v = new QVBoxLayout( this );
 
     l = new QLabel;
-    l->setPixmap( QPixmap( ":/icons/transmission-48.png" ) );
+    l->setPixmap( QPixmap( QString::fromAscii( ":/icons/transmission-48.png" ) ) );
     l->setAlignment( Qt::AlignCenter );
     v->addWidget( l );
 
     QFont f( font( ) );
     f.setWeight( QFont::Bold );
     f.setPointSize( int( f.pointSize( ) * 1.2 ) );
-    l = new QLabel( "<big>Transmission " LONG_VERSION_STRING "</big>" );
+    l = new QLabel( tr( "<big>Transmission %1</big>" ).arg( QString::fromAscii( LONG_VERSION_STRING ) ) );
     l->setAlignment( Qt::AlignCenter );
     l->setFont( f );
     l->setMargin( 8 );
     v->addWidget( l );
 
     l = new QLabel( tr( "A fast and easy BitTorrent client" ) );
-    l->setStyleSheet( "text-align: center" );
+    l->setStyleSheet( QString::fromAscii( "text-align: center" ) );
     l->setAlignment( Qt::AlignCenter );
     v->addWidget( l );
 
@@ -59,7 +59,7 @@ AboutDialog :: AboutDialog( QWidget * parent ):
     l->setAlignment( Qt::AlignCenter );
     v->addWidget( l );
 
-    l = new QLabel( "<a href=\"http://www.transmissionbt.com/\">http://www.transmissionbt.com/</a>" );
+    l = new QLabel( QString::fromAscii( "<a href=\"http://www.transmissionbt.com/\">http://www.transmissionbt.com/</a>" ) );
     l->setOpenExternalLinks( true );
     l->setAlignment( Qt::AlignCenter );
     v->addWidget( l );
@@ -87,8 +87,8 @@ void
 AboutDialog :: showCredits( )
 {
     QMessageBox::about( this, tr( "Credits" ),
-        "Charles Kerr (Backend; Daemon; GTK+; Qt)\n"
-        "Michell Livingston (Backend; OS X)\n"
-        "Kevin Glowacz (Web client)" );
+        QString::fromAscii( "Jordan Lee (Backend; Daemon; GTK+; Qt)\n"
+                            "Michell Livingston (Backend; OS X)\n"
+                            "Kevin Glowacz (Web client)" ) );
 }
 
