@@ -7,7 +7,7 @@
  *
  * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
  *
- * $Id: make-dialog.cc 11098 2010-08-02 20:55:11Z charles $
+ * $Id: make-dialog.cc 12697 2011-08-20 05:19:27Z jordan $
  */
 
 #include <cassert>
@@ -56,10 +56,12 @@ MakeDialog :: onNewButtonBoxClicked( QAbstractButton * button )
 {
     switch( myNewButtonBox->standardButton( button ) )
     {
-        case QDialogButtonBox::Open:
-std::cerr << "calling mySession.addTorrent( " << qPrintable(myTarget) << ", " << qPrintable(QFileInfo(myBuilder->top).dir().path()) << ')' << std::endl;
-            mySession.addNewlyCreatedTorrent( myTarget, QFileInfo(myBuilder->top).dir().path() );
+        case QDialogButtonBox::Open: {
+            const QString top = QString::fromLocal8Bit( myBuilder->top );
+std::cerr << "calling mySession.addTorrent( " << qPrintable(myTarget) << ", " << qPrintable(QFileInfo(top).dir().path()) << ')' << std::endl;
+            mySession.addNewlyCreatedTorrent( myTarget, QFileInfo(top).dir().path() );
             break;
+        }
         case QDialogButtonBox::Abort:
             myBuilder->abortFlag = true;
             break;
@@ -79,20 +81,21 @@ MakeDialog :: onProgress( )
     myNewProgress->setValue( (int) ((100.0 * b->pieceIndex) / denom ) );
 
     // progress label
-    const QString base( QFileInfo(b->top).baseName() );
+    const QString top = QString::fromLocal8Bit( myBuilder->top );
+    const QString base( QFileInfo(top).baseName() );
     QString str;
     if( !b->isDone )
         str = tr( "Creating \"%1\"" ).arg( base );
     else if( b->result == TR_MAKEMETA_OK )
         str = tr( "Created \"%1\"!" ).arg( base );
     else if( b->result == TR_MAKEMETA_URL )
-        str = tr( "Error: invalid announce URL \"%1\"" ).arg( b->errfile );
+        str = tr( "Error: invalid announce URL \"%1\"" ).arg( QString::fromLocal8Bit( b->errfile ) );
     else if( b->result == TR_MAKEMETA_CANCELLED )
         str = tr( "Cancelled" );
     else if( b->result == TR_MAKEMETA_IO_READ )
-        str = tr( "Error reading \"%1\": %2" ).arg( b->errfile ).arg( strerror(b->my_errno) );
+        str = tr( "Error reading \"%1\": %2" ).arg( QString::fromLocal8Bit(b->errfile) ).arg( QString::fromLocal8Bit(strerror(b->my_errno)) );
     else if( b->result == TR_MAKEMETA_IO_WRITE )
-        str = tr( "Error writing \"%1\": %2" ).arg( b->errfile ).arg( strerror(b->my_errno) );
+        str = tr( "Error writing \"%1\": %2" ).arg( QString::fromLocal8Bit(b->errfile) ).arg( QString::fromLocal8Bit(strerror(b->my_errno)) );
     myNewLabel->setText( str );
 
     // buttons
@@ -111,7 +114,7 @@ MakeDialog :: makeTorrent( )
     // get the tiers
     int tier = 0;
     QList<tr_tracker_info> trackers;
-    foreach( QString line, myTrackerEdit->toPlainText().split("\n") ) {
+    foreach( QString line, myTrackerEdit->toPlainText().split(QChar::fromAscii('\n')) ) {
         line = line.trimmed( );
         if( line.isEmpty( ) )
             ++tier;
@@ -144,13 +147,15 @@ MakeDialog :: makeTorrent( )
     myTimer.start( 100 );
 
     // the file to create
-    myTarget = QDir( myDestination ).filePath( QFileInfo(myBuilder->top).baseName() + ".torrent" );
+    const QString path = QString::fromLocal8Bit( myBuilder->top );
+    const QString torrentName = QFileInfo(path).baseName() + QString::fromAscii(".torrent");
+    myTarget = QDir( myDestination ).filePath( torrentName );
     std::cerr << qPrintable(myTarget) << std::endl;
 
     // comment
     QString comment;
     if( myCommentCheck->isChecked() )
-        comment = myCommentEdit->text().toUtf8().constData();
+        comment = myCommentEdit->text();
 
     // start making the torrent
     tr_makeMetaInfo( myBuilder,
@@ -192,7 +197,7 @@ void
 MakeDialog :: onFolderClicked( )
 {
     QFileDialog * d = new QFileDialog( this, tr( "Select Folder" ) );
-    d->setFileMode( QFileDialog::Directory );
+    d->setFileMode( QFileDialog::DirectoryOnly );
     connect( d, SIGNAL(filesSelected(const QStringList&)),
              this, SLOT(onFolderSelected(const QStringList&)) );
     d->show( );
@@ -215,7 +220,7 @@ void
 MakeDialog :: onDestinationClicked( )
 {
     QFileDialog * d = new QFileDialog( this, tr( "Select Folder" ) );
-    d->setFileMode( QFileDialog::Directory );
+    d->setFileMode( QFileDialog::DirectoryOnly );
     connect( d, SIGNAL(filesSelected(const QStringList&)),
              this, SLOT(onDestinationSelected(const QStringList&)) );
     d->show( );
@@ -333,7 +338,7 @@ MakeDialog :: MakeDialog( Session & session, QWidget * parent ):
         const QPixmap folderPixmap = folderIcon.pixmap( iconSize );
         QPushButton * b = new QPushButton;
         b->setIcon( folderPixmap );
-        b->setStyleSheet( "text-align: left; padding-left: 5; padding-right: 5" );
+        b->setStyleSheet( QString::fromAscii( "text-align: left; padding-left: 5; padding-right: 5" ) );
         myDestination = QDir::homePath();
         b->setText( myDestination );
         connect( b, SIGNAL(clicked(bool)),
@@ -347,7 +352,7 @@ MakeDialog :: MakeDialog( Session & session, QWidget * parent ):
         myFolderButton = new QPushButton;
         myFolderButton->setIcon( folderPixmap );
         myFolderButton->setText( tr( "(None)" ) );
-        myFolderButton->setStyleSheet( "text-align: left; padding-left: 5; padding-right: 5" );
+        myFolderButton->setStyleSheet( QString::fromAscii( "text-align: left; padding-left: 5; padding-right: 5" ) );
         connect( myFolderButton, SIGNAL(clicked(bool)),
                  this, SLOT(onFolderClicked(void)) );
         hig->addRow( myFolderRadio, myFolderButton );
@@ -362,7 +367,7 @@ MakeDialog :: MakeDialog( Session & session, QWidget * parent ):
         myFileButton = new QPushButton;
         myFileButton->setText( tr( "(None)" ) );
         myFileButton->setIcon( filePixmap );
-        myFileButton->setStyleSheet( "text-align: left; padding-left: 5; padding-right: 5" );
+        myFileButton->setStyleSheet( QString::fromAscii( "text-align: left; padding-left: 5; padding-right: 5" ) );
         connect( myFileButton, SIGNAL(clicked(bool)),
                  this, SLOT(onFileClicked(void)) );
         hig->addRow( myFileRadio, myFileButton );
@@ -375,7 +380,7 @@ MakeDialog :: MakeDialog( Session & session, QWidget * parent ):
     hig->addSectionTitle( tr( "Properties" ) );
 
         hig->addWideControl( myTrackerEdit = new ShortPlainTextEdit );
-        const int height = fontMetrics().size( 0, "\n\n\n\n" ).height( );
+        const int height = fontMetrics().size( 0, QString::fromAscii("\n\n\n\n") ).height( );
         myTrackerEdit->setMinimumHeight( height );
         hig->addTallRow( tr( "&Trackers:" ), myTrackerEdit );
         QLabel * l = new QLabel( tr( "To add a backup URL, add it on the line after the primary URL.\nTo add another primary URL, add it after a blank line." ) );
